#define OnlineVersion

/*********************************************************************/
/*      APPLICATION PROGRAM WRAPPER FOR IMS-DC TRANSACTIONS          */
/*                                                                   */
/*  Program Table of Contents = Inhaltsverzeichnis             Line  */
/* ----------------------------------------------------------------- */
/*  1) Header Files                                            0037  */
/*  2) Program Constants  / Konstanten                         0062  */
/*  3) IO-Buffers (Input/Output Message Queues)                0081  */
/*  4) Server User Interface (Hauptschnittstelle zu Wrapper)   0095  */
/*  5) IMS Parameter List                                      0109  */
/*  6) IMS PCB Data Stucture                                   0119  */
/*  7) DVG2 TRANIN Interface                                   0133  */
/*  8) DVG2 TRANOUT Interface                                  0163  */
/*  9) IMSP39 Field Conversion Table                           0226  */
/* 10) Function Prototypes                                     0283  */
/* 11) Main Procedure CWRAP (entered from KOOP Interface)      0311  */
/* 12) Stub Procedure CSTUB (entered from TRAM103)             0763  */
/* 13) PVBTest Stub for simulating User Program in Test        0974  */
/* 14) GetField Procedure for converting Name to Code          1005  */
/* 15) GetName  Procedure for converting Code to Name                */
/* 16) Pack_To_Unsigned Procedure for packing numbers          1047  */
/* 17) Unpack_To_Char Procedure for unpacking numbers                */
/* 18) Binary_To_Char Procedure for converting Binary to Char        */
/* 19) Dump_Area Procedure for snap dump of area               1081  */
/* 20) Precompiler Options                                     1122  */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/

#define ProgramVersion " Version 14.01.0002 "

/* -------------------- standard Includes -------------------------- */

   #pragma  runopts(env(IMS),plist(IMS))
   #include  <ims.h>
   #include  <stdio.h>
   #include  <stdlib.h>
   #include  <string.h>
   #include  <amcdef.h>
   #include  <cvgipak.h>
   #include  <cvgisys.h>
   #include  <cvgipvb.h>
   #include  <cvgifbib.h>
   #include  <stdas3.h>
   #include  <stubstd.h>
   #include  <packdll.h>

/* -------------------- DVG Includes ------------------------------ */

/* #include <tranin.h>     */
/* #include <tranout.h>    */
/* #include <dvg1in.h>     */
/* #include <dvg1out.h>    */
/* #include <logkopf.h>    */
/* intended for Host Test  */

/* ------------------ Constant Definitions ------------------- */

 #define LAENGE_TRANIN_KOPF   40
 #define LAENGE_TRANIN_DATA   76
 #define LAENGE_TRANOUT_KOPF  40
 #define LAENGE_TRANOUT_DATA  76
 #define LAENGE_NAME           8
 #define LAENGE_PSB           20
 #define LAENGE_PCB           32
 #define LAENGE_API         1058
 #define INP_MSG_LNG        1028
 #define OUT_MSG_LNG        1028
 #define MAX_MSG_LNG        1028
 #define MAX_FELD_LNG         72
 #define MAX_FIELDS           20
 #define BLANK                ' '

/********************* Internal IO-Buffers ************************/

/*-------------------- Input Message Queue -----------------------*/

   static char XTRANIN1028;

/*-------------------- Output Message Queue ----------------------*/

   static char XTRANOUT1028;

/*-------------------- String for saving IMS Params --------------*/

   char  User_PSB20;

/* ------------------- Global Data Structures ------------------- */

 /*--------------- User Input Message from Server ----------------*/
 /* This is a String of ASCII characters transfered from the UNIX */
 /*---------------------------------------------------------------*/

/*  struct of WRAPPING User Interface     */
    struct 
       char    User_Trans8;
       char    anzFelder2;
       char    Bediener8;
       char    User_Lterm8;
       char    User_Msg_Nr4;
       char    User_Msg1028;
     USER_API;

 /*--------------- IMS Parameter Data Structure -----------------*/

    struct  
       unsigned long Anzahl;
       char Funktion4;
       unsigned long PCB_Adr;
       unsigned long IO_Area_Adr;
       char Format4;
     IMS_PSB;

 /*--------------- IMS PCB Data Structure -----------------------*/

    struct 
       char DB_Name8;
       char Seg_Stufe2;
       char Status_Code2;
       char Option4;
       char Filler4;
       char Seg_Name8;
       unsigned long Key_Lng;
       unsigned long Seg_Nr;
       char Keys8;
     IMS_PCB;

 /*--------------- DVG2 TRANIN  Message Interface ---------------*/
 /* This Structure is a C Version of the Assembler Interface     */
 /*--------------------------------------------------------------*/

    struct  
     struct 
        short ils;
        short ires;
        short segnr;
        short tls;
        char  tvk;
        char  tfnr;
        char  trans9;
        char  transx2;
        char  innr4;
        char  term4;
        char  tart;
        char  tkomp8;
        char  wun;
        char  steuer;
      Tranin_Kopf;
     char  Tranin_DataINP_MSG_LNG;
 TRANIN;

 /*--------------- DVG2 TRANOUT Message Interface ---------------*/
 /* This Structure is a C Version of the Assembler Interface     */
 /*--------------------------------------------------------------*/

    struct  
     struct 
        short ols;
        char  ores;
        char  prb;
        struct 
          char  ident_1;
          char  ident_2_6;
          char  ident_7;
         ident;
        char  frei;
        short segn;
        short slang;
        char  svk;
        char  sfnr;
        char  bldsteu2;
        char  fktkz;
        char  onnrst23;
        char  onnr4;
        char  sltermv4;
        char  sltermn4;
        char  stran2;
        char  sutyp;
        char  sregion;
        char  ssteu;
        char  frei22;
        char  svordr2;
        char  sart;
        char  skomp8;
        char  wunsch;
        char  sbed15;
        struct 
           char sinsstnr;
           char sbst2;
           char sliste;
           struct 
              char skto5;
              char filler14;
            snanwd;
         snafa;
        char sin3st2;
        char sgv2;
        char sbed25;
        char sbutag4;
        char sgvtag4;
        char sgvzeit2;
        char frei38;
      Tranout_Kopf;
  /* struct         Array of Output Fields
        short sls;
        char  svkd;
        char  sfnrd;
        char  sattr1;
        char  sattr2;
        char  datout;
      Tranout_Feld;      */
     char  Tranout_DataOUT_MSG_LNG;
 TRANOUT;

 /*--------------- Field Conversion Data from IM38ST ------------*/
 /* This Table is a copy of the IM38ST Field Name Table ---------*/
 /*--------------------------------------------------------------*/

 struct FieldEntry 
    char   *string;
    char   code;
    char   Jbetu;
 ;

 struct FieldEntry KKKE_tr 41 =
       
               "I1.FELD",'x01',' ',
               "ABWK"   ,'x6D','J',
               "AKBS"   ,'x58','N',
               "ART"    ,'x04','N',
               "AUSZ"   ,'x06','N',
               "BED1"   ,'xFC','N',
               "BED2"   ,'xFD','N',
               "BISKZ"  ,'x74','N',
               "BLZ"    ,'x09','N',
               "BST"    ,'x0A','N',
               "BTBS"   ,'x5B','N',
               "BWBS"   ,'x5C','N',
               "DISPO"  ,'x6E','J',
               "EURO"   ,'x73','J',
               "FAKT"   ,'x5D','N',
               "GEMKTO" ,'x71','N',
               "GHK"    ,'x8D','J',
               "GRAUS"  ,'x6F','J',
               "GRP"    ,'x16','N',
               "KTO"    ,'x01','N',
               "KTOALT" ,'x19','N',
               "KUNR"   ,'x1B','N',
               "KWG14"  ,'x70','N',
               "KZ"     ,'x1D','N',
               "KZAV"   ,'x62','J',
               "MEIN"   ,'x66','N',
               "MIANZ"  ,'x8A','N',
               "MKZ"    ,'x26','N',
               "SAB"    ,'x2B','N',
               "SCHUFA" ,'x69','N',
               "SDIR"   ,'x8C','N',
               "TERM"   ,'x34','N',
               "TEXT"   ,'x35','N',
               "TGINF"  ,'x72','J',
               "UMSMON" ,'x79','N',
               "UMSSOR" ,'x7A','N',
               "VORRAT" ,'x77','J',
               "VWZ"    ,'x3D','N',
               "WE"     ,'x4B','N',
               "ZUSCHL" ,'x6B','N',
               "LASTKEY",'xFF','N',
    ;

/********************* End of Global Data *************************/

/* --------------- Function Prototypes ------------------------- */
    typedef struct parms
       
       long  psb;
       long  tramname;
       long  shitname;
       
        TRAM9X_PARMS;

    typedef struct stub_parms
       
       long  stub_anz;
       long  stub_funk;
       long  stub_PCB;
       long  stub_ioa;
       long  stub_form;
       
        STUB_PARMS;

    int PVBtest (char *StubName,
                 char *PSB);

    char GetField ( char* );
    int  Pack_To_Unsigned(char *,char *,int);

    int  dump_area(char * ,char *,int );

/********************* End of Prototypes **************************/
/********************* Program CWRAP ******************************/

/* --------------- Program Entry Point -------------------------- */

#ifdef OnlineVersion
    unsigned long CWRAP_std (char* User_Param, char** PCB_Addr_List)
#else
    unsigned long CWRAP     (char* User_Param, char** PCB_Addr_List)
#endif



/*----------------- Local Program Data ----------------------------*/

       OBJID        ObjId = 0;
       ENVIRONMENT  ev = 0;
       char  Packed_510;
       char  Buffer10;

       char  ProgName8 = "TRAM103";     /* TEST   */
  /*   char  ProgName8 = "GIRO00X";        TEST   */
       char  tramname_IMSP398 = "IMSP39  ";
       char  shitname_ASM8;
       char  PSB20;
       char  Fields3;
       char  Field_ID;
       char  Out_Char;
       char  Response_Key;
       int   RetCode = 0;
       int   Input_Index = 0;
       int   Start_Index = 0;
       int   Field_Index = 0;
       int   Tranin_Index = 0;
       short Field_Nr  = 0;
       short Field_Lng = 0;
       short Stop_Loop = 0;
       int  I = 0;
       int  J = 0;
       int  K = 0;
       int  Input_Msg_Lng = 0;
       int  Output_Msg_Lng = 0;

       TRAM9X_PARMS tram9x_call;

 /*---------------- Assembler Field Names--------------------------*/

     int Name_Index = 0;
     char Asm_Name1008;

 /*---------------- Program Pointers ------------------------------*/

       char *psb         = (char*)&PSB;
       char *Prog_Ptr    = (char*)&ProgName;
       char *PSB_Ptr     = (char*)&IMS_PSB;
       char *PCB_Ptr     = (char*)&IMS_PCB;
       char *Xin_Ptr     = (char*)&XTRANIN;
       char *Xout_Ptr    = (char*)&XTRANOUT;
       char *Struct_Ptr  = (char*)&TRANIN;
       char *Tranin_Ptr  = (char*)&TRANIN.Tranin_Data;
       char *Tranout_Ptr = (char*)&TRANOUT.Tranout_Data;
       char *Param_Ptr   = (char*)&USER_API;
       char *psb_ptr     = (char*)&User_PSB;
       char *Input_Ptr   = (char*)&USER_API.User_Msg;
       char *Output_Ptr  = (char*)&USER_API.User_Msg;
       char *Data_Ptr    = (char*)&TRANIN.Tranin_Data;
       char *Kopf_Ptr    = (char*)&TRANIN.Tranin_Kopf;

 /*------------------- File Pointers ------------------------------*/

       FILE *tpin, *tpout;
       unsigned int Char_in;
       unsigned int Char_out;
       unsigned int tmpi1;
       unsigned int tmpi2;
       unsigned int tmpi3;
       unsigned int tmpi4;
       char** lolab;
       char** paci;
       char cbf21;

/********************* End of Data Declarations *******************/

 /* PROCEDURe  Division           */

 /*----------------  check DVG Data Structures --------------------*/
    puts ("n------------------------------------------------------ ");
    puts (" ----------------- Wrapper 3 Started ------------------ ");
    printf(" -----------------%s-----------------n",ProgramVersion);
    puts (" ------------------------------------------------------ ");

 /*---------Setup Input Buffer from User Input Parameters ---------*/

#ifdef OnlineVersion
    rtlGetGlobalEnvironment(&ev);    /* ONLINE */
#else
    rtlInitEnvironment(&ev);         /* BATCH  */
#endif

    /* -------------------------------------------------------*/
    /*------ Test Routine for producing the User Input Msg----*/
    /* -------------------------------------------------------*/
    strncpy (USER_API.User_Trans,    "KKKE    ",   8);
    strncpy (USER_API.anzFelder,     "05",         2);
    strncpy (USER_API.Bediener,      "A101ZQ9 ",   8);
    strncpy (USER_API.User_Lterm,    "99721267",   8);
    strncpy (USER_API.User_Msg_Nr,   "0101",       4);
    strncpy (USER_API.User_Msg,
 "KTO=161907*KUNR=27005*ART=2*BST=001*BED1=22222 22006*"
 ,strlen(
 "KTO=161907*KUNR=27005*ART=2*BST=001*BED1=22222 22006*"
 ));

    /* -------------------------------------------------------*/

    /* to fake input
    memcpy(Param_Ptr, User_Param, LAENGE_API);
    *******/
    tmpi1 = (int) PCB_Addr_List;
    printf("Ptr to list of adresses : %08xn",tmpi1);
    lolab = PCB_Addr_List;
    for (I=0;I<4;I++)
    
        paci = (char**) *(lolab);
        tmpi2 = (int) paci;
        memset(cbf,' ',21);
        memcpy(cbf,(char*)paci,20);
        cbf20=0;
        printf("Ptr to PCB%d: %08x PCB: %-40sn",I+1,tmpi2,cbf);
        lolab++;

    
    /*printf(" ev = %08x n ", ev);*/
    /*
    puts ("                                                     ");
    puts (" User PSB   =          ");
    for (J = 0; J < 20; J++)
       putchar (User_PSBJ);
    */
    puts ("                                                     ");
    printf(" User Transaction  =   ");
    for (J = 0; J < 8; J++)
       putchar (USER_API.User_TransJ);

    puts ("                                                     ");
    printf(" User Name =           ");
    for (J = 0; J < 8; J++)
       putchar (USER_API.BedienerJ);

    puts ("                                                     ");
    printf(" Number of Fields =    ");
    for (J = 0; J < 2; J++)
       putchar (USER_API.anzFelderJ);

    puts ("                                                     ");
    printf(" Terminal Name =       ");
    for (J = 0; J < 8; J++)
       putchar (USER_API.User_LtermJ);

    puts ("                                                     ");
    printf(" Terminal Msg Number = ");
    for (J = 0; J < 4; J++)
       putchar (USER_API.User_Msg_NrJ);

    puts ("                                                     ");
    printf(" User Data String = n %s",USER_API.User_Msg);
 /*   for (J = 0; J < 30; J++)
       putchar (USER_API.User_MsgJ);   */

 /*-------------------- Assign a Test PCB for GIRO ----------------*/
 /* ???? Where do we get the real PCB under IMS                    */

       strcpy (IMS_PCB.DB_Name, "GIRODB ");
       strcpy (IMS_PCB.Seg_Stufe, "01");
       strcpy (IMS_PCB.Status_Code, "  ");
       strcpy (IMS_PCB.Option, "OPT1");
       strcpy (IMS_PCB.Filler, "FILL");
       strcpy (IMS_PCB.Seg_Name, "GIROROOT");

 /*-------------------- Assign User Program & PSB -----------------*/

       memcpy(PSB_Ptr,psb_ptr,LAENGE_PSB);
       memcpy(psb,psb_ptr,LAENGE_PSB);

 /*-------------------- Setup Tranin Kopf  ------------------------*/

       TRANIN.Tranin_Kopf.ils = 40;
       TRANIN.Tranin_Kopf.ires = 0;
       TRANIN.Tranin_Kopf.segnr = 1;
       TRANIN.Tranin_Kopf.tls = 34;
       TRANIN.Tranin_Kopf.tvk = 0;
       TRANIN.Tranin_Kopf.tfnr = 0xFF;

       strncpy (TRANIN.Tranin_Kopf.trans, USER_API.User_Trans, 8);
       TRANIN.Tranin_Kopf.trans8 = 0X40;
 /*    Hexidezimal Transaktionskz from IMSP39                     */
 /*    TRANIN.Tranin_Kopf.transx = IMSP39(USER_API.User_Trans);   */
       TRANIN.Tranin_Kopf.transx0 = 0X00;
       TRANIN.Tranin_Kopf.transx1 = 0XA6;  /*  KKKE trans       */

 /*    TRANIN.Tranin_Kopf.innr = System()                         */
       strncpy (TRANIN.Tranin_Kopf.innr, USER_API.User_Msg_Nr, 4);

 /*    Pack Logical Terminal Number                               */
    /* to copy terminal id in PCB (GIRO needs it) */
    lolab = PCB_Addr_List;
    paci = (char**) *(lolab);
    memcpy(paci,USER_API.User_Lterm,8);
       RetCode=Pack_To_Unsigned(Buffer,USER_API.User_Lterm,8);
       RetCode = dump_area ("TermId",Buffer,4);
       strncpy (TRANIN.Tranin_Kopf.term,Buffer,4);

       TRANIN.Tranin_Kopf.tart = 0x10;
       strncpy (TRANIN.Tranin_Kopf.tkomp, "11111111", 8);
       TRANIN.Tranin_Kopf.wun = 0x00;
       TRANIN.Tranin_Kopf.steuer = 0x02;

 /*-------------------- Setup Tranin DVG2  ------------------------*/

       memset (Data_Ptr,BLANK,INP_MSG_LNG);
       strncpy (Fields, USER_API.anzFelder,2);
       Fields2 = '0';
   /*  Field_Nr = atoi(Fields); */
       strncpy (Fields, USER_API.anzFelder,2);
       Field_Nr = 10 * ( Fields0 - '0') + (Fields1 - '0');
   /*  printf (" Number of Fields given 1=%02x 1=%02x "
                                        ,Fields0,Fields1);*/
       printf("n Field_Nr = %0d", Field_Nr);
       Tranin_Index = 0;
       Start_Index = 0;
       Name_Index = 0;
       for (Field_Index = 0;
            Field_Index < Field_Nr && Field_Index < MAX_FIELDS;
            Field_Index++)
       
       /* puts (" Next Field will be processed ");
          printf("n Field_Index = %0d", Field_Index);
          printf("n Field_Nr    = %0d", Field_Nr);
          puts (" User Input String =          ");
          for (J = 0; J < 30; J++)
             putchar (USER_API.User_MsgJ);
          scanf ("%c", Response_Key);           Test Output  */

          for (J = 0; J < 8; J++)
             Asm_NameName_IndexJ = 0x40;

          I = 0;
          Stop_Loop = 0;
          while (   I < MAX_FELD_LNG
                 && Input_Index < INP_MSG_LNG
                 && Stop_Loop == 0)
          
             if (USER_API.User_MsgInput_Index == '=' 
                 USER_API.User_MsgInput_Index == '*' 
                 USER_API.User_MsgInput_Index == 'n')
                Stop_Loop = 1;
             else
                Asm_NameName_IndexI =
                USER_API.User_MsgInput_Index;
             Input_Index++;
             I++;
          ;
          Asm_NameName_Index7 = '0';

      /*  puts (" Field Name =          ");
          for (J = 0; J < 8; J++)
          
             Out_Char = Asm_NameName_IndexJ;
             putchar(Out_Char);
          ;   */
      /*  scanf ("%c", Response_Key);  Test Output   */

          Start_Index = Input_Index;
          I = 0;
          Stop_Loop = 0;
          while (   I < MAX_FELD_LNG
                 && Input_Index < INP_MSG_LNG
                 && Stop_Loop == 0)
          
             if (USER_API.User_MsgInput_Index == '*' 
                 USER_API.User_MsgInput_Index == 'n')
                Stop_Loop = 1;
             Input_Index++;
             I++;
          ;

          I = I - 1;
          Field_Lng = I + 4;
       /* puts ("                                               ");
          puts (" Field Loop terminated                         ");
          printf("n Field_Lng = %0d", Field_Lng);  Test Output */
          if (USER_API.User_MsgInput_Index == '*')
              Input_Index = Input_Index + 1;
          Data_Ptr = &TRANIN.Tranin_DataTranin_Index;
          memcpy(Data_Ptr,&Field_Lng,2);
          TRANIN.Tranin_Kopf.ils = TRANIN.Tranin_Kopf.ils + Field_Lng;
          Tranin_Index = Tranin_Index + 2;
          TRANIN.Tranin_DataTranin_Index = 0;
          Tranin_Index = Tranin_Index + 1;

      /*  Get Field Code from IMSP39 Routine  */
          Field_ID = GetField(Asm_NameName_Index);
          printf ("n User data Name = %s", Asm_NameName_Index);
          if (Field_ID == 0 )
          
              printf(" Field_Id = %02x, Name not found in table",0);
          
          else
          
              printf(" Field_Id = %02x", Field_ID);
          
          Name_Index = Name_Index + 1;

          TRANIN.Tranin_DataTranin_Index = Field_ID;
          Tranin_Index++;
          for (K = 0; K < I; K++)
          
            TRANIN.Tranin_DataTranin_Index =
            USER_API.User_MsgStart_Index;
            Tranin_Index++;
            Start_Index++;
          ;
       ;

       puts ("                                                     ");
       printf("n Tranin Length = %0d", TRANIN.Tranin_Kopf.ils);
       RetCode=dump_area("Tranin Kopf Data",
                         (char*)&TRANIN.Tranin_Kopf,
                         40);
       /*
       for (J = 0; J < 40; J++)
       
          Kopf_Ptr = (char*)&TRANIN.Tranin_Kopf;
          putchar (Kopf_PtrJ);
       ;
       */
       /*
       printf("n Tranin Data Fields = ");
       for (J = 0; J < 40; J++)
          putchar (TRANIN.Tranin_DataJ);
       */
       RetCode=dump_area("Tranin Data Fields",
                         TRANIN.Tranin_Data,
                         100);

    puts ("                                                        ");
    puts (" ----------------- User Data Assigned ----------------- ");

 /*--------------------  Fill Input Buffer ------------------------*/

       Input_Msg_Lng = LAENGE_TRANIN_KOPF
                     + (LAENGE_TRANIN_DATA * Field_Nr);
       if (Input_Msg_Lng > MAX_MSG_LNG)
           Input_Msg_Lng = MAX_MSG_LNG;
       Xin_Ptr = (char*)&XTRANIN;
       Struct_Ptr = (char*)&TRANIN.Tranin_Kopf;
       memcpy(Xin_Ptr,Struct_Ptr,Input_Msg_Lng);

       IMS_PSB.IO_Area_Adr = (unsigned long)Struct_Ptr;

 /*--------------------  Save Input Buffer ------------------------*/


   /*  puts ("                                                   ");
       puts (" XTRANIN is: ");
       for (I = 0; I < 80; I++)
           putchar (XTRANINI); */
       dump_area("XTRANIN",XTRANIN,100);
 /*****tpin = fopen("DD:WRAPIN", "wb,type=record");
       if (tpin == NULL)
           puts (" TPINP File not opened  !");
       else
           fwrite(XTRANIN, INP_MSG_LNG, 1, tpin);

       fclose(tpin);
       fflush(stdout);******************************/

 /*--------------------  Initialize output Buffer -----------------*/

       memset(Xout_Ptr,BLANK,OUT_MSG_LNG);

 /*----------------  call  User Program with PSB ------------------*/

       puts ("                                                   ");
       puts (" +--------------- --------------------------------+");
       puts (" ! User Program will be invoked:                  !");
       puts (" +------------------------------------------------+");
       printf (" User Program is: ");
       for ( I = 0; I < LAENGE_NAME; I++ )
           putchar (Prog_PtrI);
       /*
       puts ("                                                   ");
       puts (" IMS_PSB is: ");
       for (I = 0; I < LAENGE_PSB; I++)
           putchar (psbI);

       puts ("                                                   ");
       puts (" IMS_PCB is: ");
       for (I = 0; I < LAENGE_PCB; I++)
           putchar (PCB_PtrI);
       */
       puts ("                                                   ");
  /*   puts (" TRANIN_KOPF is: ");
       Struct_Ptr = (char*)&TRANIN.Tranin_Kopf;
       for (I = 0; I < LAENGE_TRANIN_KOPF; I++)
           putchar (Struct_PtrI); */
       dump_area("TRANIN_KOPF",
                 (char*)&TRANIN.Tranin_Kopf,
                 LAENGE_TRANIN_KOPF);
       puts ("                                                   ");
  /*   puts (" Input Buffer is: ");
       for (I = 0; I < 80; I++)
           putchar (XTRANINI);    */
       dump_area("Iput Buffer",XTRANIN,80);

       puts ("                                                   ");
       puts (" +------------------------------------------------+");
       puts ("                                                   ");
       fflush(stdout);

/*---------------------------------------------------------------*/
/*            Set TRAM100 Parameters 1 = PSB, 2 = X80, 3 = N     */
/*---------------------------------------------------------------*/

       tram9x_call.psb      = (long)PCB_Addr_List;
       tram9x_call.tramname = (long)&tramname_IMSP39;
       shitname_ASM0 = 0x80;
       shitname_ASM1 = 'N';
       shitname_ASM2 = ' ';
       shitname_ASM3 = ' ';
       tram9x_call.shitname = (long)&shitname_ASM;

 /****************************************************************/
 /*    this is the call of the assemler TRAM100 (TRAM???)        */
 /****************************************************************/
       if (cprolog(ProgName) == NULL)  
          printf("cprolog nicht erfolgreich.n");
          return;
       ;
       fflush(stdout);

 /*   ---------> CALL TRAM103 with IMS PCB Addresses             */
      RetCode = PVB_isload (ProgName,(void**)&tram9x_call);
 /****************************************************************/
 /*    this is the call fake call for test purposes              */
 /*    RetCode = PVBtest    (ProgName,(void**)PSB);    TEST      */
 /****************************************************************/
       puts (" ");
       puts (" -------------- Control Returned  ---------------- ");
       printf (" Return_Code: %0d", RetCode);

 /*--------------------  Fill Output Buffer -----------------------*/

       Field_Nr = 2;
       Output_Msg_Lng =  LAENGE_TRANOUT_KOPF
                      + (LAENGE_TRANOUT_DATA * Field_Nr);
       if (Output_Msg_Lng > MAX_MSG_LNG)
           Output_Msg_Lng = MAX_MSG_LNG;

  /*   puts ("n XTRANOUT is: ");
       for (I = 0; I < 80; I++)
           putchar (XTRANOUTI);
       putchar('n');                    */
       dump_area("XTRANOUT",XTRANOUT,80);
 /*--------------------  Pass User Output back ------------------*/

 /*    memcpy(Msg_Ptr,Xout_Ptr,Output_Msg_Lng);    TEST    */
       strcpy(USER_API.User_Msg, "Transaction was processedn");

 /*    memcpy(User_Param, Param_Ptr, LAENGE_API); */
 /*    memcpy(User_Param, Param_Ptr, 70);
       *(User_Param+70) = '0';
 */
       strcpy(User_Param+4," Transaction was processedn");
       dump_area("Output Message",User_Param,80);

/*     scanf ("%c", Response_Key);              for PC Test      */
       puts (" -------------- WrapTest Finished ---------------- ");
       fflush(stdout);
       return;


/********************* End of Main ( CWRAP **********************/

/********************************************************************/
/*     STUB FOR SATISFYING IMS TERMINAL REQUESTS                    */
/*     This routine is entered from TRAM100 when an                 */
/*     ASMTDLI call is intercepted in a0000000 (GU from terminal)   */
/*     or G0000000 (insert to terminal)                             */
/********************************************************************/

unsigned long DCSTUB_std(STUB_PARMS* Stub_Parm)


/* ---------------- Global Data for Link Test ------------------- */
    struct  
       unsigned long Anzahl;
       char Funktion4;
       unsigned long CB_Adr;
       unsigned long IO_Area_Adr;
       char Format4;
     IMS_PSB;

    typedef struct  
       char DB_Name8;
       char Seg_Stufe2;
       char Status_Code2;
       char Option4;
       char Filler4;
       char Seg_Name8;
       unsigned long Key_Lng;
       unsigned long Seg_Nr;
       char Keys8;
     IMS_PCB;

    typedef struct  
       int  ils;
       int  ires;
       int  segnr;
       int  tls;
       char tvk;
       char tfnr;
       char trans9;
       char transx2;
       char innr4;
       char term4;
       char tart;
       char tkomp8;
       char wun;
       char steuer;
     IO_AREA;


 /*----------------  main Program data ----------------------------*/
    static int TestCase_Index = 0;
    static int TestCase_Nr = 3;
    static int IO_Funk = 0;
    int RetCode = 0;
    int I;

    IMS_PCB Test_PCB3 = 
    'GIRODB1 ','01','  ','OPT ',0,'GIROSEG1',4,1,' ',
    'GIRODB2 ','02','  ','OPT ',0,'GIROSEG2',4,1,' ',
    'GIRODB3 ','03','  ','OPT ',0,'GIROSEG3',4,1,' ',
    ;

    IO_AREA TestCase3 = 
    6,1,3,40,'A','1','TRANCODE1','T1','1111','TERM','1','KOMPONTE','1','A',
    6,1,3,40,'B','2','TRANCODE2','T2','2222','TERM','2','KOMPONTE','2','B',
    6,1,3,40,'C','3','TRANCODE3','T3','3333','TERM','3','KOMPONTE','3','C'
    ;

 /*------------------- File Pointers ------------------------------*/

    FILE *tpin, *tpout;

 /*----------------  Program Pointers -----------------------------*/

    char *PCB_Ptr  = (char*)&Test_PCB1;
    char *Test_Ptr = (char*)&TestCase1;
    char *PSB_Ptr  = (char*)&IMS_PSB;
    char *In_Ptr   = (char*)&XTRANIN;
    char *Out_Ptr  = (char*)&XTRANOUT;
    char *IO_Ptr;

    unsigned int tmpi1;
    unsigned int tmpi2;
    unsigned int tmpi3;
    unsigned int tmpi4;
    int tmpi5;
    char** lolab;
    char** paci;
    char cbf21;

 /*----------------  check DVG Data Structures --------------------*/
    puts (" ------------------------------------------------------- ");
    puts (" ---------------------WrapStub Invoked ----------------- ");
    puts ("                                                         ");
    puts ("   Check current Interface Data Structures               ");
    puts ("                                                         ");
    puts("DVGTEST: TRANIN   is ok");  /* verify compilation */
    puts("DVGTEST: TRANOUT  is ok");  /* verify compilation */
    puts("DVGTEST: DVG1IN   is ok");  /* verify compilation */
    puts("DVGTEST: DVG1OUT  is ok");  /* verify compilation */
    puts ("                                                         ");

 /*----------------  copy User IMS Parameters ---------------------*/

    tmpi1 = (int) Stub_Parm;
    printf("Ptr to list of adresses : %08xn",tmpi1);
    lolab =(char**) Stub_Parm;
    for (I=0;I<4;I++)
    
        paci = (char**) *(lolab);
        tmpi2 = (int) paci;
        memset(cbf,' ',21);
        memcpy(cbf,(char*)paci,20);
        cbf20=0;
        printf(" Ptr to Par%d: %08x ",I+1,tmpi2);
        dump_area("Contents",(char*)paci,32);
        lolab++;

    
    puts ("                                                         ");
    /*printf(" ev = %08x n ", ev);*/

 /*----------------  copy User IMS PSB ----------------------------*/

 /*----------------  Process IO Request ---------------------------*/

    IO_Ptr = (char*)(Stub_Parm->stub_ioa);
    IO_Funk = 3;
    if (strncmp((char*)(Stub_Parm->stub_funk), "GU  ", 4)==0)
        IO_Funk = 0;
    if (strncmp((char*)(Stub_Parm->stub_funk), "ISRT", 4)==0)
        IO_Funk = 1;
    puts ("                                                   ");
    printf (" IO-Funktion: %s", IO_Funk==0 ? "GU  ":"ISRT");

    switch (IO_Funk)
    

 /*----------------  Restore Input Message ---------------------*/
       case 0:

 /*------------------ Read Input Message from Input Queue ----------*/
       /*  Read Work File TPIN into Input Buffer       */

       /**tpin = fopen("DD:WRAPIN", "rb, type = record");
          if (tpin == NULL)
              puts (" TPIN File not opened  !");
          else
              fread (&XTRANIN, INP_MSG_LNG, 1, tpin);

          fclose(tpin);** remove for CICS Test *********/

       /*  Copy Input Buffer into TRANIN Structure     */

          memcpy (IO_Ptr, In_Ptr, INP_MSG_LNG);
          puts ("                                                   ");
          tmpi5=(int)*((short *)IO_Ptr);
          dump_area("Input Message",IO_Ptr,tmpi5);
          break;

 /*----------------  Save  Output Message ----------------------*/
       case 1:

       /*  Fill Output Buffer from TRANOUT Structure     */

          memcpy (Out_Ptr, IO_Ptr, OUT_MSG_LNG);
          puts ("                                                   ");
          tmpi5=(int)*((short *)IO_Ptr);
          dump_area("Output Message",IO_Ptr,tmpi5);
          Out_Ptr = Out_Ptr + OUT_MSG_LNG;

 /*------------------ Store Output Message in Output Queue ---------*/
       /*  Write Output Buffer into Work File TPOUT      */

       /**tpout = fopen("DD:WRAPOUT", "wb, type = record");
          if (tpout == NULL)
              puts (" TPOUT File not opened  !");
          else
              fwrite(&XTRANOUT, OUT_MSG_LNG, 1, tpout);

          fclose(tpout);** removed for CICS Test ****************/
          break;

 /*----------------  Neither GU not ISRT  ----------------------*/
       default:

          puts ("                                                  ");
          puts (" ---- IMS Call neither Input nor Output ----");
          break;

    ;  /* End of IO Function Selection  */

 /*----------------  Return to User Program -----------------------*/
    fflush(stdout);
    puts ("                                                      ");
    puts (" ------------- Control returned to User ------------- ");
    printf (" Return_Code: %0d", RetCode);
    RetCode=dump_area("IMS I/O area",IO_Ptr, INP_MSG_LNG);
    puts ("                                                      ");
    puts (" ----------------- TestStub Finished ---------------- ");
    fflush(stdout);
    return 0;


/********************************************************************/
/*     SUBROUTINES for Data Conversion & Data Presentation          */
/*       dump_area creates a snap dump of storage area              */
/*       Packed_to_Unsigned converts decimal to unsigned packed     */
/*       Get_Field converts Field_Name to Field_Code                */
/********************************************************************/

/*--------------------- Test Stub ------------------------------*/
/*  This is a simulation of TRAM100 for the sole purpose of     */
/*  testing CWRAP without a user program                        */
/*--------------------------------------------------------------*/

int PVBtest (char *StubName, char *Stub_PSB)

    int I;
    char Out;
    puts ("                                                      ");
    puts ("------------------ User Program started ------------- ");
    puts (" Program called is: ");
    for (I = 0; I < 8; I++) 
      Out = StubNameI;
      putchar (Out);
    ;
    puts ("                                                      ");
    puts (" PSB passed is: ");
    for (I = 0; I < 20; I++) 
      Out = Stub_PSBI;
      putchar (Out);
    ;
    puts ("                                                      ");
    puts ("----------------- User Program terminated ----------- ");
    puts ("                                                      ");
    fflush(stdout);
    return 0;


/************************ End of PVBTest *****************************/

/*--------------------- Field Type Conversion ------------------*/
/*  This is a temporary solution for simulating IMSP38 in       */
/*  converting Field Names to Field Codes in DVG2 Interface     */
/*--------------------------------------------------------------*/

/*********************************************************************/
/*   Return Field number codes for transaction KKKE                  */
/* MVS EBCDIC character set changes                                  */
/* c x'43'  all;c x'??'  all;c x'63'  all;c x'FC'  all;          */
/* c x'ec'  all;                                                    */
/*********************************************************************/

char  GetField (char *User_Param)

     int  i;
     char cc = '0';
     char tmpc9;
     for (i=0;i<8;i++)
     
         tmpci=*(User_Param++);
         if (tmpci == ' ')
         
             tmpci = '0';
         

     
     tmpc8 = '0';
     for (i=0;i<sizeof(KKKE_tr)/sizeof(struct FieldEntry);i++)
     
         if (strcmp(KKKE_tri.string,tmpc) == 0)
         
             cc = KKKE_tri.code;
             i =  sizeof(KKKE_tr)/sizeof(struct FieldEntry);
         
     
     /*printf ("n Name=%s,Code=%02x",tmpc,cc);*/
     return(cc);



/******************* End of Field Conversion ************************/

/*--------------------- Packing Routine ------------------------*/
/*  This is a routine for packing a decimal number without a    */
/*  sign                                                        */
/*--------------------------------------------------------------*/

int  Pack_To_Unsigned(char * output,char * input,int length)

    unsigned char wrk;
    unsigned char wrki30;
    int rc = 0;
    memset(output,0xff,(length+1)/2);
    memset(wrki,0x00,30);
    memcpy(wrki,input,length);
    for (;length>0;length--)
    
      if (!isdigit(wrkilength-1))
        rc = 1;
      wrk = wrkilength-1 & 0x0f;
      if (length % 2 == 0)
      
        *(output+((length-1)/2)) &= 0xf0;
        *(output+((length-1)/2)) = wrk;
      
      else
      
        *(output+((length-1)/2)) &= 0x0f;
        *(output+((length-1)/2)) = wrk << 4;
      
    
    return(rc);


/******************* End of Pack Routine ****************************/

/*--------------------- Snap Dump of Data Area -----------------*/
/*  This is a routine for dumping a string from a given address */
/*  by a given length                                           */
/*--------------------------------------------------------------*/

int dump_area(char * dumpid,char *area,int lng)

    int interval=0;
    int wrk;
    int lngsv;
    char * savept ;
    char * savept1 ;
    printf("n %s:n",dumpid);
    savept = area;
    savept1 = area;
    lngsv = lng;
    lng = (lng+15)/16*16;
    while(interval<lng)
    
      if ( interval%16 == 0)
        printf(" %04x: ",interval);
      if (interval < lngsv)
      
        printf("%02x",*(area++));
      
      else
      
        printf("  ");
      
      interval++;
      if ( interval%4 == 0)
        putchar(' ');
      if ( interval%16 == 0)
      
        printf("  !");
        for (wrk=0;wrk<=15;wrk++)
        
          if (savept1 - savept <= lngsv)
          
            if (isprint(*(savept)))
              putchar(*(savept));
            else
              putchar('.');
            savept++;
           
           else
             putchar(' ');
        
        printf("!n");
      
    


/******************* End of Dump Routine *****************************/

/************************ End of Subroutines  ************************/
/*************  C Precompiler Options for Action Module **************/

#ifdef OnlineVersion
    EXEC AMSTART
       LOGAK CWRAP     INST std
       LOGAK DCSTUB    INST std ;
#endif

/************************ End of Source ******************************/
