//----------------------------------------------------------------------------
//  Project Pliwrap
//  BahrData / SES GmbH
//  Copyright  1999. All Rights Reserved.
//
//  SUBSYSTEM:    Pliwrap Application
//  FILE:         plwrmdic.cpp
//  AUTHOR:       Helmuth C. Bahr
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TPliwrapMDIClient (TMDIClient).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <owl/listbox.h>
#include <stdio.h>
#include <dir.h>

#include "plwrpapp.h"
#include "plwrplsb.h"
#include "plwrmdi1.h"
#include "plwrmdic.h"

#include "dirs.h"
#include "param.h"
#include "opendir.h"
#include "selpli.h"
#include "selectf.h"
#include "adddirs.h"
#include "selobj.h"
#include "sellab.h"
#include "editor.h"


//{{TPliwrapMDIClient Implementation}}


//
// Build a response table for all messages/commands handled
// by TPliwrapMDIClient derived from TMDIClient.
//
DEFINE_RESPONSE_TABLE1(TPliwrapMDIClient, TMDIClient)
//{{TPliwrapMDIClientRSP_TBL_BEGIN}}
  EV_COMMAND(CM_MDIFILEOPEN, CmFileOpen),
  EV_COMMAND(CM_ADDDIR, CmAddDir),
  EV_COMMAND(CM_ADDCOPY, CmAddCopy),
  EV_COMMAND(CM_EDITOR, CmEditor),
  EV_COMMAND(CM_PARA, CmPara),
  EV_COMMAND(CM_VIEW, CmView),
  EV_COMMAND(CM_PROGRAM, CmProgram),
  EV_COMMAND(CM_PROCEDURES, CmProcedure),
  EV_COMMAND(CM_TRANSACTION, CmTransaction),
  EV_COMMAND(CM_MODULE, CmModule),
//{{TPliwrapMDIClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TPliwrapMDIClient
// ~~~~~~~~~~~
// Construction/Destruction handling.
//
TPliwrapMDIClient::TPliwrapMDIClient(TModule* module)
:
  TMDIClient(module)
{
  ChildCount = 0;

  // INSERT>> Your constructor code here.

}


TPliwrapMDIClient::~TPliwrapMDIClient()
{
  Destroy();

  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TPliwrapMDIClient
// ~~~~~~~~~~~
// MDIClient site initialization.
//
void TPliwrapMDIClient::SetupWindow()
{
  // Default SetUpWindow processing.
  //
  char *sz;

  TMDIClient::SetupWindow();

  TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

	 theApp->GetModuleFileName (modulpath, MAX_PATH);

	 sz = strrchr (modulpath, 92);
	 *sz = '\0';

	 GetExtensions ();
	 GetIni ();
	 WriteIni ();
}



//*********************************************
void TPliwrapMDIClient::CmModule()
{
 //
 // Show the modal dialog.
 //
 char xfile[MAX_PATH];
 char filename[MAX_PATH];
 char pathext[MAX_PATH];
 char pathext2[MAX_PATH];
 char execbuf[256];

 int  savsel;

 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

 if (!GetExecIni()) return;
 strcpy (theApp->execini,execini);

 theApp->select=0;
 strcpy (filename,"\0");
 strcpy (theApp->title,"Wrap Module File Selection");
 // nderung von inputdir zu outputdir
 TSelectPli(theApp->MainWindow,filename, outputdir).Execute();
 if (!theApp->select) return;
 if (!strlen(filename)) return;
 // nderung von inputdir zu outputdir
 sprintf(progfile,"%s\\%s",outputdir,filename);
 GetName (progname, progfile);
 if (!WriteExecIni ()) return;
 sprintf(execbuf, "%s\\modwrap %s", modulpath, execini);
 WinExec(execbuf, SW_SHOWNORMAL);
}

//*********************************************
void TPliwrapMDIClient::CmTransaction()
{
 //
 // Show the modal dialog.
 //
 char xfile[MAX_PATH];
 char filename[MAX_PATH];
 char pathext[MAX_PATH];
 char pathext2[MAX_PATH];
 char execbuf[256];

 char *sz;

 int  savsel;

 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

 if (!GetExecIni()) return;
 strcpy (theApp->execini,execini);

 theApp->select=0;
 strcpy (filename,"\0");
 strcpy (theApp->title,"Wrap Transaction File Selection");
 // nderung von inputdir zu outputdir
 TSelectPli(theApp->MainWindow,filename, outputdir).Execute();
 if (!theApp->select) return;
 if (!strlen(filename)) return;
 // nderung von inputdir zu outputdir
 sprintf(progfile,"%s\\%s",outputdir,filename);
 GetName (progname, progfile);
 if (!WriteExecIni ()) return;
 sprintf(execbuf, "%s\\tranwrap %s", modulpath, execini);
 WinExec(execbuf, SW_SHOWNORMAL);
}


//*********************************************
void TPliwrapMDIClient::CmProgram()
{
 //
 // Show the modal dialog.
 //
 char xfile[MAX_PATH];
 char filename[MAX_PATH];
 char pathext[MAX_PATH];
 char pathext2[MAX_PATH];
 char execbuf[256];

 int  savsel;

 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

 if (!GetExecIni()) return;
 strcpy (theApp->execini,execini);

 theApp->select=0;
 strcpy (filename,"\0");
 strcpy (theApp->title,"Wrap Program File Selection");
 // nderung von inputdir zu outputdir
 TSelectPli(theApp->MainWindow,filename, outputdir).Execute();
 if (!theApp->select) return;
 if (!strlen(filename)) return;
 // nderung von inputdir zu outputdir
 sprintf(progfile,"%s\\%s",outputdir,filename);

 strcpy (xfile, progfile);
 strcpy (theApp->worklib,workdir);
 GetName (progname, progfile);
 strcpy(theApp->objname,progname);
 theApp->select=0;
 // nderung von inputdir zu outputdir
 TSelectObj(theApp->MainWindow,outputdir, xfile).Execute();
 if (!theApp->select) return;
 sprintf (objfile,"%s\\%s.obj",outputdir,theApp->objname);
 if (!WriteExecIni ()) return;
 sprintf(execbuf, "%s\\progwrap %s", modulpath, execini);
 WinExec(execbuf, SW_SHOWNORMAL);
}

//*********************************************
void TPliwrapMDIClient::CmProcedure()
{
 //
 // Show the modal dialog.
 //
 char xfile[MAX_PATH];
 char filename[MAX_PATH];
 char pathext[MAX_PATH];
 char pathext2[MAX_PATH];
 char execbuf[256];

 int  savsel;

 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

 if (!GetExecIni()) return;
 strcpy (theApp->execini,execini);

 theApp->select=0;
 strcpy (filename,"\0");
 strcpy (theApp->title,"Wrap Procedures File Selection");
 // nderung von inputdir zu outputdir
 TSelectPli(theApp->MainWindow,filename, outputdir).Execute();
 if (!theApp->select) return;
 if (!strlen(filename)) return;
 // nderung von inputdir zu outputdir
 sprintf(progfile,"%s\\%s",outputdir,filename);

 strcpy (xfile, progfile);
 strcpy (theApp->worklib,workdir);
 GetName (progname, progfile);
 strcpy(theApp->objname,progname);
 theApp->select=0;
 // nderung von inputdir zu outputdir
 TSelectLab(theApp->MainWindow,outputdir, xfile).Execute();
 if (!theApp->select) return;
 sprintf (objfile,"%s\\%s.lab",outputdir,theApp->objname);
 if (!WriteExecIni ()) return;
 sprintf(execbuf, "%s\\procwrap %s", modulpath, execini);
 WinExec(execbuf, SW_SHOWNORMAL);
}


//*********************************************
void TPliwrapMDIClient::CmView ()
{
 int disk;
 char execbuf[256];

    disk = (int) outputdir[0];
 	 if (disk >= 'a') {disk = disk - 'a';}
    else {disk = disk - 'A';};
    setdisk(disk);
    chdir(outputdir);

	 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);
	 //
	 // Standarddialog Oeffnen anzeigen und Dateinamen auswaehlen.
	 //
	 *(theApp->FileData.FileName) = 0;
	 if (TFileOpenDialog(this, theApp->FileData).Execute() != IDOK) return;
	 sprintf(execbuf,"%s %s",editor, theApp->FileData.FileName);
	 WinExec(execbuf, SW_SHOWNORMAL);
}

//*********************************************
void TPliwrapMDIClient::CmAddDir ()
{
 //
 // Show the modal dialog.
 //
 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

 theApp->select=0;
 strcpy(theApp->tcopy1,copy1);
 strcpy(theApp->tcopy2,copy2);
 strcpy(theApp->tcopy3,copy3);
 strcpy(theApp->tcopy4,copy4);
 strcpy(theApp->tcopy5,copy5);

 TAdddirs(theApp->MainWindow).Execute();
 if (!theApp->select) return;
 strcpy(copy1,theApp->tcopy1);
 strcpy(copy2,theApp->tcopy2);
 strcpy(copy3,theApp->tcopy3);
 strcpy(copy4,theApp->tcopy4);
 strcpy(copy5,theApp->tcopy5);
 WriteIni ();
}

/*void TPliwrapMDIClient::CmAddCopy ()
{
 //sprintf(pathext,"%s\\*.*\0",inputdir);
 char filename[MAX_PATH];
 char execbuf[256];

 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);
 if (!GetExecIni()) return;
 theApp->select=0;
 strcpy (filename,"\0");
 strcpy (theApp->title,"AddCopy File Selection");
 // nderung von inputdir zu outputdir
 TSelectPli(theApp->MainWindow,filename, inputdir).Execute();
 if (!theApp->select) return;
 if (!strlen(filename)) return;
 if (!WriteExecIni ()) return;
 sprintf(execbuf, "%s\\addcopy %s", modulpath, execini);
 WinExec(execbuf, SW_SHOWNORMAL);
}
*/

void TPliwrapMDIClient::CmAddCopy ()
{
 //sprintf(pathext,"%s\\*.*\0",inputdir);
 char pathext[MAX_PATH];
 char pathext2[MAX_PATH];
 char execbuf[256];

 TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

 if (!GetExecIni()) return;
 strcpy(theApp->execini,execini);
 strcpy(pathext,"pli");
 theApp->select=0;
 strcpy (theApp->title,"AddIncl File Selection");
 strcpy (pathext2,"");
 TSelectFiles(theApp->MainWindow,pathext,pathext2, inputdir).Execute();
 if (!theApp->select) return;
 if (!WriteExecIni ()) return;
 sprintf(execbuf, "%s\\addincl %s", modulpath, execini);
 WinExec(execbuf, SW_SHOWNORMAL);
}

// Parameter Dialog
void TPliwrapMDIClient::CmPara()
{
  TPliwrapApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

  theApp->select=0;

  if (strcmp(version,"PLI74")==NULL) theApp->xversion=PLIOL74;
  if (strcmp(version,"PLI85")==NULL) theApp->xversion=PLIOL85;

  if (strcmp(type,"IMS")==NULL) theApp->xtype=IMS;
  if (strcmp(type,"CICS")==NULL) theApp->xtype=CICS;
  if (strcmp(type,"UTM")==NULL) theApp->xtype=UTM;
  if (strcmp(type,"MODUL")==NULL) theApp->xtype=MODUL;

  if (strcmp(mode,"EXPANDED")==NULL) theApp->xmode=EXPANDED;



  TParam(theApp->MainWindow).Execute();
  if (!theApp->select) return;


  if (theApp->xtype==IMS) strcpy(type,"IMS");
  else
  if (theApp->xtype==CICS) strcpy(type,"CICS");
  else
  if (theApp->xtype==UTM) strcpy(type,"UTM");
  else strcpy(type,"MODUL");

  if (theApp->xversion==PLIOL74) strcpy(version,"PLI74");
  else strcpy(version,"PLI85");

  if (theApp->xmode==EXPANDED) strcpy(mode,"EXPANDED");
  else strcpy(mode,"NORMAL");

  WriteIni ();
}

//--------------------------------------------------------
// TPliwrapMDIClient
// ~~~~~~~~~~~
// Menu File Open command
//
void TPliwrapMDIClient::CmFileOpen()
{
  TPliwrapApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

  theApp->select=0;
  TDirs(theApp->MainWindow,inputdir,outputdir,workdir).Execute();
  if (!theApp->select) return;
  WriteIni ();
}

//
void TPliwrapMDIClient::CmEditor()
{
  TPliwrapApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

  theApp->select=0;
  TEditor(theApp->MainWindow,editor).Execute();
  if (!theApp->select) return;
  WriteIni ();
}

void TPliwrapMDIClient::GetIni()
{
  char ini[60];
  char winini[60];
  char edt[MAX_PATH];

  TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);

  strcpy (winini,"pliwrap.ini");

  strcpy (ini, winini);
  GetPrivateProfileString("Directories", "INPUT_DIR"     , "\0", inputdir,sizeof(inputdir), winini);
  GetPrivateProfileString("Directories", "WORK_DIR" , "\0", workdir,sizeof(workdir),ini);
  GetPrivateProfileString("Directories", "OUTPUT_DIR" , "\0", outputdir,sizeof(outputdir),ini);
  GetPrivateProfileString("Directories", "COPY_DIR_1" , "\0",copy1,sizeof(copy1),ini);
  GetPrivateProfileString("Directories", "COPY_DIR_2" , "\0",copy2,sizeof(copy2),ini);
  GetPrivateProfileString("Directories", "COPY_DIR_3" , "\0",copy3,sizeof(copy3),ini);
  GetPrivateProfileString("Directories", "COPY_DIR_4" , "\0",copy4,sizeof(copy4),ini);
  GetPrivateProfileString("Directories", "COPY_DIR_5" , "\0",copy5,sizeof(copy5),ini);

  GetPrivateProfileString("Options", "TYPE" , "UTM\0",type,sizeof(type),ini);
  GetPrivateProfileString("Options", "EDITOR" , "\0",edt,sizeof(edt),ini);
  if (!strlen(edt))
   {
    sprintf(editor,"%s\\textview.exe",modulpath);
   }
  else strcpy(editor,edt);

 }

void TPliwrapMDIClient::WriteIni ()
{
 int f1,f2,f3,f4,f5,f6,f7,f8,fa,fb,fc,fd;
 char ini[60];
 char szMsg[MAX_PATH];

 strcpy (ini,"pliwrap.ini");

 f1 = WritePrivateProfileString("Directories", "INPUT_DIR",  inputdir, ini);
 f2 = WritePrivateProfileString("Directories", "OUTPUT_DIR", outputdir, ini);
 f3 = WritePrivateProfileString("Directories", "WORK_DIR",   workdir, ini);
 f4 = WritePrivateProfileString("Directories", "COPY_DIR_1", copy1, ini);
 f5 = WritePrivateProfileString("Directories", "COPY_DIR_2", copy2, ini);
 f6 = WritePrivateProfileString("Directories", "COPY_DIR_3", copy3, ini);
 f7 = WritePrivateProfileString("Directories", "COPY_DIR_4", copy4, ini);
 f8 = WritePrivateProfileString("Directories", "COPY_DIR_5", copy5, ini);

 fc = WritePrivateProfileString("Options", "TYPE", type, ini);
 fd = WritePrivateProfileString("Options", "EDITOR", editor, ini);

  if (f1 && f2 && f3 && f4 && f5 && f6 && f7 && f8 && fc && fd)
	{
	}
  else
	{
	 sprintf(szMsg, "Can not open pliwrap.ini");
	 MessageBox(szMsg,
					"PliWrap Error Message", MB_ICONSTOP);
	}
 }

int TPliwrapMDIClient::WriteExecIni ()
 {
  char ini[MAX_PATH];

  TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);
  strcpy (ini, execini);

  WritePrivateProfileString("Directories", "INPUT_DIR",  inputdir, ini);
  WritePrivateProfileString("Directories", "WORK_DIR",   workdir, ini);
  WritePrivateProfileString("Directories", "OUTPUT_DIR", outputdir, ini);

/*  WritePrivateProfileString("Files", "PROGNAME", progname, ini);
  WritePrivateProfileString("Files", "PROGFILE", progfile, ini);
*/

  WritePrivateProfileString("Files", "PROGFILE", progfile, ini);
  WritePrivateProfileString("Files", "PROGNAME", progname, ini);
  WritePrivateProfileString("Files", "DATAFILE", objfile, ini);


  WritePrivateProfileString("Directories", "COPY_DIR_1", copy1, ini);
  WritePrivateProfileString("Directories", "COPY_DIR_2", copy2, ini);
  WritePrivateProfileString("Directories", "COPY_DIR_3", copy3, ini);
  WritePrivateProfileString("Directories", "COPY_DIR_4", copy4, ini);
  WritePrivateProfileString("Directories", "COPY_DIR_5", copy5, ini);

  WritePrivateProfileString("Options", "TYPE", type, ini);
  WritePrivateProfileString("Options", "EDITOR", editor, ini);
  return 1;
 }

int TPliwrapMDIClient::GetExecIni ()
 {
	FILE *fpout;
	int i,j;
	struct dostime_t t;
	char init[8];
	char buf[80];
	//*****************************************
		  strcpy (execini, workdir);
		  i = strlen(execini);
		  _dos_gettime(&t);
		  sprintf(init,"%2d%2d%2d%2d",t.hour,t.minute,t.second,t.hsecond);
		  j=0;
		  execini[i] = '\\';
		  i++;
		  while (j<8)
		  {
			if (isdigit(init[j]))
			{execini[i]=init[j];
			 i++;
			}
			j++;
		  }
		  execini[i]='\0';

	fpout  = fopen(execini, "w");

	if (!fpout)
	 {
	  sprintf(buf,"Can not open %s",execini);
	  MessageBox(buf,"Pliwrap Error Message",MB_ICONSTOP);
	  return 0;
	 }
   fclose(fpout);
	return 1;
	//*****************************************
 }

void TPliwrapMDIClient::GetExtensions()
{
 FILE *fp;
 char extfile[MAX_PATH];
 char record[MAX_PATH];
 int i;

  TPliwrapApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), TPliwrapApp);
  theApp->max_extensions=0;
  sprintf(extfile,"%s\\file.ext",modulpath);

  fp = fopen(extfile,"r");

  if (fp)
	{
	 while (fgets(record,MAX_PATH,fp)!=NULL)
	  {
		i=0;
		while (i<3 && record[i]!='\n')
		 {i++;}
		record[i]='\0';
		strcpy(theApp->extensions[theApp->max_extensions],record);
		theApp->max_extensions++;
	  }
	 fclose(fp);
	}
}

void TPliwrapMDIClient::GetName (char *name, char *file)
{
 char *sz;
 char xhfile[MAX_PATH];

 strcpy (xhfile, file);
 sz = strrchr (xhfile, 92);
 if (sz) sz++;

 if (sz)
  {strcpy (name, sz);}
 else
  {strcpy (name, xhfile);}

 sz = strrchr (name, 46);
 if (sz)
  {
	*sz = '\0';
  }
}

