      $SET ANS85
       IDENTIFICATION DIVISION.
       PROGRAM-ID. LIEFAUFT.
      ******************************************************************
      *  PROGRAM NAME: LIEFAUFT.                                       *
      *  SYSTEM  NAME: AUFTRAG = AUFTRAGBEARBEITUNG                    *
      *  PURPOSE:      subprogram for producing the supplier orders    *
      *  VERSION:      1.1 (07.96)                                     *
      *  LANGUAGE:     MICROSOFT (R) COBOL OPTIMIZIG COMPILER          *
      *                VERSION 3.00A ("MICROFOCUS COBOL")              *
      ******************************************************************
      *  PROGRAM INPUTS:                                               *
      *    - Datei der Lieferposten von Auftragsverarbeitung           *
      *    - Indexed sequentielle Datei der Lieferanten                *
      *    - Indexed sequentielle Datei der Lieferantenpreise          *
      *  PROGRAM OUTPUTS:                                              *
      *    - Liste der Auftrge an Lieferanten                         *
      *  PROGRAM WORK FILES:                                           *
      *    - Datei zur Sortierung der Lieferposten                     *
      *  PROGRAM FUNCTIONS:                                            *
      *    - open  Lieferpostendatei                                   *
      *    - open  Lieferposten-Workfile                               *
      *    - Do for each Lieferposten                                  *
      *      - read  Lieferposten                                      *
      *      - select Lieferant                                        *
      *      - write Lieferposten Workfile                             *
      *    - EndLoop                                                   *
      *    - close Lieferpostendatei                                   *
      *    - close Lieferposten-Workfile.                              *
      ******************************************************************
      *    - sort  Lieferposten Workfile                               *
      ******************************************************************
      *    - open  Lieferposten Workfile                               *
      *    - init  Lieferauftragsdatei                                 *
      *    - Do for each Lieferant                                     *
      *      - init  Lieferauftrag                                     *
      *      - Do for each Lieferposten                                *
      *        - write Lieferposten Workfile                           *
      *      - EndLoop                                                 *
      *      - sum   Lieferauftrag                                     *
      *    - Endloop                                                   *
      *    - close Lieferposten Workfile                               *
      *    - close Lieferauftragsdatei                                 *
      ******************************************************************
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SOURCE-COMPUTER. IBM-PC.
       OBJECT-COMPUTER. IBM-PC.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *  Datei der Lieferposten von Auftragsverarbeitung
          SELECT LIEFERPOSTENDATEI ASSIGN TO 'Liefpost.dat'
                 ORGANIZATION IS LINE SEQUENTIAL
                 FILE STATUS IS LIEFPOST-STATUS.
      *  Datei der sortierten Lieferposten
          SELECT LIEFERPOSTEN-WORKFILE ASSIGN TO 'WorkFile.dat'
                 ORGANIZATION IS LINE SEQUENTIAL
                 FILE STATUS IS WORKFILE-STATUS.
      *  Datei zur Sortierung der Lieferposten
          SELECT LIEFERPOSTEN-SORTFILE ASSIGN TO 'SortFile.dat'
                 SORT STATUS IS SORTFILE-STATUS.
      *  Liste der Auftrge an Lieferanten
          SELECT LIEFERAUFTRAGSDATEI ASSIGN TO 'Liefauft.txt'
                 ORGANIZATION IS LINE SEQUENTIAL
                 FILE STATUS IS LIEFAUFT-STATUS.
      *  Indexed sequentielle Datei der Lieferanten
          SELECT LIEFERANTEN ASSIGN TO 'LIEFIXS'
                 ORGANIZATION IS INDEXED ACCESS IS DYNAMIC
                 RECORD KEY IS LIEF-NR
                 FILE STATUS IS LIEFERANT-STATUS.
      *  Indexed sequentielle Datei der Lieferantenpreise
          SELECT PREISDATEI ASSIGN TO 'PREISIXS'
                 ORGANIZATION IS INDEXED ACCESS IS DYNAMIC
                 RECORD KEY IS ART-NR
                 FILE STATUS IS PREIS-STATUS.
      ******************************************************************
      *
       DATA DIVISION.
      *
      ******************************************************************
      *
       FILE SECTION.
      *
      ******************************************************************
      *
      *  Datei der Lieferposten von Auftragsverarbeitung
      *
       FD  LIEFERPOSTENDATEI
           RECORD CONTAINS 32 CHARACTERS
           DATA RECORD IS LIEFERPOSTEN.
       01  LIEFERPOSTEN.
           05  SATZKZ-LP                     PIC XX.
           05  ARTIKELNR                     PIC 9(6).
           05  ARTIKELNAME                   PIC X(20).
           05  BESTELLMENGE                  PIC 9999.
      *
      *  Datei der sortierten Lieferposten
      *
       FD  LIEFERPOSTEN-WORKFILE
           RECORD CONTAINS 44 CHARACTERS
           DATA RECORD IS WORKREC.
       01  WORKREC.
           05  WORK-SATZKZ                   PIC XX.
           05  WORK-LIEFERANTNR              PIC 9(6).
           05  WORK-ARTIKELNR                PIC 9(6).
           05  WORK-ARTIKELNAME              PIC X(20).
           05  WORK-BESTELLMENGE             PIC 9999.
           05  WORK-STUECKPREIS              PIC 9999V99.
      *
      *  Datei zur Sortierung der Lieferposten
      *
       SD  LIEFERPOSTEN-SORTFILE
           RECORD CONTAINS 44 CHARACTERS
           DATA RECORD IS SORTREC.
       01  SORTREC.
           05  SORT-SATZKZ                   PIC XX.
           05  SORT-LIEFERANTNR              PIC 9(6).
           05  SORT-ARTIKELNR                PIC 9(6).
           05  SORT-ARTIKELNAME              PIC X(20).
           05  SORT-BESTELLMENGE             PIC 9999.
           05  SORT-STUECKPREIS              PIC 9999V99.
      *
      *  Indexed sequentielle Datei der Lieferanten
      *
       FD  LIEFERANTEN
           RECORD CONTAINS 92 CHARACTERS
           DATA RECORD IS LIEFERANT.
       01  LIEFERANT.
           05  SATZKZ                        PIC XX.
           05  LIEF-NR                       PIC 9(6).
           05  LIEFERZEIT                    PIC 99.
           05  LIEFERWEG                     PIC 99.
               88  LUFT                      VALUE 3.
               88  BAHN                      VALUE 2.
               88  SCHIFF                    VALUE 1.
               88  LKW                       VALUE 0.
           05  LIEFERANTENDATEN.
               10  LIEFERANT-NAME            PIC X(30).
               10  LIEFERANTANSCHRIFT.
                   15  LIEFERANT-LAND        PIC X(5).
                   15  LIEFERANT-PLZ         PIC 9(5).
                   15  LIEFERANT-ORT         PIC X(20).
                   15  LIEFERANT-STRASSE     PIC X(20).
      *
      *  Indexed sequentielle Datei der Lieferantenpreise
      *
       FD  PREISDATEI
           RECORD CONTAINS 20 CHARACTERS
           DATA RECORD IS ARTIKELPREIS.
       01  ARTIKELPREIS.
           05  SATZKZ-AP                     PIC XX.
           05  ART-NR                        PIC 9(6).
           05  ART-LIEF-NR                   PIC 9(6).
           05  ART-LIEF-PREIS                PIC 9999V99.
      *
      *  Liste der Auftrge an Lieferanten
      *
       FD  LIEFERAUFTRAGSDATEI
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS AUFTRAGSSATZ.
       01  AUFTRAGSSATZ.
           05  AUFTRAGSZEILE                 PIC X(72).
      *
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *
      ******************************************************************
      *
      *  File Status Fields
       77  LIEFPOST-STATUS                   PIC XX.
           88  OK-LIEFPOST                   VALUE ZERO.
           88  EOF-LIEFPOST                  VALUE '09'.
       77  WORKFILE-STATUS                   PIC XX.
           88  OK-WORKFILE                   VALUE ZERO.
           88  EOF-WORKFILE                  VALUE '09'.
       77  SORTFILE-STATUS                   PIC XX.
           88  OK-SORTFILE                   VALUE ZERO.
           88  EOF-SORTFILE                  VALUE '09'.
       77  LIEFAUFT-STATUS                   PIC XX.
           88  OK-LIEFAUFT                   VALUE ZERO.
           88  EOF-LIEFAUFT                  VALUE '09'.
       77  LIEFERANT-STATUS                  PIC XX.
           88  OK-LIEFERANTEN                VALUE ZERO.
           88  KEY-LIEFERANTEN               VALUE '05'.
           88  EOF-LIEFERANTEN               VALUE '09'.
       77  PREIS-STATUS                      PIC XX.
           88  OK-PREISE                     VALUE ZERO.
           88  KEY-PREISE                    VALUE '05'.
           88  EOF-PREISE                    VALUE '09'.
       77  FEHLER-CODE                       PIC 99.
           88  AUFTRAG-OK                    VALUE ZERO.
           88  BAD-LIEFERPOSTEN              VALUE 1.
           88  NO-OFFER                      VALUE 2.
           88  NO-LIEFERANT                  VALUE 3.
       77  CONTINUE-KEY                      PIC XX.
      ************************************************************
       01  LISTENKONSTANTEN.
           05  ZEILEN-MAX                    PIC 99  VALUE 72.
           05  SEITEN-MAX                    PIC 99  VALUE 99.
       01  LISTENVARIABELN.
           05  TAGESDATUM.
               10  JAHR                      PIC 99.
               10  MONAT                     PIC 99.
               10  TAG                       PIC 99.
           05  LISTENDATUM.
               10  LISTENTAG                 PIC 99.
               10  FILLER                    PIC X   VALUE '/'.
               10  LISTENMONAT               PIC 99.
               10  FILLER                    PIC X   VALUE '/'.
               10  LISTENJAHR                PIC 99.
           05  SEITENNR                      PIC 999 VALUE 1.
           05  ZEILENANZAHL                  PIC 99  VALUE 73.
           05  LETZTE-LIEFERANTNR            PIC 9(6) VALUE ZEROES.
       01  LIEFERBEDINGUNGEN.
           05  LIEFERMODUS-1               PIC X(30) VALUE
              'Lieferung mit Luftfracht  '.
           05  LIEFERMODUS-2               PIC X(30) VALUE
              'Lieferung mit der Bahn    '.
           05  LIEFERMODUS-3               PIC X(30) VALUE
              'Lieferung mit Schiff      '.
           05  LIEFERMODUS-4               PIC X(30) VALUE
              'Lieferung mit LKW         '.
           05  LIEFERANTENPROGRAMM           PIC X(10) VALUE 'Liefern'.
       01  LIEFERAUFTRAGSKOPF.
           05  KOPFZEILE-1.
               10  FILLER                    PIC X     VALUE '+'.
               10  FILLER                    PIC X(70) VALUE ALL '-'.
               10  FILLER                    PIC X     VALUE '+'.
           05  KOPFZEILE-2.
               10  FILLER                    PIC X     VALUE '|'.
               10  FILLER                    PIC X(70) VALUE SPACES.
               10  FILLER                    PIC X     VALUE '|'.
           05  KOPFZEILE-3.
               10  FILLER                    PIC X     VALUE '|'.
               10  FILLER                    PIC X(2)  VALUE SPACES.
               10  FILLER                    PIC X(8) VALUE 'TITEL:  '.
               10  KOPFZEILE-TITEL           PIC X(42).
               10  FILLER                    PIC X(8) VALUE 'SEITE:  '.
               10  KOPFZEILE-SEITE           PIC 9(4).
               10  FILLER                    PIC X(6) VALUE SPACES.
               10  FILLER                    PIC X     VALUE '|'.
           05  KOPFZEILE-4.
               10  FILLER                    PIC X     VALUE '|'.
               10  FILLER                    PIC X(2)  VALUE SPACES.
               10  FILLER                    PIC X(8) VALUE 'QUELLE: '.
               10  KOPFZEILE-PROGRAM         PIC X(42).
               10  KOPFZEILE-4-4             PIC X(8) VALUE 'DATUM:  '.
               10  KOPFZEILE-DATUM           PIC X(8).
               10  FILLER                    PIC X(2) VALUE SPACES.
               10  FILLER                    PIC X     VALUE '|'.
           05  TITELZEILE-7                  PIC X(72).
           05  TITELZEILE-8.
               10  FILLER                    PIC X     VALUE '+'.
               10  FILLER                    PIC X(70) VALUE ALL '-'.
               10  FILLER                    PIC X     VALUE '+'.
           05  FUSSZEILE-1.
               10  FILLER                    PIC X     VALUE '+'.
               10  FILLER                    PIC X(70) VALUE ALL '-'.
               10  FILLER                    PIC X     VALUE '+'.
           05  FUSSZEILE-2                   PIC X(72).
           05  FUSSZEILE-3                   PIC X(72).
           05  FUSSZEILE-4                   PIC X(72).
       01  LIEFERVARIABELN.
           05  BEST-PREIS                    PIC 9999V99 VALUE 9999.99.
           05  BEST-LIEFERANT                PIC 9(6) VALUE ZERO.
           05  AUFTRAGSNR                    PIC 9(6) VALUE ZERO.
           05  LIEF-POSTEN-NR                PIC 99 VALUE ZERO.
           05  RECHENWERTE.
               10  GESAMTPREIS               PIC 99999V99 VALUE ZERO.
               10  LIEFERMENGE               PIC 999 VALUE ZERO.
               10  LIEFERBETRAG              PIC 9999999V99 VALUE ZERO.
       01  LIEFERAUFTRAGSTITEL.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(70) VALUE
           '  Rechnung der Universitt Regensburg              '.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-1.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            '        Kundenkz: '.
           05  LIEFERAUFTRAGSKOPF-LIEF-KZ    PIC 9(6).
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Lieferauftragsnr: '.
           05  LIEFERAUFTRAGSKOPF-AUFTRAGNR  PIC X(8).
           05  FILLER                        PIC X(8)  VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-2.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Lieferantenname : '.
           05  LIEFERAUFTRAGSKOPF-NAME       PIC X(30).
           05  FILLER                        PIC X(16) VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-3.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(2)  VALUE SPACES.
           05  FILLER                        PIC X(22) VALUE
                                            'Lieferantenanschrift: '.
           05  LIEFERAUFTRAGSKOPF-STRASSE    PIC X(30).
           05  FILLER                        PIC X(16) VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-4.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(24) VALUE SPACES.
           05  LIEFERAUFTRAGSKOPF-LAND       PIC X(4).
           05  LIEFERAUFTRAGSKOPF-PLZ        PIC X(6).
           05  LIEFERAUFTRAGSKOPF-ORT        PIC X(30).
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSKOPF-5.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(6)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            ' Lieferungsmodus: '.
           05  LIEFERAUFTRAGSKOPF-LIEFMODUS  PIC X(30).
           05  FILLER                        PIC X(16) VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERPOSTENTITEL.
           05  FILLER                        PIC X    VALUE '|'.
           05  FILLER                        PIC X(9) VALUE 'Bestellnr'.
           05  FILLER                        PIC X    VALUE SPACES.
           05  FILLER                        PIC X(9) VALUE 'Artikelnr'.
           05  FILLER                        PIC X    VALUE SPACES.
           05  FILLER                        PIC X(20) VALUE
              ' Artikelbezeichnung '.
           05  FILLER                        PIC X(7) VALUE SPACES.
           05  FILLER                        PIC X(5) VALUE 'Menge'.
           05  FILLER                        PIC XX   VALUE SPACES.
           05  FILLER                        PIC X(5) VALUE 'Preis'.
           05  FILLER                        PIC XXX  VALUE SPACES.
           05  FILLER                        PIC X(5) VALUE 'Summe'.
           05  FILLER                        PIC XXX  VALUE SPACES.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERPOSTENZEILE.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(2)  VALUE SPACES.
           05  LIEFERZEILE-BESTELLNR         PIC X(6).
           05  FILLER                        PIC X(2) VALUE SPACES.
           05  LIEFERZEILE-ARTIKELNR         PIC X(6).
           05  FILLER                        PIC X(2) VALUE SPACES.
           05  LIEFERZEILE-ARTIKELNAME       PIC X(20).
           05  FILLER                        PIC X(6) VALUE SPACES.
           05  LIEFERZEILE-BESTELLMENGE      PIC ZZZZZ9.
           05  FILLER                        PIC X(2) VALUE SPACES.
           05  LIEFERZEILE-STUECKPREIS       PIC ZZZ9.99.
           05  FILLER                        PIC X    VALUE SPACES.
           05  LIEFERZEILE-GESAMTPREIS       PIC ZZ,Z99.99.
           05  FILLER                        PIC X    VALUE SPACE.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSFUSS-1.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(10) VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Bestellungsanzahl:'.
           05  LIEFERAUFTRAGSFUSS-MENGE      PIC ZZZ9.
           05  FILLER                        PIC X(9)  VALUE SPACES.
           05  FILLER                        PIC X(18) VALUE
                                            'Lieferungsbetrag: '.
           05  LIEFERAUFTRAGSFUSS-BETRAG     PIC ZZZ,ZZ9.99.
           05  FILLER                        PIC X     VALUE SPACE.
           05  FILLER                        PIC X     VALUE '|'.
       01  LIEFERAUFTRAGSFUSS-2.
           05  FILLER                        PIC X     VALUE '|'.
           05  FILLER                        PIC X(10) VALUE SPACES.
           05  FILLER                        PIC X(52) VALUE
               'Die bestellte Ware ist innerhalb 30 Tage zu liefern '.
           05  FILLER                        PIC X(8)  VALUE SPACE.
           05  FILLER                        PIC X     VALUE '|'.
       01  FEHLERZEILE.
           05  FILLER                        PIC X     VALUE '|'.
           05  FEHLERMELDUNG                 PIC X(70).
           05  FILLER                        PIC X     VALUE '|'.
       01  FEHLERMELDUNGEN.
           05  FM-1                          PIC X(50) VALUE
              '   Lieferpostendaten sind fehlerhaft            '.
           05  FM-2                          PIC X(50) VALUE
              '   Artikel wird nicht mehr geliefert            '.
           05  FM-3                          PIC X(50) VALUE
              '   Lieferant ist nicht mehr bekannt             '.
      ************************************************************
      ************************************************************
       PROCEDURE DIVISION.
      *****************************************************
       LIEFERPOSTENVERARBEITUNGS SECTION.
      *    - open  Lieferpostendatei                                   *
      *    - open  Workfile                                            *
      *    - Do for each Lieferposten                                  *
      *      - read  Lieferposten                                      *
      *      - select Lieferant                                        *
      *      - write Lieferposten Workfile                             *
      *    - EndLoop                                                   *
      *    - close Lieferpostendatei                                   *
      *    - close Workfile                                            *
      *****************************************************
       OPEN-LIEFERPOSTEN.
           MOVE ZEROES TO LIEFPOST-STATUS.
           OPEN INPUT LIEFERPOSTENDATEI.
           IF LIEFPOST-STATUS NOT = ZEROES
              DISPLAY 'LIEFERPOSTENDATEI not found'
              DISPLAY 'Press Enter to terminate !'
              ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
TEST       DISPLAY 'Lieferpostendatei opened '.
           MOVE ZEROES TO PREIS-STATUS.
           OPEN INPUT PREISDATEI
           IF PREIS-STATUS NOT = ZEROES
              DISPLAY 'PREISDATEN not found'
              DISPLAY 'Press Enter to terminate !'
              ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
TEST       DISPLAY 'Preisdatei opened '.
           MOVE ZEROES TO WORKFILE-STATUS.
           OPEN OUTPUT LIEFERPOSTEN-WORKFILE.
TEST       DISPLAY 'Workfile opened '.
TEST       ACCEPT CONTINUE-KEY.
      *****************************************************
       READ-LIEFERPOSTEN.
           MOVE ZEROES TO LIEFPOST-STATUS.
           PERFORM UNTIL EOF-LIEFPOST
              READ LIEFERPOSTENDATEI
              AT END MOVE '09' TO LIEFPOST-STATUS
              NOT AT END
                 MOVE SATZKZ-LP    TO WORK-SATZKZ
                 MOVE ARTIKELNR    TO WORK-ARTIKELNR
                 MOVE ARTIKELNAME  TO WORK-ARTIKELNAME
                 MOVE BESTELLMENGE TO WORK-BESTELLMENGE
                 MOVE ZEROES       TO WORK-LIEFERANTNR
                 PERFORM FIND-LIEFERANT
                 MOVE BEST-LIEFERANT TO WORK-LIEFERANTNR
                 MOVE BEST-PREIS   TO WORK-STUECKPREIS
                 WRITE WORKREC
              END-READ
           END-PERFORM.
TEST       DISPLAY 'Lieferposten copied and Lieferant selected'
TEST       ACCEPT CONTINUE-KEY.
           EXIT.
      *****************************************************
       CLOSE-VERSANDPOSTEN.
           CLOSE PREISDATEI.
           CLOSE LIEFERPOSTENDATEI
           CLOSE LIEFERPOSTEN-WORKFILE.
           EXIT.
      *****************************************************************
      *****************************************************************
       SORTIERUNGS      SECTION.
      *    - sort  Lieferposten Workfile                               *
           MOVE ZERO TO SORTFILE-STATUS
           SORT LIEFERPOSTEN-SORTFILE ON ASCENDING KEY SORT-LIEFERANTNR
                 USING  LIEFERPOSTEN-WORKFILE
                 GIVING LIEFERPOSTEN-WORKFILE.
           IF NOT OK-SORTFILE
              DISPLAY 'Error on sorting Lieferposten '
              DISPLAY 'Sort File Status:' SORTFILE-STATUS
              DISPLAY 'Press Enter to Continue !!!'
              ACCEPT CONTINUE-KEY
           END-IF.
TEST       DISPLAY 'Sort Lieferposten by Lieferantennr completed'
TEST       ACCEPT CONTINUE-KEY.
      *****************************************************************
      *****************************************************************
       LIEFERPOSTENAUSGABE   SECTION.
      *    - open  Lieferposten Workfile                               *
      *    - init  Lieferauftragsdatei                                 *
      *    - Do for each Lieferant                                     *
      *      - init  Lieferauftrag                                     *
      *      - Do for each Lieferposten                                *
      *        - write Lieferposten Workfile                           *
      *      - EndLoop                                                 *
      *      - sum   Lieferauftrag                                     *
      *    - Endloop                                                   *
      *    - close Lieferposten Workfile                               *
      *    - close Lieferauftragsdatei                                 *
      *****************************************************
       INIT-LIEFERAUFTRAG.
           MOVE ZEROES TO LIEFERANT-STATUS.
           OPEN INPUT LIEFERANTEN.
           IF LIEFERANT-STATUS NOT = ZEROES
              DISPLAY 'LIEFERANTENDATEN not found'
              DISPLAY 'Press Enter to terminate !'
              ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
TEST       DISPLAY 'Lieferantendatei opened '.
           MOVE 'Lieferauftrag   ' TO KOPFZEILE-TITEL.
           MOVE 'Lieferauftragsbearbeitung ' TO KOPFZEILE-PROGRAM.
           PERFORM OPEN-LISTE.
           EXIT.
      *****************************************************
       OPEN-LIEFERPOSTEN-WORKFILE.
           MOVE ZEROES TO WORKFILE-STATUS.
           OPEN INPUT LIEFERPOSTEN-WORKFILE.
           IF WORKFILE-STATUS NOT = ZEROES
              DISPLAY 'LIEFERPOSTEN-WORKFILE not found'
              DISPLAY 'Press Enter to terminate !'
              ACCEPT  CONTINUE-KEY
              GOBACK
           END-IF.
TEST       DISPLAY 'Workfile was reopened '.
      *****************************************************
       READ-SORTIERTE-LIEFERPOSTEN.
           MOVE ZEROES TO WORKFILE-STATUS.
           PERFORM UNTIL EOF-WORKFILE
              READ LIEFERPOSTEN-WORKFILE
              AT END MOVE '09' TO WORKFILE-STATUS
              NOT AT END
                  IF WORK-LIEFERANTNR NOT = LETZTE-LIEFERANTNR
                     IF LETZTE-LIEFERANTNR NOT = ZEROES
                        PERFORM DRUCKE-LIEFERAUFTRAGSFUSS
                     END-IF
                     IF WORK-SATZKZ = 'LP'
                        MOVE ZERO TO FEHLER-CODE
                        PERFORM GET-LIEFERANTENDATEN
                        IF FEHLER-CODE = ZERO
                           PERFORM DRUCKE-LIEFERAUFTRAGSKOPF
                        END-IF
                     ELSE
                        MOVE 1 TO FEHLER-CODE
                     END-IF
                  END-IF
                  IF WORK-SATZKZ = 'LP'
                     PERFORM DRUCKE-LIEFERPOSTEN
                  ELSE
                     MOVE 1 TO FEHLER-CODE
                  END-IF
              END-READ
              MOVE WORK-LIEFERANTNR TO LETZTE-LIEFERANTNR
           END-PERFORM.
           PERFORM DRUCKE-LIEFERAUFTRAGSFUSS.
TEST       DISPLAY 'Lieferauftraege have been printed'
TEST       ACCEPT CONTINUE-KEY.
           EXIT.
      *****************************************************
       CLOSE-RECHNUNGSPOSTEN.
           CLOSE LIEFERANTEN.
           CLOSE LIEFERPOSTEN-WORKFILE.
           PERFORM CLOSE-LISTE.
           STOP RUN.
      *****************************************************************
      *****************************************************************
       LIEFERAUFTRAGAUSGABE   SECTION.
      *    - write Lieferauftragkopf                                   *
      *    - write Lieferauftragsposten                                *
      *    - write Lieferauftragsfuss                                  *
      *****************************************************
       DRUCKE-LIEFERAUFTRAGSKOPF.
           ADD  1 TO AUFTRAGSNR
           MOVE AUFTRAGSNR TO LIEFERAUFTRAGSKOPF-AUFTRAGNR
           MOVE WORK-LIEFERANTNR TO LIEFERAUFTRAGSKOPF-LIEF-KZ
           MOVE LIEFERANT-NAME TO LIEFERAUFTRAGSKOPF-NAME
           MOVE LIEFERANT-PLZ TO LIEFERAUFTRAGSKOPF-PLZ
           MOVE LIEFERANT-ORT TO LIEFERAUFTRAGSKOPF-ORT
           MOVE LIEFERANT-STRASSE TO LIEFERAUFTRAGSKOPF-STRASSE
           MOVE LIEFERANT-LAND TO LIEFERAUFTRAGSKOPF-LAND
           EVALUATE LIEFERWEG
               WHEN 3
                   MOVE LIEFERMODUS-3 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
               WHEN 2
                   MOVE LIEFERMODUS-2 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
               WHEN 1
                   MOVE LIEFERMODUS-1 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
               WHEN OTHER
                   MOVE LIEFERMODUS-4 TO LIEFERAUFTRAGSKOPF-LIEFMODUS
           END-EVALUATE
           PERFORM PRINT-HEADER
           MOVE ZERO TO LIEFERMENGE
           MOVE ZERO TO LIEFERBETRAG
           MOVE ZERO TO LIEF-POSTEN-NR.
      *****************************************************
       DRUCKE-LIEFERPOSTEN.
           ADD 1 TO LIEF-POSTEN-NR
           MOVE LIEF-POSTEN-NR   TO LIEFERZEILE-BESTELLNR
           MOVE WORK-ARTIKELNR   TO LIEFERZEILE-ARTIKELNR
           MOVE WORK-ARTIKELNAME TO LIEFERZEILE-ARTIKELNAME
           MOVE WORK-BESTELLMENGE TO LIEFERZEILE-BESTELLMENGE
           MOVE WORK-STUECKPREIS TO LIEFERZEILE-STUECKPREIS
           ADD WORK-BESTELLMENGE TO LIEFERMENGE
           MULTIPLY WORK-BESTELLMENGE BY WORK-STUECKPREIS
              GIVING GESAMTPREIS
           MOVE GESAMTPREIS      TO LIEFERZEILE-GESAMTPREIS
           ADD GESAMTPREIS TO LIEFERBETRAG
           PERFORM PRINT-LINE.
      *****************************************************
       DRUCKE-LIEFERAUFTRAGSFUSS.
           EVALUATE FEHLER-CODE
              WHEN 1
                 MOVE FM-1 TO FEHLERMELDUNG
                 MOVE FEHLERZEILE TO FUSSZEILE-2
              WHEN 2
                 MOVE FM-2 TO FEHLERMELDUNG
                 MOVE FEHLERZEILE TO FUSSZEILE-2
              WHEN 3
                 MOVE FM-3 TO FEHLERMELDUNG
                 MOVE FEHLERZEILE TO FUSSZEILE-2
              WHEN OTHER
                 MOVE LIEFERMENGE TO LIEFERAUFTRAGSFUSS-MENGE
                 MOVE LIEFERBETRAG TO LIEFERAUFTRAGSFUSS-BETRAG
                 MOVE LIEFERAUFTRAGSFUSS-1 TO FUSSZEILE-2
                 MOVE LIEFERAUFTRAGSFUSS-2 TO FUSSZEILE-3
           END-EVALUATE.
           PERFORM PRINT-FOOTING
           PERFORM PAGE-CHANGE.
           EXIT.
      *****************************************************************
      *****************************************************************
       ACCESS-SUBROUTINES SECTION.
      *****************************************************
       FIND-LIEFERANT.
           MOVE 9999.99 TO BEST-PREIS.
           MOVE ZERO TO BEST-LIEFERANT.
           MOVE ZERO TO PREIS-STATUS.
           MOVE ARTIKELNR TO ART-NR.
           START PREISDATEI KEY >= ART-NR
              INVALID KEY
                 MOVE '05' TO PREIS-STATUS
           END-START.
           IF PREIS-STATUS = ZERO
              READ PREISDATEI
                 INVALID KEY
                    MOVE '05' TO PREIS-STATUS
                 NOT INVALID KEY
                    MOVE ART-LIEF-PREIS TO BEST-PREIS
                    MOVE ART-LIEF-NR    TO BEST-LIEFERANT
              END-READ
              PERFORM UNTIL PREIS-STATUS NOT = ZERO
                         OR ART-NR NOT = ARTIKELNR
                 READ PREISDATEI NEXT RECORD
                    AT END
                       MOVE '09' TO PREIS-STATUS
                    NOT AT END
                       IF ART-NR = ARTIKELNR
                          IF ART-LIEF-PREIS < BEST-PREIS
                             MOVE ART-LIEF-PREIS TO BEST-PREIS
                             MOVE ART-LIEF-NR    TO BEST-LIEFERANT
                          END-IF
                       END-IF
                 END-READ
              END-PERFORM
           ELSE
              MOVE 2 TO FEHLER-CODE
           END-IF.
TEST       DISPLAY 'Lieferant mit niedrigsten Preis selected '
           EXIT.
      *****************************************************
       GET-LIEFERANTENDATEN.
           MOVE ZERO TO LIEFERANT-STATUS.
           MOVE WORK-LIEFERANTNR TO LIEF-NR
           READ LIEFERANTEN
              INVALID KEY
                 MOVE '05' TO LIEFERANT-STATUS
           END-READ.
           IF LIEFERANT-STATUS NOT = ZERO
              MOVE 3 TO FEHLER-CODE
           ELSE
TEST          DISPLAY 'Lieferant was found '
           END-IF.
           EXIT.
      *****************************************************************
      *****************************************************************
       LIST-SUBROUTINES SECTION.
      *****************************************************
       OPEN-LISTE.
           OPEN OUTPUT LIEFERAUFTRAGSDATEI.
           ACCEPT TAGESDATUM FROM DATE.
           MOVE TAG TO LISTENTAG.
           MOVE MONAT TO LISTENMONAT.
           MOVE JAHR TO LISTENJAHR.
           MOVE LISTENDATUM TO KOPFZEILE-DATUM.
           MOVE SEITENNR TO KOPFZEILE-SEITE.
           MOVE LIEFERANTENPROGRAMM TO KOPFZEILE-PROGRAM
           MOVE LIEFERAUFTRAGSTITEL TO TITELZEILE-7
           EXIT.
      *****************************************************
       PRINT-HEADER.
           MOVE SEITENNR TO KOPFZEILE-SEITE
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-2
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-3
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-4
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-2
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           WRITE AUFTRAGSSATZ FROM TITELZEILE-7
           WRITE AUFTRAGSSATZ FROM TITELZEILE-8
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-1
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-2
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-3
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-4
           WRITE AUFTRAGSSATZ FROM LIEFERAUFTRAGSKOPF-5
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           WRITE AUFTRAGSSATZ FROM LIEFERPOSTENTITEL
           WRITE AUFTRAGSSATZ FROM KOPFZEILE-1
           MOVE 16 TO ZEILENANZAHL
           ADD 1 TO SEITENNR.
           EXIT.
      *****************************************************
       PRINT-LINE.
           MOVE LIEFERPOSTENZEILE TO AUFTRAGSZEILE.
           WRITE AUFTRAGSSATZ.
           ADD 1 TO ZEILENANZAHL
           EXIT.
      *****************************************************
       PRINT-FOOTING.
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-1
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-2
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-3
           WRITE AUFTRAGSSATZ FROM FUSSZEILE-1
           ADD 3 TO ZEILENANZAHL
           EXIT.
      *****************************************************
       PAGE-CHANGE.
           MOVE SPACES TO AUFTRAGSSATZ
           MOVE  '/' TO AUFTRAGSSATZ
           WRITE AUFTRAGSSATZ AFTER PAGE.
           EXIT.
      *****************************************************
       CLOSE-LISTE.
           CLOSE LIEFERAUFTRAGSDATEI.
           DISPLAY 'Lieferauftragsdatei wurde erstellt in LiefAuft.txt'
           DISPLAY 'Press Enter to terminate !!!'
           ACCEPT CONTINUE-KEY.
      * end of source
      ******************************************************************
