      ******************************************************************
      *   PROGRAM  FOR  CONVERTING AN XML FILE TO COBOL                *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. DISPATCH.
      *AUTHOR.     H. SNEED.
      *DATE.       JAN.2001.
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  XML FILE PROCESSING                           *
      * Open XML File                                                  *
      * create a  Record from the XML Description                      *
      * Pass Record to calling Program                                 *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *
           SELECT SCHEMA-FILE  ASSIGN TO DYNAMIC XSD-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS XSD-STAT.
      *
           SELECT XML-FILE  ASSIGN TO DYNAMIC INPUT-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS INPUT-STAT.
      *
           SELECT TEST-LOG ASSIGN TO DYNAMIC LOG-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS LOG-STAT.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
      *
       FD  SCHEMA-FILE
           RECORD CONTAINS 92 CHARACTERS
           DATA RECORD IS XSD-REC.
       01  XSD-REC                          PIC X(92).
      *
       FD  XML-FILE
           RECORD CONTAINS 120 CHARACTERS
           DATA RECORD IS XML-REC.
       01  XML-REC                          PIC X(120).
      *
       FD  TEST-LOG
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS LOG-REC.
       01  LOG-REC.
           05  LOG-REC-NR                   PIC 9(3).
           05  FILLER                       PIC X.
           05  LOG-TEXT                     PIC X(64).
           05  FILLER                       PIC X(4).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  INPUT-FILE                        PIC X(80).
       77  XSD-FILE                          PIC X(80).
       77  LOG-FILE                          PIC X(80).
       77  LOG-STAT                          PIC XX.
       77  INPUT-STAT                        PIC XX.
           88  INPUT-OK                      VALUE '00'.
           88  INPUT-EOF                     VALUE '09'.
       77  XSD-STAT                          PIC XX.
           88  XSD-OK                        VALUE '00'.
           88  XSD-EOF                       VALUE '09'.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9999.
           05  J                             PIC 9999.
           05  K                             PIC 9999.
      ****************************************************************
      *
      *                          Pointer Fields
       01  POINTERS.
           05  XML-PTR                       PIC 9999 BINARY.
           05  NAME-PTR                      PIC 9999 BINARY.
           05  VAL-PTR                       PIC 9999 BINARY.
           05  REC-PTR                       PIC 9999 BINARY.
           05  XSD-PTR                       PIC 9999 BINARY.
           05  REC-LNG                       PIC 9999 BINARY.
           05  REC-LENGTH                    PIC 9999 BINARY.
           05  FIELD-PTR                     PIC 9999 BINARY.
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  FELD-MAX                      PIC 9999 VALUE 200.
           05  REC-MAX                       PIC 9999 VALUE 2048.
           05  LINE-MAX                      PIC 9999 VALUE 120.
           05  NAME-MAX                      PIC 9999 VALUE 30.
           05  NUM-MAX                       PIC 9999 VALUE 16.
           05  VALUE-MAX                     PIC 9999 VALUE 80.
           05  FILE-NAME-LNG                 PIC 9999 VALUE 16.
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  END-SW                        PIC 9    VALUE 0.
           05  DEC-SW                        PIC 9    VALUE 0.
           05  COM-SW                        PIC 9    VALUE 0.
           05  DOC-SW                        PIC 9    VALUE 0.
           05  FIL-SW                        PIC 9    VALUE 0.
           05  REC-SW                        PIC 9    VALUE 0.
           05  OPEN-SW                       PIC 9    VALUE 0.
           05  READ-SW                       PIC 9    VALUE 0.
           05  SIGN-SW                       PIC 9    VALUE 0.
           05  FOUND-SW                      PIC 9    VALUE 0.
           05  VALUE-SW                      PIC 9    VALUE 0.
           05  COMMENT-SW                    PIC 9    VALUE 0.
      *
      *****************************************************************
      *  XML DOCUMENT                                                 *
      *****************************************************************
      *    XML Document Lines
       01  XML-LINES.
           05  XML-LINE                          PIC X(92).
           05  XML-CHARACTERS    REDEFINES XML-LINE.
               10  XML-CHAR                      PIC X OCCURS 92 TIMES.
      *****************************************************************
      *  XSD SCHEMA                                                   *
      *****************************************************************
      *    XSD Schema Lines
       01  XSD-LINES.
           05  XSD-LINE                          PIC X(92).
           05  XSD-CHARACTERS    REDEFINES XSD-LINE.
               10  XSD-CHAR                      PIC X OCCURS 92 TIMES.
      *
      ****************************************************************
      *    Global working data
       01  GLOBAL-DATA.
           05  REC-NR                            PIC 999 VALUE ZERO.
           05  FIELD-NR                          PIC 999 VALUE ZERO.
           05  DATA-NAME                         PIC X(30).
           05  NAME-STRING           REDEFINES   DATA-NAME.
               10  NAME-CHAR                     PIC X OCCURS 30  TIMES.
           05  FIELD-VALUE                       PIC X(80).
           05  VALUE-STRING          REDEFINES   FIELD-VALUE.
               10  VALUE-CHAR                    PIC X OCCURS 80  TIMES.
           05  REC-VALUE                         PIC X(2048).
           05  REC-STRING            REDEFINES   REC-VALUE.
               10  REC-CHAR                      PIC X OCCURS 2048 TIMES.
      ****************************************************************
      *    Local data description fields
       01  LOCAL-DATA.
           05  SET-VALUE-DATA.
               10  FIELD-NAME                    PIC X(30).
               10  FIELD-PICT                    PIC X(18).
               10  FIELD-LEV                     PIC 99.
               10  FIELD-POS                     PIC 9999 BINARY.
               10  FIELD-LNG                     PIC 9999 BINARY.
               10  FIELD-TYPE                    PIC X.
                   88  TYPE-CHAR                 VALUE 'X'.
                   88  TYPE-ALPHA                VALUE 'A'.
                   88  TYPE-DECIMAL              VALUE 'D'.
                   88  TYPE-PACKED               VALUE 'P'.
                   88  TYPE-FLOAT                VALUE 'F'.
                   88  TYPE-BINARY-2             VALUE 'I'.
                   88  TYPE-BINARY-4             VALUE 'B'.
                   88  TYPE-POINTER              VALUE 'T'.
                   88  TYPE-HEX                  VALUE 'H'.
      ****************************************************************
      *    Numeric Data Conversion Fields
           05  CONVERT-NUMERIC-DATA.
               10  NR-DIGITS                     PIC 999.
               10  NUMERIC-VALUE                 PIC 9(16).
               10  NUMERIC-STRING REDEFINES NUMERIC-VALUE.
                   15  NUM-CHAR                  PIC X OCCURS 16 TIMES.
               10  DECIMAL-VALUE                 PIC 9(16).
               10  PACKED-VALUE                  PIC 9(16) COMP-3.
               10  SHORT-VALUE                   PIC 9(4)  BINARY.
               10  SHORT-HEX-VALUE    REDEFINES SHORT-VALUE
                                                 PIC X(2).
               10  LONG-VALUE                    PIC 9(8)  BINARY.
               10  LONG-HEX-VALUE     REDEFINES LONG-VALUE
                                                 PIC X(4).
               10  SIGNED-VALUE                  PIC S9(16).
               10  HEX-SIGNED-VALUE REDEFINES SIGNED-VALUE
                                                 PIC X(16).
               10  DECIMAL-VALUE-S               PIC S9(15).
               10  PACKED-VALUE-S                PIC S9(15) COMP-3.
               10  SHORT-VALUE-S                 PIC S9(4)  BINARY.
               10  SHORT-HEX-VALUE-S  REDEFINES SHORT-VALUE-S
                                                 PIC X(2).
               10  LONG-VALUE-S                  PIC S9(8)  BINARY.
               10  LONG-HEX-VALUE-S   REDEFINES LONG-VALUE-S
                                                 PIC X(4).
      ****************************************************************
           05  UNSIGNED-PACKED-DATA.
      *    UnSigned Packed Conversion Fields
               10  PACKED-VALUE-2                PIC 999    COMP-3.
               10  HEX-VALUE-2        REDEFINES PACKED-VALUE-2
                                                 PIC X(2).
               10  PACKED-VALUE-3                PIC 9(5)   COMP-3.
               10  HEX-VALUE-3        REDEFINES PACKED-VALUE-3
                                                 PIC X(3).
               10  PACKED-VALUE-4                PIC 9(7)   COMP-3.
               10  HEX-VALUE-4        REDEFINES PACKED-VALUE-4
                                                 PIC X(4).
               10  PACKED-VALUE-5                PIC 9(9)   COMP-3.
               10  HEX-VALUE-5        REDEFINES PACKED-VALUE-5
                                                 PIC X(5).
               10  PACKED-VALUE-6                PIC 9(11)  COMP-3.
               10  HEX-VALUE-6        REDEFINES PACKED-VALUE-6
                                                 PIC X(6).
               10  PACKED-VALUE-7                PIC 9(13)  COMP-3.
               10  HEX-VALUE-7        REDEFINES PACKED-VALUE-7
                                                 PIC X(7).
               10  PACKED-VALUE-8                PIC 9(15)  COMP-3.
               10  HEX-VALUE-8        REDEFINES PACKED-VALUE-8
                                                 PIC X(8).
      ****************************************************************
      *    Signed Packed Conversion Fields
           05  SIGNED-PACKED-DATA.
               10  PACKED-VALUE-2-S              PIC S999   COMP-3.
               10  HEX-VALUE-2-S      REDEFINES PACKED-VALUE-2-S
                                                 PIC X(2).
               10  PACKED-VALUE-3-S              PIC S9(5)  COMP-3.
               10  HEX-VALUE-3-S      REDEFINES PACKED-VALUE-3-S
                                                 PIC X(3).
               10  PACKED-VALUE-4-S              PIC S9(7)  COMP-3.
               10  HEX-VALUE-4-S      REDEFINES PACKED-VALUE-4-S
                                                 PIC X(4).
               10  PACKED-VALUE-5-S              PIC S9(9)  COMP-3.
               10  HEX-VALUE-5-S      REDEFINES PACKED-VALUE-5-S
                                                 PIC X(5).
               10  PACKED-VALUE-6-S              PIC S9(11) COMP-3.
               10  HEX-VALUE-6-S      REDEFINES PACKED-VALUE-6-S
                                                 PIC X(6).
               10  PACKED-VALUE-7-S              PIC S9(13) COMP-3.
               10  HEX-VALUE-7-S      REDEFINES PACKED-VALUE-7-S
                                                 PIC X(7).
               10  PACKED-VALUE-8-S              PIC S9(15) COMP-3.
               10  HEX-VALUE-8-S      REDEFINES PACKED-VALUE-8-S
                                                 PIC X(8).
      ****************************************************************
      *    Fields for handling schema numeric attributes
           05  CONVERT-SCHEMA-NUMBERS.
               10  FELD-POS-X                    PIC XXXX.
               10  FELD-POS-N   REDEFINES FELD-POS-X
                                                 PIC 9999.
               10  FELD-LNG-X                    PIC XXXX.
               10  FELD-LNG-N   REDEFINES FELD-LNG-X
                                                 PIC 9999.
               10  FELD-DIM-X                    PIC XXXX.
               10  FELD-DIM-N   REDEFINES FELD-DIM-X
                                                 PIC 9999.
      ****************************************************************
      *    Record Description is a copy member
TEST***    COPY DATATAB.
      ****************************************************************
      *    Record Description is a copy member
       01  DATATAB.
           05  FIL-NAME                          PIC X(30).
           05  REC-NAME                          PIC X(30).
           05  FELD-NR                           PIC 999 VALUE ZERO.
           05  FELD                              OCCURS 200 TIMES.
               10  REC-TYPE                      PIC X(30).
               10  FELD-NAME                     PIC X(30).
               10  FELD-LEV                      PIC 99.
               10  FELD-TYP                      PIC XX.
               10  FELD-POS                      PIC 9999.
               10  FELD-LNG                      PIC 9999.
               10  FELD-DIM                      PIC 9999.
               10  FELD-PIC                      PIC X(18).
               10  FELD-DEF                      PIC X(30).
      *
      ****************************************************************
      *
      ****************************************************************
TEST   LINKAGE SECTION.
      *                          Parameters
       77  XML-FUNCODE                       PIC XX.
       77  XML-RETCODE                       PIC XX.
       77  XML-FILNAME                       PIC X(8).
       77  XML-METHOD                        PIC X(30).
      *    Data Record passed back to the Caller
       77  DATA-REC-LNG                      PIC 9(8)  USAGE BINARY.
       01  DATA-REC                          PIC X(2048).
       01  DATA-CHARS                        REDEFINES DATA-REC.
           05 DATA-CHAR                      PIC X  OCCURS 2048 TIMES
                                             DEPENDING ON REC-LNG.
       01  SIGNED-PACKED-2                   PIC S9(3) USAGE COMP-3.
       01  SIGNED-PACKED-3                   PIC S9(5) USAGE COMP-3.
      *
      ******************************************************************
      *
      /
       PROCEDURE DIVISION USING  XML-FUNCODE,
                                 XML-RETCODE,
                                 XML-FILNAME,
                                 XML-METHOD,
                                 DATA-REC-LNG,
                                 DATA-REC.
      ******************************************************************
      *     Main  Routine     ******************************************
      ******************************************************************
       XMLSTUB-CONTROL SECTION.
      ******************************************************************
       XML-STUB-CONTROL.
      ***********************
           IF OPEN-SW = ZERO
              DISPLAY '+-------------------------------------------+'
              DISPLAY '| DISPATCH Program Version 1.2 of June 2002 |'
              DISPLAY '+-------------------------------------------+'
              DISPLAY '| XML Input File Name:' XML-FILNAME
TEST***       ACCEPT  CONTINUE-KEY
              PERFORM CREATE-DATA-TAB
              PERFORM OPEN-XML-FILE
              MOVE 1 TO OPEN-SW
           END-IF
           EVALUATE XML-FUNCODE
           WHEN 'RD'
              PERFORM READ-XML-FILE
           WHEN 'WR'
              MOVE '09' TO XML-RETCODE
              PERFORM ERROR-EXIT
           WHEN OTHER
              MOVE '09' TO XML-RETCODE
              PERFORM ERROR-EXIT
           END-EVALUATE.
           IF XML-RETCODE NOT = ZERO
              CLOSE TEST-LOG
              CLOSE XML-FILE
              MOVE ZERO TO OPEN-SW
           END-IF
TEST***    DISPLAY 'DISPATCH completed '
TEST***    ACCEPT  CONTINUE-KEY
           GOBACK.
      *****************************************************
       ERROR-EXIT.
      ***********************
           CLOSE TEST-LOG
           CLOSE XML-FILE
           MOVE ZERO TO OPEN-SW
TEST***    DISPLAY 'DISPATCH terminated'
TEST***    DISPLAY 'XML-Retcode = ' XML-RETCODE
TEST***    ACCEPT  CONTINUE-KEY
           GOBACK.
      ******************************************************************
       READ-XML-FILE SECTION.
      *****************************************************
       CHECK-REC-NAME.
      ***********************
TEST  *    DISPLAY 'Check Rec Name '
           MOVE 0 TO END-SW
           MOVE 0 TO REC-SW
           MOVE SPACES TO DATA-NAME
           PERFORM READ-XML-LINE
           IF XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
              IF XML-CHAR (XML-PTR) = '/'
                 PERFORM CHECK-FIL-ENDE
              ELSE
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) NOT = SPACE
                    ADD 1 TO XML-PTR
                 END-PERFORM
                 MOVE 0 TO NAME-PTR
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) = SPACE
                            OR XML-CHAR (XML-PTR) = '>'
                    ADD 1 TO NAME-PTR
                    MOVE XML-CHAR (XML-PTR) TO NAME-CHAR (NAME-PTR)
                    ADD 1 TO XML-PTR
                 END-PERFORM
              END-IF
           END-IF.
TEST***    DISPLAY 'Rec-Name = ' REC-NAME
TEST***    DISPLAY 'Data Name= ' DATA-NAME
TEST***    ACCEPT CONTINUE-KEY.
           IF DATA-NAME = REC-NAME
              MOVE 1 TO REC-SW
           ELSE
              MOVE 1 TO I
              PERFORM UNTIL I > FELD-NR
                         OR I > FELD-MAX
                         OR REC-SW = 1
                 IF DATA-NAME = REC-TYPE (I)
                    MOVE 1 TO REC-SW
                    MOVE REC-TYPE (I) TO REC-NAME
                 END-IF
                 ADD 1 TO I
              END-PERFORM
              IF REC-SW = ZERO
                 MOVE '92' TO XML-RETCODE
                 PERFORM ERROR-EXIT
              END-IF
           END-IF.
      *****************************************************
       READ-ALL-FIELDS.
      ***********************
           PERFORM INIT-VALUES.
           MOVE ZERO TO END-SW
           MOVE ZERO TO READ-SW
           MOVE ZERO TO FIELD-NR
           PERFORM UNTIL INPUT-EOF
                      OR END-SW = 1
              IF READ-SW = ZERO
                 PERFORM READ-XML-LINE
                 PERFORM CHECK-REC-ENDE
              END-IF
              IF END-SW = ZERO
                 MOVE ZERO TO VALUE-SW
                 PERFORM CHECK-VALUE
                 IF VALUE-SW = 1
                    PERFORM SET-VALUE
                    ADD 1 TO FIELD-NR
                 END-IF
              END-IF
              MOVE ZERO TO READ-SW
           END-PERFORM.
      *****************************************************
       RETURN-RECORD.
      ***********************
TEST***    DISPLAY 'Return-Record '
           IF INPUT-STAT NOT = ZEROES
              MOVE INPUT-STAT TO XML-RETCODE
              MOVE 'End of XML Data File'  TO LOG-TEXT
              WRITE LOG-REC
           ELSE
              MOVE 1 TO REC-PTR
              PERFORM UNTIL REC-PTR > REC-LNG
                         OR REC-PTR > REC-MAX
                 MOVE REC-CHAR (REC-PTR) TO DATA-CHAR (REC-PTR)
                 ADD 1 TO REC-PTR
              END-PERFORM
              ADD 1 TO REC-NR
              MOVE SPACES TO LOG-REC
              MOVE REC-NR TO LOG-REC-NR
              MOVE DATA-REC (1:64) TO LOG-TEXT
              WRITE LOG-REC
           END-IF.
TEST***    DISPLAY 'Data-Rec: ' DATA-REC (1:REC-LNG)
TEST***    DISPLAY 'Record Processed '
TEST***    DISPLAY 'Nr. Fields = ' FIELD-NR
TEST***    DISPLAY 'Press Enter to return to Caller '
TEST***    ACCEPT CONTINUE-KEY.
      ******************************************************************
      *     Sub Routines      ******************************************
      ******************************************************************
       XMLSTUB-SUB SECTION.
      *****************************************************
      *****************************************************
       CHECK-REC-ENDE.
      ***********************
TEST***    DISPLAY 'Check Rec Ende '
TEST***    DISPLAY 'Rec-Name = ' REC-NAME
           MOVE 0 TO END-SW
           MOVE SPACES TO DATA-NAME
           IF XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
              IF XML-CHAR (XML-PTR) = '/'
                 ADD 1 TO XML-PTR
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) NOT = SPACE
                    ADD 1 TO XML-PTR
                 END-PERFORM
                 MOVE 0 TO NAME-PTR
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) = SPACE
                            OR XML-CHAR (XML-PTR) = '>'
                    ADD 1 TO NAME-PTR
                    MOVE XML-CHAR (XML-PTR) TO NAME-CHAR (NAME-PTR)
                    ADD 1 TO XML-PTR
                 END-PERFORM
              END-IF
           END-IF.
           IF DATA-NAME (1:NAME-PTR) = REC-NAME
              MOVE 1 TO END-SW
           END-IF.
TEST***    DISPLAY 'Rec-Name = ' REC-NAME
TEST***    DISPLAY 'Data Name= ' DATA-NAME
TEST***    DISPLAY 'Rec-End-SW = '  END-SW.
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************
       INIT-VALUES.
      ***********************
TEST***    DISPLAY 'Init Record'
           MOVE SPACES TO REC-VALUE.
           MOVE 1 TO I.
           PERFORM UNTIL I > FELD-NR
              IF REC-TYPE (I) = REC-NAME
                 MOVE FELD-TYP (I) TO FIELD-TYPE
                 MOVE FELD-POS (I) TO FIELD-PTR
                 MOVE FELD-LNG (I) TO FIELD-LNG
                 IF FIELD-TYPE  = 'B' OR 'P' OR 'D' OR 'F' OR  'I'
                    MOVE ZEROES TO REC-VALUE (FIELD-PTR:FIELD-LNG)
                 ELSE
                    MOVE SPACES TO REC-VALUE (FIELD-PTR:FIELD-LNG)
                 END-IF
              END-IF
              ADD 1 TO I
           END-PERFORM.
TEST***    DISPLAY 'Rec-Value:' REC-VALUE
TEST***    DISPLAY 'Rec-Lng:  ' REC-LNG
TEST***    ACCEPT  CONTINUE-KEY.
      *****************************************************
       CHECK-VALUE.
      ***********************
TEST***    DISPLAY 'Check Field Value'
           MOVE 0 TO VALUE-SW
           MOVE 1 TO XML-PTR
           MOVE SPACES TO DATA-NAME
           MOVE SPACES TO FIELD-VALUE
      *  Skip to first Tag
           PERFORM UNTIL INPUT-EOF
                      OR XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
              IF XML-PTR > LINE-MAX
                 PERFORM READ-XML-LINE
              END-IF
           END-PERFORM.
      *  Check Tag
           IF XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
              IF XML-CHAR (XML-PTR) = '/'
READ             MOVE 0 TO READ-SW
              ELSE
      *  Extract name from Tag
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) NOT = SPACE
                    ADD 1 TO XML-PTR
                 END-PERFORM
                 MOVE 0 TO NAME-PTR
                 PERFORM UNTIL INPUT-EOF
                            OR XML-CHAR (XML-PTR) = '/'
                            OR XML-CHAR (XML-PTR) = '>'
                            OR XML-CHAR (XML-PTR) = SPACE
                    ADD 1 TO NAME-PTR
                    MOVE XML-CHAR (XML-PTR) TO NAME-CHAR (NAME-PTR)
                    ADD 1 TO XML-PTR
                 END-PERFORM
TEST***          DISPLAY 'Data-Name:' DATA-NAME
TEST***          ACCEPT  CONTINUE-KEY
      *  Skip over Spaces after Name **********************
                 IF XML-CHAR (XML-PTR) = SPACE
                 AND XML-PTR < LINE-MAX
                    PERFORM UNTIL INPUT-EOF
                               OR XML-CHAR (XML-PTR) = '>'
                       ADD 1 TO XML-PTR
                       IF XML-PTR > LINE-MAX
                          PERFORM READ-XML-LINE
                       END-IF
                    END-PERFORM
                 END-IF
      *  Extract Value from between Tags
                 IF XML-CHAR (XML-PTR) = '>'
                    MOVE ZERO TO VAL-PTR
                    ADD 1 TO XML-PTR
                    PERFORM UNTIL XML-PTR > LINE-MAX
                               OR XML-CHAR (XML-PTR) NOT = SPACE
                       ADD 1 TO XML-PTR
                    END-PERFORM
                    IF XML-PTR > LINE-MAX
                       PERFORM READ-XML-LINE
                    END-IF
                    PERFORM UNTIL INPUT-EOF
                               OR VAL-PTR > VALUE-MAX
                               OR XML-CHAR (XML-PTR) = '<'
                       IF XML-CHAR (XML-PTR) NOT = SPACE
                          ADD 1 TO VAL-PTR
                          MOVE XML-CHAR (XML-PTR)
                            TO VALUE-CHAR (VAL-PTR)
                       END-IF
                       ADD 1 TO XML-PTR
                       IF XML-PTR > LINE-MAX
                          PERFORM READ-XML-LINE
                       END-IF
                    END-PERFORM
      *  Check closing Tag ********************************
                    IF XML-CHAR (XML-PTR) = '<'
      *  This was a Container Element with no Value *******
      *  set read-SW to 1 to prevent reading next line
                       IF VAL-PTR = ZERO
NOREAD                    MOVE 1 TO READ-SW
                       ELSE
      *  Skip closing Tag of Element **********************
                          PERFORM UNTIL INPUT-EOF
                                     OR XML-CHAR (XML-PTR) = '>'
                             ADD 1 TO XML-PTR
                             IF XML-PTR > LINE-MAX
                                PERFORM READ-XML-LINE
                             END-IF
                          END-PERFORM
READ                      MOVE 0 TO READ-SW
                       END-IF
      *  Skip Empty Tag
                       IF XML-CHAR (XML-PTR)  = '/' AND
                          XML-CHAR (XML-PTR + 1)  = '>'
READ                      MOVE 0 TO READ-SW
                       END-IF
                    ELSE
      *  End of Tag not found
READ                   MOVE 0 TO READ-SW
                    END-IF
                 END-IF
              END-IF
           END-IF.
           IF FIELD-VALUE NOT = SPACES
              MOVE 1 TO VALUE-SW
           END-IF.
TEST***    DISPLAY 'Rec-Name = '  REC-NAME
TEST***    DISPLAY 'Data-Name= '  DATA-NAME
TEST***    DISPLAY 'Data-Value='  FIELD-VALUE
TEST***    DISPLAY 'Value-SW = '  VALUE-SW.
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************
       SET-VALUE.
      ***********************
           MOVE 1 TO I.
           PERFORM UNTIL I > FELD-NR
                      OR DATA-NAME = FELD-NAME (I)
              ADD 1 TO I
           END-PERFORM
           IF DATA-NAME = FELD-NAME (I)
      ***************** In case Record is the only attribute ****
              IF DATA-NAME = FELD-NAME (I + 1)
                 ADD 1 TO I
              END-IF
      ***********************************************************
              MOVE FELD-POS (I) TO FIELD-PTR
              MOVE FELD-LEV (I) TO FIELD-LEV
              MOVE FELD-LNG (I) TO FIELD-LNG
              MOVE FELD-TYP (I) TO FIELD-TYPE
              MOVE FELD-PIC (I) TO FIELD-PICT
              IF FIELD-TYPE = 'B' OR 'F' OR 'P' OR 'I'
                 PERFORM CONVERT-NUMERIC
              ELSE
                 IF DATA-NAME = 'RETURN-CODE'
                 OR DATA-NAME = 'STATUS-CODE'
                 OR DATA-NAME = 'PF-KEY'
                    MOVE FIELD-VALUE (1:2) TO XML-RETCODE
                 ELSE
                    IF DATA-NAME = 'NEXT-METHOD'
                       MOVE FIELD-VALUE (1:2) TO XML-METHOD
                    ELSE
                       MOVE FIELD-VALUE (1:FIELD-LNG)
                         TO REC-VALUE (FIELD-PTR:FIELD-LNG)
                    END-IF
                 END-IF
              END-IF
TEST***       DISPLAY 'Field-Name = ' FELD-NAME (I)
TEST***       DISPLAY 'Field-Lev  = ' FIELD-LEV
TEST***       DISPLAY 'Field-Typ  = ' FIELD-TYPE
TEST***       DISPLAY 'Field-Ptr  = ' FIELD-PTR
TEST***       DISPLAY 'Field-Lng  = ' FIELD-LNG
TEST***       DISPLAY 'Field-Pic  = ' FIELD-PICT
TEST***       DISPLAY 'Rec-Value  = ' REC-VALUE (1:80)
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
      *****************************************************
       CONVERT-NUMERIC.
      ***********************
      * convert numeric character strings to decimal, binary or packed
      ***********************
      * set up numeric value
           MOVE ZEROES TO NUMERIC-VALUE
           MOVE ZEROES TO SIGNED-VALUE
TEST***    DISPLAY 'Field Value:' FIELD-VALUE
      * check if minus value
           MOVE ZERO TO SIGN-SW
           IF VALUE-CHAR (1) = '-'
              MOVE 2 TO SIGN-SW
              MOVE ZERO TO VALUE-CHAR (1)
           END-IF
           IF VALUE-CHAR (1) = '+'
              MOVE 1 TO SIGN-SW
              MOVE ZERO TO VALUE-CHAR (1)
           END-IF
      * count number of digits
           MOVE 1 TO J
           MOVE ZERO TO NR-DIGITS
           PERFORM UNTIL J > NUM-MAX
                      OR VALUE-CHAR (J) = SPACE
              ADD 1 TO NR-DIGITS
              ADD 1 TO J
           END-PERFORM.
      * move field value to numeric value
           MOVE NR-DIGITS TO J.
           MOVE NUM-MAX TO K.
           PERFORM UNTIL J = 1 OR K = 1
              IF VALUE-CHAR (J) NUMERIC
              OR VALUE-CHAR (J) = '-' OR '+'
                 MOVE VALUE-CHAR (J) TO NUM-CHAR (K)
                 SUBTRACT 1 FROM K
              END-IF
              SUBTRACT 1 FROM J
           END-PERFORM.
      * copy plus value
TEST***    DISPLAY 'SIGN-SW=' SIGN-SW
           IF SIGN-SW = 1
              COMPUTE SIGNED-VALUE = NUMERIC-VALUE * 1
           END-IF.
           IF SIGN-SW = 2
              COMPUTE SIGNED-VALUE = NUMERIC-VALUE * -1
           END-IF.
TEST***    DISPLAY 'Numeric Value:' NUMERIC-VALUE
TEST***    DISPLAY 'Signed  Value:' SIGNED-VALUE
TEST***    ACCEPT CONTINUE-KEY
      *  Convert decimal numeric to binary, packed or decimal
           EVALUATE FIELD-TYPE
              WHEN 'I'
                 IF SIGN-SW > ZERO
                    MOVE SIGNED-VALUE TO SHORT-VALUE-S
TEST***             DISPLAY 'ShortValue: ' SHORT-VALUE-S
                    MOVE SHORT-HEX-VALUE-S TO REC-VALUE
                                             (FIELD-PTR:FIELD-LNG)
                 ELSE
                    MOVE NUMERIC-VALUE TO SHORT-VALUE
TEST***             DISPLAY 'ShortValue: ' SHORT-VALUE
                    MOVE SHORT-HEX-VALUE TO REC-VALUE
                                           (FIELD-PTR:FIELD-LNG)
                 END-IF
              WHEN 'B'
                 IF SIGN-SW > ZERO
                    MOVE SIGNED-VALUE TO LONG-VALUE-S
TEST***             DISPLAY 'LongValue: ' SHORT-VALUE-S
                    MOVE LONG-HEX-VALUE-S TO REC-VALUE
                                            (FIELD-PTR:FIELD-LNG)
                 ELSE
                    MOVE NUMERIC-VALUE TO LONG-VALUE
TEST***             DISPLAY 'LongValue: ' SHORT-VALUE
                    MOVE LONG-HEX-VALUE TO REC-VALUE
                                          (FIELD-PTR:FIELD-LNG)
                 END-IF
              WHEN 'F'
                 IF SIGN-SW > ZERO
                    MOVE SIGNED-VALUE TO LONG-VALUE-S
                    MOVE LONG-HEX-VALUE-S TO REC-VALUE
                                            (FIELD-PTR:FIELD-LNG)
                 ELSE
                    MOVE NUMERIC-VALUE TO LONG-VALUE
                    MOVE LONG-HEX-VALUE TO REC-VALUE
                                          (FIELD-PTR:FIELD-LNG)
                 END-IF
              WHEN 'P'
                 IF SIGN-SW > ZERO
                    PERFORM ASSIGN-PACKED-S
                 ELSE
                    PERFORM ASSIGN-PACKED
                 END-IF
              WHEN OTHER
                 IF SIGN-SW > ZERO
                    MOVE SIGNED-VALUE TO DECIMAL-VALUE-S
                    MOVE DECIMAL-VALUE-S TO REC-VALUE
                                           (FIELD-PTR:FIELD-LNG)
                 ELSE
                    MOVE NUMERIC-VALUE TO DECIMAL-VALUE
                    MOVE DECIMAL-VALUE TO REC-VALUE
                                         (FIELD-PTR:FIELD-LNG)
                 END-IF
           END-EVALUATE.
      *****************************************************
       ASSIGN-PACKED.
      ***********************
      * adjust length of unsigned packed fields
      ***********************
           EVALUATE FIELD-LNG
              WHEN 1
              WHEN 2
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-2
                 MOVE HEX-VALUE-2 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 3
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-3
                 MOVE HEX-VALUE-3 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 4
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-4
                 MOVE HEX-VALUE-4 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 5
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-5
                 MOVE HEX-VALUE-5 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 6
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-6
                 MOVE HEX-VALUE-6 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 7
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-7
                 MOVE HEX-VALUE-7 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN OTHER
                 MOVE NUMERIC-VALUE TO PACKED-VALUE-8
                 MOVE HEX-VALUE-8 TO REC-VALUE (FIELD-PTR:FIELD-LNG)
           END-EVALUATE.
      *****************************************************
       ASSIGN-PACKED-S.
      ***********************
      * adjust length of unsigned packed fields
      ***********************
           EVALUATE FIELD-LNG
              WHEN 1
              WHEN 2
                MOVE SIGNED-VALUE TO PACKED-VALUE-2-S
                MOVE HEX-VALUE-2-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 3
                MOVE SIGNED-VALUE TO PACKED-VALUE-3-S
TEST***         DISPLAY 'Packed Value:' PACKED-VALUE-3-S
                MOVE HEX-VALUE-3-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 4
                MOVE SIGNED-VALUE TO PACKED-VALUE-4-S
TEST***         DISPLAY 'Packed Value:' PACKED-VALUE-4-S
                MOVE HEX-VALUE-4-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 5
                MOVE SIGNED-VALUE TO PACKED-VALUE-5-S
TEST***         DISPLAY 'Packed Value:' PACKED-VALUE-5-S
                MOVE HEX-VALUE-5-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 6
                MOVE SIGNED-VALUE TO PACKED-VALUE-6-S
TEST***         DISPLAY 'Packed Value:' PACKED-VALUE-6-S
                MOVE HEX-VALUE-6-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN 7
                MOVE SIGNED-VALUE TO PACKED-VALUE-7-S
                MOVE HEX-VALUE-7-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
              WHEN OTHER
                MOVE SIGNED-VALUE TO PACKED-VALUE-8-S
                MOVE HEX-VALUE-8-S TO REC-VALUE (FIELD-PTR:FIELD-LNG)
           END-EVALUATE.
TEST***    MOVE REC-VALUE (FIELD-PTR:FIELD-LNG) TO HEX-SIGNED-VALUE.
TEST***    DISPLAY 'Final Signed Value is:' SIGNED-VALUE.
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************
       READ-XML-LINE.
TEST***    DISPLAY 'Read XML Line '
           READ XML-FILE
           AT END
              MOVE '09' TO INPUT-STAT
           NOT AT END
              MOVE XML-REC TO XML-LINE
              MOVE 1 TO XML-PTR
              PERFORM UNTIL XML-PTR > LINE-MAX
                         OR XML-CHAR (XML-PTR) NOT = SPACE
                 ADD 1 TO XML-PTR
              END-PERFORM
TEST***       DISPLAY 'XML-LINE:' XML-LINE
TEST***       ACCEPT CONTINUE-KEY
           END-READ.
      *****************************************************************
       OPEN-XML-FILE SECTION.
      *****************************************************
       OPEN-XML-FILES.
      ***********************
TEST***    DISPLAY 'Open XML File '
           MOVE SPACES TO INPUT-FILE
WORK       STRING 'WORKDIR\'    DELIMITED BY SIZE
                  XML-FILNAME   DELIMITED BY SPACE
                  '.xml'        DELIMITED BY SIZE
              INTO INPUT-FILE
           END-STRING.
TEST       DISPLAY 'Input File = ' INPUT-FILE
TEST***    ACCEPT CONTINUE-KEY.
           MOVE ZEROES TO INPUT-STAT
           OPEN INPUT XML-FILE
           IF INPUT-STAT NOT = ZEROES
              DISPLAY 'XML Input File not found'
              DISPLAY 'Press Enter to return to Caller'
              ACCEPT  CONTINUE-KEY
              MOVE '90' TO XML-RETCODE
              GOBACK
           END-IF.
      *****************************************************
       SKIP-PROCESSING-LINES.
      ***********************
           MOVE ZERO TO END-SW
           MOVE ZERO TO DEC-SW
           MOVE ZERO TO COM-SW
           MOVE ZERO TO DOC-SW
           PERFORM UNTIL INPUT-EOF
                      OR END-SW = 1
              READ XML-FILE
              AT END
                 MOVE '09' TO INPUT-STAT
              NOT AT END
      *  skip leading spaces
TEST***          DISPLAY 'XML-Line:' XML-REC (1:40)
TEST***          ACCEPT CONTINUE-KEY
                 MOVE XML-REC TO XML-LINE
                 MOVE 1 TO XML-PTR
                 PERFORM UNTIL XML-LINE (XML-PTR:1) NOT = SPACES
                            OR XML-PTR > LINE-MAX
                    ADD 1 TO XML-PTR
                 END-PERFORM
      *  skip XML declarations of type <?
                 IF XML-LINE (XML-PTR:2) = '<?'
TEST***             DISPLAY 'XML-Decl:' XML-LINE (1:40)
TEST***             ACCEPT CONTINUE-KEY
                    MOVE 1 TO DEC-SW
                    PERFORM UNTIL INPUT-EOF
                               OR DEC-SW = ZERO
                       PERFORM UNTIL XML-PTR > LINE-MAX
                          IF XML-LINE (XML-PTR:2) = '?>'
                             MOVE ZERO TO DEC-SW
                          END-IF
                          ADD  1 TO XML-PTR
                       END-PERFORM
                       IF DEC-SW = 1
                          PERFORM READ-XML-LINE
                       END-IF
                    END-PERFORM
                 END-IF
      *  skip XML comments
                 IF XML-LINE (XML-PTR:4) = '<!--'
TEST***             DISPLAY 'XML-Comt:' XML-LINE (1:40)
TEST***             ACCEPT CONTINUE-KEY
                    MOVE 1 TO COM-SW
                    PERFORM UNTIL INPUT-EOF
                               OR COM-SW = ZERO
                       PERFORM UNTIL XML-PTR > LINE-MAX
                          IF XML-LINE (XML-PTR:3) = '-->'
                             MOVE ZERO TO COM-SW
                          END-IF
                          ADD  1 TO XML-PTR
                       END-PERFORM
                       IF COM-SW = 1
                          PERFORM READ-XML-LINE
                       END-IF
                    END-PERFORM
                 END-IF
      *  skip XML DOC Type Definition
                 IF XML-LINE (XML-PTR:2) = '<!'
TEST***             DISPLAY 'XML-DDT:'  XML-LINE (1:40)
TEST***             ACCEPT CONTINUE-KEY
                    MOVE 1 TO DOC-SW
                    PERFORM UNTIL INPUT-EOF
                               OR DOC-SW = ZERO
                       PERFORM UNTIL XML-PTR > LINE-MAX
                          IF XML-LINE (XML-PTR:1) = '>'
                             MOVE ZERO TO DOC-SW
                          END-IF
                          ADD  1 TO XML-PTR
                       END-PERFORM
                       IF DOC-SW = 1
                          PERFORM READ-XML-LINE
                       END-IF
                    END-PERFORM
                 END-IF
      *  recognize start of data
                 IF XML-CHAR (XML-PTR) = '<'
                    COMPUTE J = XML-PTR + 1
                    PERFORM UNTIL XML-CHAR (J) NOT = SPACES
                               OR J > LINE-MAX
                       ADD 1 TO J
                    END-PERFORM
                    IF XML-CHAR (J) >= 'A' AND <= 'Z'
                    OR XML-CHAR (J) >= 'a' AND <= 'z'
TEST***                DISPLAY 'XML-DATA:'  XML-REC (1:40)
TEST***                ACCEPT CONTINUE-KEY
                       MOVE 1 TO END-SW
                    END-IF
                 END-IF
              END-READ
           END-PERFORM.
      *****************************************************
       CHECK-FIL-NAME.
      ***********************
TEST***    DISPLAY 'Check File Name '
           MOVE XML-REC TO XML-LINE
           MOVE 0 TO FIL-SW
           MOVE 1 TO XML-PTR
           MOVE SPACES TO DATA-NAME
           PERFORM UNTIL XML-PTR > LINE-MAX
                      OR XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
           END-PERFORM.
           IF XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
              IF XML-CHAR (XML-PTR) = '/'
                 CONTINUE
              ELSE
                 MOVE 0 TO NAME-PTR
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) = '>'
                    IF XML-CHAR (XML-PTR) NOT = SPACE
                       ADD 1 TO NAME-PTR
                       MOVE XML-CHAR (XML-PTR) TO NAME-CHAR (NAME-PTR)
                    END-IF
                    ADD 1 TO XML-PTR
                 END-PERFORM
              END-IF
           END-IF.
TEST***    DISPLAY 'File-Name = ' XML-FILNAME
TEST***    DISPLAY 'Data Name= ' DATA-NAME
TEST***    ACCEPT CONTINUE-KEY.
           IF DATA-NAME (1:8) = XML-FILNAME
              MOVE 1 TO FIL-SW
           ELSE
              MOVE '91' TO XML-RETCODE
EXIT          PERFORM ERROR-EXIT
           END-IF.
      *****************************************************
       CHECK-FIL-ENDE.
      ***********************
TEST***    DISPLAY 'Check Fil Ende '
TEST***    DISPLAY 'Fil-Name = ' FIL-NAME
           MOVE 1 TO XML-PTR
           MOVE SPACES TO DATA-NAME
           PERFORM UNTIL XML-PTR > LINE-MAX
                      OR XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
           END-PERFORM.
           IF XML-CHAR (XML-PTR) = '<'
              ADD 1 TO XML-PTR
              IF XML-CHAR (XML-PTR) = '/'
                 ADD 1 TO XML-PTR
                 MOVE 0 TO NAME-PTR
                 PERFORM UNTIL XML-PTR > LINE-MAX
                            OR XML-CHAR (XML-PTR) = '>'
                    IF XML-CHAR (XML-PTR) NOT = SPACE
                       ADD 1 TO NAME-PTR
                       MOVE XML-CHAR (XML-PTR) TO NAME-CHAR (NAME-PTR)
                    END-IF
                    ADD 1 TO XML-PTR
                 END-PERFORM
                 IF DATA-NAME (1:NAME-PTR) = FIL-NAME
                    MOVE 1 TO END-SW
                    MOVE '09' TO INPUT-STAT
                    MOVE INPUT-STAT TO XML-RETCODE
EXIT                GOBACK
                 END-IF
              END-IF
           END-IF.
      *****************************************************************
      *CREATE-DATA-TAB SECTION.
      *****************************************************
      * set up the data description table
TEST***    COPY MAKETAB.
      *****************************************************************
      * SECTION TO CREATE INTERNAL DATA TABLE                         *
      *****************************************************************
       CREATE-DATA-TAB SECTION.
      *****************************************************************
       OPEN-XSD-SCHEMA.
TEST***    DISPLAY 'Open Schema File '
           MOVE SPACES TO XSD-FILE
WORK       STRING 'WORKDIR\'    DELIMITED BY SIZE
                  XML-FILNAME   DELIMITED BY SPACE
                  '.xsd'        DELIMITED BY SIZE
              INTO XSD-FILE
           END-STRING.
TEST       DISPLAY 'Schema File = ' XSD-FILE
TEST***    ACCEPT CONTINUE-KEY.
           MOVE ZEROES TO XSD-STAT
           OPEN INPUT SCHEMA-FILE.
           IF XSD-STAT NOT = ZEROES
              DISPLAY 'Schema File not found'
              DISPLAY 'Press Enter to return to Caller'
              ACCEPT  CONTINUE-KEY
              MOVE '09' TO XML-RETCODE
              GOBACK
           END-IF.
      *****************************************************
       PROCESS-XSD-SCHEMA.
TEST***    DISPLAY 'read the schema file'.
           MOVE SPACES TO REC-NAME.
           MOVE ZERO TO FELD-NR.
           MOVE ZEROES TO XSD-STAT
      *  scan thru the Schema File
           PERFORM UNTIL FELD-NR > FELD-MAX
                      OR XSD-EOF
              READ SCHEMA-FILE
              AT END
                 MOVE '09' TO XSD-STAT
              NOT AT END
                 MOVE XSD-REC TO XSD-LINE
TEST***          DISPLAY XSD-LINE
TEST***          ACCEPT CONTINUE-KEY
      *  check line for attributes
                 MOVE 1 TO XSD-PTR
                 PERFORM UNTIL XSD-PTR > LINE-MAX
      *  check File
                    IF XSD-LINE (XSD-PTR:5) = '#file'
                       PERFORM PROCESS-FILE
                    END-IF
      *  check Record
                    IF XSD-LINE (XSD-PTR:7) = '#record'
                    OR XSD-LINE (XSD-PTR:7) = '#params'
                    OR XSD-LINE (XSD-PTR:5) = '#view'
                    OR XSD-LINE (XSD-PTR:4) = '#map'
                       PERFORM PROCESS-RECORD
                    END-IF
      *  check Group
                    IF XSD-LINE (XSD-PTR:6) = '#group'
                       PERFORM PROCESS-GROUP
                    END-IF
      *  check Field
                    IF XSD-LINE (XSD-PTR:5) = '#char'
                    OR XSD-LINE (XSD-PTR:4) = '#dec'
                    OR XSD-LINE (XSD-PTR:7) = '#packed'
                    OR XSD-LINE (XSD-PTR:7) = '#binary'
                    OR XSD-LINE (XSD-PTR:6) = '#float'
                    OR XSD-LINE (XSD-PTR:5) = '#edit'
                    OR XSD-LINE (XSD-PTR:4) = '#bit'
                    OR XSD-LINE (XSD-PTR:4) = '#hex'
                    OR XSD-LINE (XSD-PTR:8) = '#pointer'
                       PERFORM PROCESS-FIELD
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
              END-READ
           END-PERFORM.
TEST***    DISPLAY 'The schema file was read'.
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       CLOSE-XSD-SCHEMA.
      *  close out the Schema File
TEST***    DISPLAY 'close the schema file'.
           IF REC-LNG > REC-MAX
              MOVE REC-MAX TO REC-LNG
           END-IF.
           CLOSE SCHEMA-FILE.
TEST***    DISPLAY 'Record Description read for ' REC-NAME
TEST***    DISPLAY 'Nr. of Fields is : '  FELD-NR
TEST***    DISPLAY 'Record Lng    is : '  REC-LNG
TEST***    DISPLAY 'Press Enter to continue !!'
TEST***    ACCEPT  CONTINUE-KEY.
      *****************************************************************
       CREATE-DATA-TAB-ENTRY SECTION.
      *****************************************************************
       PROCESS-FILE.
      *  convert an element of type file to file name
           MOVE SPACES TO FIL-NAME
           ADD 7 TO XSD-PTR
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO FIL-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
              END-IF
           END-IF.
TEST***    DISPLAY 'File Read = ' FIL-NAME
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       PROCESS-RECORD.
      *  convert an element of type record to a record name
           MOVE SPACES TO REC-NAME
           ADD 7 TO XSD-PTR
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 MOVE SPACES TO REC-NAME
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO REC-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
              END-IF
           END-IF.
TEST***    DISPLAY 'Record Read = ' REC-NAME
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       PROCESS-GROUP.
      *  convert an element of type group to a field entry
           ADD 1 TO FELD-NR
           MOVE REC-NAME TO REC-TYPE (FELD-NR)
           MOVE ZEROES TO FELD-LEV  (FELD-NR)
           MOVE SPACES TO FELD-NAME (FELD-NR)
           MOVE SPACES TO FELD-TYP  (FELD-NR)
           MOVE ZEROES TO FELD-LNG  (FELD-NR)
           MOVE ZEROES TO FELD-POS  (FELD-NR)
           MOVE ZEROES TO FELD-DIM  (FELD-NR)
           MOVE SPACES TO FELD-DEF  (FELD-NR).
           ADD 7 TO XSD-PTR
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 MOVE SPACES TO FIELD-NAME
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 MOVE FIELD-NAME TO FELD-NAME (FELD-NR)
              END-IF
           END-IF.
      *************************************************************
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  check level
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 5 TO XSD-PTR
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 IF XSD-LINE (XSD-PTR:1) = '"'
                    ADD 1 TO XSD-PTR
                    MOVE XSD-LINE (XSD-PTR:2) TO FELD-LEV (FELD-NR)
                 END-IF
              END-IF
           END-READ.
      *************************************************************
      *  read the schema line with occurs
      *  read the schema line with redefinition
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  check level
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                 IF XSD-LINE (XSD-PTR:9) = 'maxOccurs'
                    ADD 9 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-DIM-X
                       MOVE FELD-DIM-N TO FELD-DIM (FELD-NR)
                    END-IF
                 END-IF
                 IF XSD-LINE (XSD-PTR:5) = 'redef'
                    ADD 5 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE ZERO TO K
                       MOVE SPACES TO FIELD-NAME
                       PERFORM UNTIL XSD-PTR > LINE-MAX
                                  OR K > NAME-MAX
                                  OR XSD-CHAR (XSD-PTR) = '"'
                          IF XSD-CHAR (XSD-PTR) NOT = SPACE
                             ADD 1 TO K
                             MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                          END-IF
                          ADD 1 TO XSD-PTR
                       END-PERFORM
                       MOVE FIELD-NAME TO FELD-DEF (FELD-NR)
                    END-IF
                    READ SCHEMA-FILE
                    AT END
                       MOVE '09' TO XSD-STAT
                    NOT AT END
                       MOVE XSD-REC TO XSD-LINE
TEST***                DISPLAY XSD-LINE
TEST***                ACCEPT CONTINUE-KEY
                       MOVE 1 TO XSD-PTR
                    END-READ
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
TEST***    DISPLAY 'Group Read = ' FELD-NAME (FELD-NR)
TEST***    DISPLAY 'Group Level= ' FELD-LEV  (FELD-NR)
TEST***    DISPLAY 'Group Occurs=' FELD-DIM  (FELD-NR)
TEST***    DISPLAY 'Group Redef =' FELD-DEF  (FELD-NR)
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       PROCESS-FIELD.
      *  convert an element of type field to a field entry
      *  initialize table entry
           ADD 1 TO FELD-NR
           MOVE REC-NAME TO REC-TYPE (FELD-NR)
           MOVE ZEROES TO FELD-LEV  (FELD-NR)
           MOVE SPACES TO FELD-NAME (FELD-NR)
           MOVE SPACES TO FELD-TYP  (FELD-NR)
           MOVE ZEROES TO FELD-LNG  (FELD-NR)
           MOVE ZEROES TO FELD-POS  (FELD-NR)
           MOVE ZEROES TO FELD-DIM  (FELD-NR)
           MOVE SPACES TO FELD-DEF  (FELD-NR).
      *  assign the field type
           IF XSD-LINE (XSD-PTR:5) = '#char'
              MOVE 'X' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:4) = '#dec'
              MOVE 'D' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:7) = '#packed'
              MOVE 'P' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:7) = '#binary'
              MOVE 'B' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:6) = '#float'
              MOVE 'F' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:5) = '#edit'
              MOVE 'X' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:4) = '#bit'
              MOVE 'L' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:4) = '#hex'
              MOVE 'H' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:8) = '#pointer'
              MOVE 'T' TO FELD-TYP  (FELD-NR)
           END-IF.
      *  assign the field name
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 MOVE SPACES TO FIELD-NAME
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 MOVE FIELD-NAME TO FELD-NAME (FELD-NR)
              END-IF
           END-IF.
      *************************************************************
      *  read the schema line with the level
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the field level
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 5 TO XSD-PTR
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 IF XSD-LINE (XSD-PTR:1) = '"'
                    ADD 1 TO XSD-PTR
                    MOVE XSD-LINE (XSD-PTR:2) TO FELD-LEV (FELD-NR)
                 END-IF
              END-IF
           END-READ.
      *************************************************************
      *  read the schema line with occurance
      *  read the schema line with redefinition
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the field occurs
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                 IF XSD-LINE (XSD-PTR:9) = 'maxOccurs'
                    ADD 9 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-DIM-X
                       MOVE FELD-DIM-N TO FELD-DIM (FELD-NR)
                    END-IF
                 END-IF
                 IF XSD-LINE (XSD-PTR:5) = 'redef'
                    ADD 5 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE ZERO TO K
                       MOVE SPACES TO FIELD-NAME
                       PERFORM UNTIL XSD-PTR > LINE-MAX
                                  OR K > NAME-MAX
                                  OR XSD-CHAR (XSD-PTR) = '"'
                          IF XSD-CHAR (XSD-PTR) NOT = SPACE
                             ADD 1 TO K
                             MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                          END-IF
                          ADD 1 TO XSD-PTR
                       END-PERFORM
                       MOVE FIELD-NAME TO FELD-DEF (FELD-NR)
                    END-IF
                    READ SCHEMA-FILE
                    AT END
                       MOVE '09' TO XSD-STAT
                    NOT AT END
                       MOVE XSD-REC TO XSD-LINE
TEST***                DISPLAY XSD-LINE
TEST***                ACCEPT CONTINUE-KEY
                       MOVE 1 TO XSD-PTR
                    END-READ
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
      *************************************************************
      *  read the schema line with position and length
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the field redefinition if there is one
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
      *  assign the field position
                 IF XSD-LINE (XSD-PTR:3) = 'pos'
                    ADD 3 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-POS-X
                       MOVE FELD-POS-N TO FELD-POS (FELD-NR)
                    END-IF
                 END-IF
      *  assign the field position
                 IF XSD-LINE (XSD-PTR:3) = 'lng'
                    ADD 3 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-LNG-X
                       MOVE FELD-LNG-N TO FELD-LNG (FELD-NR)
                    END-IF
      *************************************************************
      * change type binary to integer if length = 2
                    IF FELD-TYP (FELD-NR) = 'B'
                       IF FELD-LNG (FELD-NR) = 2
                          MOVE 'I' TO FELD-TYP (FELD-NR)
                       END-IF
                    END-IF
      *************************************************************
      * calculate record length after each field     **************
                    COMPUTE REC-LENGTH =  FELD-POS-N
                                       + (FELD-LNG-N * FELD-DIM-N)
                    IF REC-LENGTH > REC-LNG
                       MOVE REC-LENGTH TO REC-LNG
                    END-IF
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
      *************************************************************
      *  read the schema line with picture and usage
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the picture
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
      *  assign the field position
                 IF XSD-LINE (XSD-PTR:3) = 'pic'
                    ADD 3 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE ZERO TO K
                       MOVE SPACES TO FIELD-PICT
                       PERFORM UNTIL XSD-PTR > LINE-MAX
                                  OR K > NAME-MAX
                                  OR XSD-CHAR (XSD-PTR) = '"'
                          IF XSD-CHAR (XSD-PTR) NOT = SPACE
                             ADD 1 TO K
                             MOVE XSD-CHAR (XSD-PTR) TO FIELD-PICT (K:1)
                          END-IF
                          ADD 1 TO XSD-PTR
                       END-PERFORM
                       MOVE FIELD-PICT TO FELD-PIC (FELD-NR)
                    END-IF
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
TEST***    DISPLAY 'Field Name = ' FELD-NAME (FELD-NR)
TEST***    DISPLAY 'Field Level= ' FELD-LEV  (FELD-NR)
TEST***    DISPLAY 'Field Occur =' FELD-DIM  (FELD-NR)
TEST***    DISPLAY 'Field Redef =' FELD-DEF  (FELD-NR)
TEST***    DISPLAY 'Field Type  =' FELD-TYP  (FELD-NR)
TEST***    DISPLAY 'Field Pos   =' FELD-POS  (FELD-NR)
TEST***    DISPLAY 'Field Lng   =' FELD-LNG  (FELD-NR)
TEST***    DISPLAY 'Field Pic   =' FELD-PIC  (FELD-NR)
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      ***************************************************************
      *
      ********************** end of source *****************************
