      ******************************************************************
      *   PROGRAM  FOR  CONVERTING A COBOL DATA RECORD TO XML DOCUMENT *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. XM059O  .
      *AUTHOR.     H. SNEED.
      *DATE.       JUNE, 2001
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  XML FILE PROCESSING                           *
      * If new File Read XML Record Description                        *
      * Open XML Output File                                           *
      * Create an XML Document from the COBOL Record Area              *
      * Write Record in Output File                                    *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *
           SELECT SCHEMA-FILE  ASSIGN TO DYNAMIC XSD-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS XSD-STAT.
      *
           SELECT XML-FILE  ASSIGN TO DYNAMIC OUTPUT-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL
           FILE STATUS IS OUTPUT-STAT.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
      *
       FD  SCHEMA-FILE
           RECORD CONTAINS 92 CHARACTERS
           DATA RECORD IS XSD-REC.
       01  XSD-REC                          PIC X(92).
      *
       FD  XML-FILE
           RECORD CONTAINS 120 CHARACTERS
           DATA RECORD IS XML-REC.
       01  XML-REC                          PIC X(120).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  OUTPUT-FILE                       PIC X(80).
       77  XSD-FILE                          PIC X(80).
       77  OUTPUT-STAT                       PIC XX.
       77  XSD-STAT                          PIC XX.
           88  XSD-OK                        VALUE '00'.
           88  XSD-EOF                       VALUE '09'.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9999.
           05  J                             PIC 9999.
           05  K                             PIC 9999.
           05  SOURCE-INDEX                  PIC 9999.
           05  NEST-INDEX                    PIC 99 VALUE ZERO.
      ****************************************************************
      *
      *                          Pointer Fields
       01  POINTERS.
           05  CODE-PTR                      PIC 9999 BINARY.
           05  NAME-PTR                      PIC 9999 BINARY.
           05  NEST-PTR                      PIC 9999 BINARY.
           05  VAL-PTR                       PIC 9999 BINARY.
           05  REC-PTR                       PIC 9999 BINARY.
           05  XSD-PTR                       PIC 9999 BINARY.
           05  END-PTR                       PIC 9999 BINARY.
           05  REC-LNG                       PIC 9999 BINARY.
           05  FIELD-PTR                     PIC 9999 BINARY.
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  FELD-MAX                      PIC 9999 VALUE 200.
           05  REC-MAX                       PIC 9999 VALUE 2048.
           05  LINE-MAX                      PIC 9999 VALUE 120.
           05  NAME-MAX                      PIC 9999 VALUE 30.
           05  NEST-MAX                      PIC 9999 VALUE 10.
           05  NUM-MAX                       PIC 9999 VALUE 16.
           05  VALUE-MAX                     PIC 9999 VALUE 40.
           05  PIC-LNG                       PIC 9999 VALUE 18.
           05  FILE-NAME-LNG                 PIC 9999 VALUE 16.
           05  SOURCE-LINE-MAX               PIC 9999 VALUE 10.
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  END-SW                        PIC 9    VALUE 0.
           05  OPEN-SW                       PIC 9    VALUE 0.
           05  SIGN-SW                       PIC 9    VALUE 0.
           05  COMMA-SW                      PIC 9    VALUE 0.
           05  FOUND-SW                      PIC 9    VALUE 0.
           05  VALUE-SW                      PIC 9    VALUE 0.
           05  COMMENT-SW                    PIC 9    VALUE 0.
      *
      *****************************************************************
      *  XML DOCUMENT                                                 *
      *****************************************************************
       01  XML-LINES.
           05  XML-LINE                          PIC X(92).
           05  XML-CHARACTERS    REDEFINES XML-LINE.
               10  XML-CHAR                      PIC X OCCURS 92 TIMES.
      *
      ****************************************************************
      *  XML Data Description Outputs                                 *
      *****************************************************************
           05  XML-HEADER-LINE.
               10  XML-HEADER                    PIC X(40) VALUE
                  '<?xml version="1.0"?>                   '.
               10  FILLER                        PIC X(32).
           05  XSL-HEADER-LINE.
               10  XSL-HEADER                    PIC X(40) VALUE
                  '<?xsl:stylesheet version = "1.0"?>      '.
               10  FILLER                        PIC X(32).
      *
TEST***    COPY XMLCOB.
      *---------------------------------------------------------------*
      *    COBOL/XML Document NameSpace                               *
      *---------------------------------------------------------------*
      * XML/COBOL Processing Commands
      *
       01  XML-CONSTANTS.
           05 XML-PROC-NR         PIC 999       VALUE 2.
           05 XML-COMM-NR         PIC 999       VALUE 4.
           05 XML-TYPE-NR         PIC 999       VALUE 4.
       01  XML-PROC-CMDS.
           05 XML-PROC-1        PIC X(50)       VALUE
             '<?xml version = "1.0" encoding = "ISO-8859-1"?>   '.
           05 XML-PROC-2        PIC X(50)       VALUE
             '                                        '.
           05 XML-PROC-3        PIC X(50)       VALUE
             '                                        '.
           05 XML-PROC-4        PIC X(50)       VALUE
             '                                        '.
       01  XML-PROC-LINES  REDEFINES XML-PROC-CMDS.
           05 XML-PROC-LINE     PIC X(50)       OCCURS 4 TIMES.
      *---------------------------------------------------------------*
      * XML/COBOL Comment Lines
      *
       01  XML-COMMENTS.
           05 XML-COMM-1        PIC X(50)       VALUE
             '<!--This Document was generated from COBOL Progam:'.
           05 XML-COMM-2        PIC X(50)       VALUE
             '              by the Sneed Tool XM059O    on date:'.
           05 XML-COMM-2        PIC X(50)       VALUE
             '                                                  '.
           05 XML-COMM-4        PIC X(50)       VALUE
             '  -->                                             '.
       01  XML-COMM-LINES  REDEFINES XML-COMMENTS.
           05 XML-COMM-LINE     PIC X(50)       OCCURS 4 TIMES.
      *---------------------------------------------------------------*
      * XML/COBOL Schema Elements
      *
       01  XML-ELEMENT-TYPES.
           05 XML-DOCTYPE       PIC X(8)        VALUE 'DOCTYPE '.
           05 XML-SYSTEM        PIC X(8)        VALUE ' SYSTEM '.
           05 FILLER            PIC X(8)        VALUE '        '.
           05 FILLER            PIC X(8)        VALUE '        '.
       01  XML-ELEMENT-TYPEN REDEFINES XML-ELEMENT-TYPES.
           05 XML-COBOL-TYPE    PIC X(8)        OCCURS 4 TIMES.
      *---------------------------------------------------------------*
      *
      *****************************************************************
      *  XSD SCHEMA                                                   *
      *****************************************************************
       01  XSD-LINES.
           05  XSD-LINE                          PIC X(92).
           05  XSD-CHARACTERS    REDEFINES XSD-LINE.
               10  XSD-CHAR                      PIC X OCCURS 92 TIMES.
      *
      *****************************************************************
      *
TEST***    COPY XSDCOB.
      *---------------------------------------------------------------*
      *    COBOL/XSD Schema Namespace                                 *
      *---------------------------------------------------------------*
      * XSD/COBOL Processing Commands
      *
       01  XSD-CONSTANTS.
           05 X-PROC-NR           PIC 999       VALUE 2.
           05 X-TYPE-NR           PIC 999       VALUE 14.
           05 X-CONTENT-NR        PIC 999       VALUE 4.
           05 X-MODEL-NR          PIC 999       VALUE 2.
           05 X-OCCUR-NR          PIC 999       VALUE 4.
       01  XSD-PROC-CMDS.
           05 X-PROC-1          PIC X(40)       VALUE
             '<?xml version = "1.0">                  '.
           05 X-PROC-2          PIC X(40)       VALUE
             '                                        '.
           05 X-PROC-3          PIC X(40)       VALUE
             '                                        '.
           05 X-PROC-4          PIC X(40)       VALUE
             '                                        '.
       01  XSD-PROC-LINES  REDEFINES XSD-PROC-CMDS.
           05 X-PROC-LINE       PIC X(40)       OCCURS 4 TIMES.
      *---------------------------------------------------------------*
      * XSD/COBOL Comment Lines
      *
       01  XSD-COMMENTS.
           05 X-COMM-1          PIC X(40)       VALUE
             '<!--This schema was generated from prog:'.
           05 X-COMM-2          PIC X(40)       VALUE
             '    by the Sneed Tool GENSCHEMA on date:'.
           05 X-COMM-3          PIC X(40)       VALUE
             '  -->                                   '.
      *---------------------------------------------------------------*
      * XSD/COBOL Schema Elements
      *
       01  XSD-ELEMENT-TYPES.
           05 X-SCHEMA          PIC X(8)        VALUE 'schema  '.
           05 X-SCHEMA-END      PIC X(8)        VALUE '/schema '.
           05 X-SCHEMA-ATTR-1   PIC X(8)        VALUE ' name = '.
           05 X-SCHEMA-NAME     PIC X(40)       VALUE
             'MySchema                                '.
           05 X-SCHEMA-ATTR-2   PIC X(8)        VALUE ' xmlns= '.
           05 X-SCHEMA-NS       PIC X(8)        VALUE 'XSDCOB  '.
           05 X-SCHEMA-STRU     PIC X(40)       VALUE
             'XSDCOB:complexType                      '.
           05 X-SCHEMA-STRU-END PIC X(40)       VALUE
             '/XSDCOB:complexType                     '.
           05 X-SCHEMA-ITEM     PIC X(40)       VALUE
             'XSDCOB:element                          '.
      *---------------------------------------------------------------*
      * XSD/COBOL Schema Attributes
      *
       01  XSD-ATTRIBUTES.
           05 X-ATTR-TYPE       PIC X(8)        VALUE ' type = '.
           05 X-ATTR-NAME       PIC X(8)        VALUE ' name = '.
           05 X-ATTR-CONTENT    PIC X(11)       VALUE ' content = '.
           05 X-ATTR-LEVEL      PIC X(9)        VALUE ' level = '.
           05 X-ATTR-MODEL      PIC X(9)        VALUE ' model = '.
           05 X-ATTR-OCCURS     PIC X(10)       VALUE ' occurs = '.
           05 X-ATTR-MIN        PIC X(13)       VALUE ' minOccurs = '.
           05 X-ATTR-MAX        PIC X(13)       VALUE ' maxOccurs = '.
           05 X-ATTR-PIC        PIC X(7)        VALUE ' pic = '.
           05 X-ATTR-LNG        PIC X(7)        VALUE ' lng = '.
           05 X-ATTR-POS        PIC X(7)        VALUE ' pos = '.
           05 X-ATTR-USE        PIC X(9)        VALUE ' usage = '.
           05 X-ATTR-REDEF      PIC X(9)        VALUE ' redef = '.
           05 X-ATTR-VALUE      PIC X(9)        VALUE ' value = '.
           05 X-ATTR-ADJUST     PIC X(10)       VALUE ' adjust = '.
      *---------------------------------------------------------------*
      * XSD/COBOL Schema Attribute Types
      *
       01  XSD-COBOL-TYPEN.
           05 X-TYPE-FILE       PIC X(8)        VALUE '#file   '.
           05 X-TYPE-REC        PIC X(8)        VALUE '#record '.
           05 X-TYPE-MAP        PIC X(8)        VALUE '#map    '.
           05 X-TYPE-VIEW       PIC X(8)        VALUE '#view   '.
           05 X-TYPE-GROUP      PIC X(8)        VALUE '#group  '.
           05 X-TYPE-CHAR       PIC X(8)        VALUE '#char   '.
           05 X-TYPE-DEC        PIC X(8)        VALUE '#dec    '.
           05 X-TYPE-PACK       PIC X(8)        VALUE '#packed '.
           05 X-TYPE-BIN        PIC X(8)        VALUE '#binary '.
           05 X-TYPE-FLOAT      PIC X(8)        VALUE '#float  '.
           05 X-TYPE-EDIT       PIC X(8)        VALUE '#edit   '.
           05 X-TYPE-BIT        PIC X(8)        VALUE '#bit    '.
           05 X-TYPE-PTR        PIC X(8)        VALUE '#pointer'.
           05 X-TYPE-HEX        PIC X(8)        VALUE '#hex    '.
       01  XSD-COBOL-TYPES REDEFINES XSD-COBOL-TYPEN.
           05 XSD-COBOL-TYPE    PIC X(8)        OCCURS 14 TIMES.
      *---------------------------------------------------------------*
      * XSD/COBOL Schema Content Types
      *
       01  XSD-COBOL-CONTS.
           05 X-ELEMENTS        PIC X(8)        VALUE 'eltOnly '.
           05 X-TEXT            PIC X(8)        VALUE 'TextOnly'.
           05 X-MIXED           PIC X(8)        VALUE 'mixed   '.
           05 X-ANY             PIC X(8)        VALUE 'any     '.
       01  XSD-COBOL-CONTENTS REDEFINES XSD-COBOL-CONTS.
           05 XSD-COBOL-CONTENT PIC X(8)        OCCURS 4 TIMES.
      *---------------------------------------------------------------*
      * XSD/COBOL Schema Models
      *
       01  XSD-COBOL-MODELLE.
           05 X-OPEN            PIC X(8)        VALUE 'open    '.
           05 X-CLOSED          PIC X(8)        VALUE 'closed  '.
       01  XSD-COBOL-MODELS REDEFINES XSD-COBOL-MODELLE.
           05 XSD-COBOL-MODEL   PIC X(8)        OCCURS 2 TIMES.
      *---------------------------------------------------------------*
      * XSD/COBOL Schema Occurs
      *
       01  XSD-COBOL-OCCURANCES.
           05 X-OPTIONAL        PIC X(10)       VALUE 'OPTIONAL'.
           05 X-ZEROTON         PIC X(10)       VALUE 'ZEROORMORE'.
           05 X-ONETON          PIC X(10)       VALUE 'ONEORMORE'.
           05 X-MTON            PIC X(10)       VALUE '         '.
       01  XSD-COBOL-OCCURS REDEFINES XSD-COBOL-OCCURANCES.
           05 XSD-COBOL-OCCUR   PIC X(10)       OCCURS 4 TIMES.
      *---------------------------------------------------------------*
      *
      ****************************************************************
      *    Source Output Statements
      *        Indexed by SOURCE-INDEX Limited by LINES-MAX
      *
       01  SOURCE-STMT.
           05  SOURCE-LINES.
               10  SOURCE-LINE               PIC X(92)
                                             OCCURS 10 TIMES.
           05  SOURCE-STRING                 REDEFINES SOURCE-LINES.
      *            Indexed by SOURCE-PTR Limited by END-PTR
               10  SOURCE-CHAR               PIC X
                                             OCCURS 920 TIMES.
      ****************************************************************
       01  GLOBAL-DATA.
           05  REC-NR                            PIC 999 VALUE ZERO.
           05  FIELD-NR                          PIC 999 VALUE ZERO.
           05  CODE-LINE                         PIC X(120).
           05  CODE-STRING           REDEFINES   CODE-LINE.
               10  CODE-CHAR                     PIC X OCCURS 120 TIMES.
           05  DATA-NAME                         PIC X(30).
           05  NAME-STRING           REDEFINES   DATA-NAME.
               10  NAME-CHAR                     PIC X OCCURS 30  TIMES.
           05  FIELD-VALUE                       PIC X(80).
           05  VALUE-STRING          REDEFINES   FIELD-VALUE.
               10  VALUE-CHAR                    PIC X OCCURS 80  TIMES.
           05  REC-VALUE                         PIC X(2048).
           05  REC-STRING            REDEFINES   REC-VALUE.
               10  REC-CHAR                      PIC X OCCURS 2048 TIMES.
      ****************************************************************
       01  LOCAL-DATA.
           05  NEST-LEV                          PIC 99.
           05  STACK-NAME                        PIC X(30).
           05  SET-VALUE-DATA.
               10  FIELD-NAME                    PIC X(30).
               10  FIELD-PICT                    PIC X(18).
               10  FIELD-LEV                     PIC 99.
               10  FIELD-POS                     PIC 9999 BINARY.
               10  FIELD-LNG                     PIC 9999 BINARY.
               10  FIELD-TYPE                    PIC X.
                   88  TYPE-CHAR                 VALUE 'X'.
                   88  TYPE-ALPHA                VALUE 'A'.
                   88  TYPE-DECIMAL              VALUE 'D'.
                   88  TYPE-PACKED               VALUE 'P'.
                   88  TYPE-FLOAT                VALUE 'F'.
                   88  TYPE-BINARY               VALUE 'B'.
                   88  TYPE-POINTER              VALUE 'T'.
      ****************************************************************
           05  CONVERT-NUMERIC-DATA.
               10  PICTURE-VALUE                 PIC X(18).
               10  PICTURE-STRING REDEFINES PICTURE-VALUE.
                   15  PIC-CHAR                  PIC X OCCURS 18 TIMES.
               10  DECIMAL-VALUE                 PIC 9(16).
               10  DECIMAL-STRING REDEFINES DECIMAL-VALUE.
                   15  DEC-CHAR                  PIC X OCCURS 16 TIMES.
               10  PACKED-VALUE                  PIC 9(16) COMP-3.
               10  SIGNED-VALUE                  PIC S9(15).
               10  NR-DIGITS                     PIC 99.
               10  PRE-DIGITS                    PIC 99.
               10  POST-DIGITS                   PIC 99.
               10  SIG-DIGITS                    PIC 99.
               10  LEAD-ZEROS                    PIC 99.
               10  NUM-LNG                       PIC 99.
               10  PICT-NUMS                     PIC XX.
               10  PICT-NUM REDEFINES PICT-NUMS  PIC 99.
      ****************************************************************
               10  NUMERIC-VALUE                 PIC X(16).
               10  NUMERIC-STRING REDEFINES NUMERIC-VALUE.
                   15  NUM-CHAR                  PIC X OCCURS 16 TIMES.
      *    UnSigned Binary Conversion Fields
               10  SHORT-VALUE        REDEFINES NUMERIC-VALUE
                                                 PIC 9(4)  BINARY.
               10  LONG-VALUE         REDEFINES NUMERIC-VALUE
                                                 PIC 9(8)  BINARY.
      *    Signed Binary Conversion Fields
               10  SHORT-VALUE-S      REDEFINES NUMERIC-VALUE
                                                 PIC S9(4) BINARY.
               10  LONG-VALUE-S       REDEFINES NUMERIC-VALUE
                                                 PIC S9(8) BINARY.
      *    UnSigned Packed Conversion Fields
               10  PACKED-VALUE-2     REDEFINES NUMERIC-VALUE
                                                 PIC 9(3)   COMP-3.
               10  PACKED-VALUE-3     REDEFINES NUMERIC-VALUE
                                                 PIC 9(5)   COMP-3.
               10  PACKED-VALUE-4     REDEFINES NUMERIC-VALUE
                                                 PIC 9(7)   COMP-3.
               10  PACKED-VALUE-5     REDEFINES NUMERIC-VALUE
                                                 PIC 9(9)   COMP-3.
               10  PACKED-VALUE-6     REDEFINES NUMERIC-VALUE
                                                 PIC 9(11)  COMP-3.
               10  PACKED-VALUE-7     REDEFINES NUMERIC-VALUE
                                                 PIC 9(13)  COMP-3.
               10  PACKED-VALUE-8     REDEFINES NUMERIC-VALUE
                                                 PIC 9(15)  COMP-3.
               10  PACKED-VALUE-2-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(3)  COMP-3.
               10  PACKED-VALUE-3-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(5)  COMP-3.
               10  PACKED-VALUE-4-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(7)  COMP-3.
               10  PACKED-VALUE-5-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(9)  COMP-3.
               10  PACKED-VALUE-6-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(11) COMP-3.
               10  PACKED-VALUE-7-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(13) COMP-3.
               10  PACKED-VALUE-8-S   REDEFINES NUMERIC-VALUE
                                                 PIC S9(15) COMP-3.
      ****************************************************************
           05  CONVERT-SCHEMA-DATA.
               10  FELD-POS-X                    PIC XXXX.
               10  FELD-POS-N   REDEFINES FELD-POS-X
                                                 PIC 9999.
               10  FELD-LNG-X                    PIC XXXX.
               10  FELD-LNG-N   REDEFINES FELD-LNG-X
                                                 PIC 9999.
               10  FELD-DIM-X                    PIC XXXX.
               10  FELD-DIM-N   REDEFINES FELD-DIM-X
                                                 PIC 9999.
      ****************************************************************
       01  NESTING-TAB.
           05  NEST-GROUP                        OCCURS 10 TIMES.
               10  NEST-LEVEL                    PIC 99.
               10  NEST-NAME                     PIC X(30).
      ****************************************************************
      *    Record Description is a copy member
TEST***    COPY DATATAB.
      ****************************************************************
      *    Record Description is a copy member
       01  DATATAB.
           05  FIL-NAME                          PIC X(30).
           05  REC-NAME                          PIC X(30).
           05  FELD-NR                           PIC 999 VALUE ZERO.
           05  FELD                              OCCURS 200 TIMES.
               10  FELD-NAME                     PIC X(30).
               10  FELD-LEV                      PIC 99.
               10  FELD-TYP                      PIC XX.
               10  FELD-POS                      PIC 9999.
               10  FELD-LNG                      PIC 9999.
               10  FELD-DIM                      PIC 9999.
               10  FELD-PIC                      PIC X(18).
               10  FELD-DEF                      PIC X(30).
      *
      ****************************************************************
      *
      ****************************************************************
TEST   LINKAGE SECTION.
      *                          Parameters
       77  XML-FUNCODE                       PIC XX.
       77  XML-RETCODE                       PIC XX.
       77  XML-FILNAME                       PIC X(8).
       77  XML-METHOD                        PIC X(30).
      *    Data Record passed back to the Caller
       77  DATA-REC-LNG                      PIC 9(8) USAGE BINARY.
       01  DATA-REC                          PIC X(2048).
       01  DATA-CHARS         REDEFINES DATA-REC.
           05 DATA-CHAR                      PIC X  OCCURS 2048 TIMES
                                             DEPENDING ON REC-LNG.
      *
      ******************************************************************
      *
      /
TEST   PROCEDURE DIVISION USING  XML-FUNCODE,
                                 XML-RETCODE,
                                 XML-FILNAME,
                                 XML-METHOD,
                                 DATA-REC-LNG,
                                 DATA-REC.
TEST  *PROCEDURE DIVISION.
      ******************************************************************
      *     Main  Routine     ******************************************
      ******************************************************************
       XMLSTUB-CONTROL SECTION.
      ******************************************************************
       XML-STUB-CONTROL.
      ***********************
           IF OPEN-SW = ZERO
              DISPLAY '+-------------------------------------------+'
              DISPLAY '| XM059O   Program Version 1.2 of July 2002 |'
              DISPLAY '+-------------------------------------------+'
              DISPLAY '| XML Output File Name:' XML-FILNAME
TEST***       DISPLAY 'DataRec:' DATA-REC (1:40)
TEST***       ACCEPT  CONTINUE-KEY
              PERFORM CREATE-DATA-TAB
              PERFORM OPEN-XML-FILE
              MOVE 1 TO OPEN-SW
           END-IF.
           EVALUATE XML-FUNCODE
           WHEN 'RD'
              MOVE '90' TO XML-RETCODE
           WHEN 'WR'
              PERFORM CREATE-XML-RECORD
              PERFORM GENERATE-XML-DOCU
              PERFORM TERM-XML-RECORD
              PERFORM CLOSE-XML-FILE
           WHEN 'CL'
              PERFORM CLOSE-XML-FILE
              MOVE ZERO TO OPEN-SW
           WHEN OTHER
              MOVE '99' TO XML-RETCODE
              PERFORM CLOSE-XML-FILE
              MOVE ZERO TO OPEN-SW
           END-EVALUATE.
TEST***    DISPLAY 'XM059O   completed '
TEST***    ACCEPT  CONTINUE-KEY
           GOBACK.
      ******************************************************************
       CREATE-XML-RECORD.
      *****************************************************
           MOVE SPACES TO REC-STRING.
           MOVE 1 TO REC-PTR
           PERFORM UNTIL REC-PTR > REC-LNG
                      OR REC-PTR > REC-MAX
              MOVE DATA-CHAR (REC-PTR) TO REC-CHAR (REC-PTR)
              ADD 1 TO REC-PTR
           END-PERFORM.
           ADD 1 TO REC-NR.
           PERFORM WRITE-DOC-RECORD.
      *****************************************************
      *    Initialise the Group Nesting Table
           MOVE 1 TO NEST-INDEX
           MOVE 1 TO NEST-LEVEL (NEST-INDEX).
           MOVE REC-NAME TO NEST-NAME (NEST-INDEX).
TEST***    DISPLAY 'Data-Rec: ' REC-STRING (1:REC-LNG)
TEST***    DISPLAY 'Record Received  '
TEST***    DISPLAY 'Press Enter to continue '
TEST***    ACCEPT CONTINUE-KEY.
      ******************************************************************
      *     Sub Routines      ******************************************
      ******************************************************************
       XML-CONVERSION  SECTION.
      *****************************************************
       GENERATE-XML-DOCU.
      ***********************
TEST***    DISPLAY 'Generate XML Docu'
TEST***    DISPLAY 'Feld Nr = ' FELD-NR
TEST***    ACCEPT CONTINUE-KEY
           MOVE 1 TO I.
           PERFORM UNTIL I > FELD-NR
              MOVE SPACES TO FIELD-VALUE
              MOVE FELD-NAME(I) TO FIELD-NAME
              MOVE FELD-POS (I) TO FIELD-PTR
              MOVE FELD-LEV (I) TO FIELD-LEV
              MOVE FELD-LNG (I) TO FIELD-LNG
              MOVE FELD-TYP (I) TO FIELD-TYPE
              MOVE FELD-PIC (I) TO FIELD-PICT
              IF FIELD-TYPE NOT = SPACE
                 MOVE REC-VALUE (FIELD-PTR:FIELD-LNG)
                   TO FIELD-VALUE (1:FIELD-LNG)
                 MOVE FIELD-VALUE (1:FIELD-LNG) TO SIGNED-VALUE
TEST***          DISPLAY 'Signed Value:' SIGNED-VALUE
                 IF FIELD-TYPE = 'D'
                    MOVE FIELD-VALUE (1:FIELD-LNG) TO NUMERIC-VALUE
                 END-IF
                 IF FIELD-TYPE = 'B' OR 'I'
                    PERFORM CONVERT-BINARY
                 END-IF
                 IF FIELD-TYPE = 'F'
                    PERFORM CONVERT-BINARY
                 END-IF
                 IF FIELD-TYPE = 'P'
                    PERFORM CONVERT-PACKED
                 END-IF
                 IF FIELD-TYPE = 'D' OR 'B' OR 'I' OR 'P' OR 'F'
                    PERFORM ADJUST-NUMERIC-VALUE
                 END-IF
SN0303           IF FIELD-NAME = 'RETURN-CODE'
SN0303           OR FIELD-NAME = 'STATUS-CODE'
SN0303           OR FIELD-NAME = 'PF-KEY'
SN0303              MOVE XML-RETCODE TO FIELD-VALUE (1:FIELD-LNG)
SN0303           END-IF
SN0209           IF FIELD-NAME = 'NEXT-METHOD'
SN0209              MOVE XML-METHOD TO FIELD-VALUE (1:FIELD-LNG)
SN0209           END-IF
TEST***          DISPLAY 'Field-Value= ' FIELD-VALUE
TEST***          ACCEPT CONTINUE-KEY
              END-IF
      *** pop   group levels
              IF FIELD-LEV NOT > NEST-LEVEL (NEST-INDEX)
                 PERFORM WRITE-DOC-GROUP-END
                 MOVE 99 TO NEST-LEVEL (NEST-INDEX)
                 MOVE SPACES TO NEST-NAME (NEST-INDEX)
                 SUBTRACT 1 FROM NEST-INDEX
                 PERFORM UNTIL FIELD-LEV NOT > NEST-LEVEL (NEST-INDEX)
                            OR NEST-INDEX = 1
                    PERFORM WRITE-DOC-GROUP-END
                    MOVE 99 TO NEST-LEVEL (NEST-INDEX)
                    MOVE SPACES TO NEST-NAME (NEST-INDEX)
                   SUBTRACT 1 FROM NEST-INDEX
                 END-PERFORM
              END-IF
      *** push  group levels
              IF FIELD-TYPE = SPACES
                 PERFORM WRITE-DOC-GROUP
                 ADD 1 TO NEST-INDEX
                 MOVE FIELD-LEV TO NEST-LEVEL (NEST-INDEX)
                 MOVE FIELD-NAME TO NEST-NAME (NEST-INDEX)
              ELSE
      *** place element
                 PERFORM WRITE-DOC-FIELD
              END-IF
              ADD 1 TO I
           END-PERFORM.
      *****************************************************
       TERM-XML-RECORD.
      ***********************
      * write remaining end tags
      ***********************
BUG********SUBTRACT 1 FROM NEST-INDEX
           IF NEST-INDEX > 1
              PERFORM UNTIL NEST-INDEX = 1
                 PERFORM WRITE-DOC-GROUP-END
                 MOVE 99 TO NEST-LEVEL (NEST-INDEX)
                 MOVE SPACES TO NEST-NAME (NEST-INDEX)
                 SUBTRACT 1 FROM NEST-INDEX
              END-PERFORM
           END-IF.
           PERFORM WRITE-DOC-RECORD-END.
      *****************************************************
       NUMERIC-CONVERSION SECTION.
      ******************************************************************
       CONVERT-PACKED.
      ***********************
      * convert packed decimal to display
      ***********************
           MOVE ZEROES TO DECIMAL-VALUE
           MOVE SPACES TO NUMERIC-VALUE
           EVALUATE FIELD-LNG
              WHEN 1
              WHEN 2
                 MOVE FIELD-VALUE (1:2) TO NUMERIC-VALUE (1:2)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-2-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-2 TO DECIMAL-VALUE
                 END-IF
              WHEN 3
                 MOVE FIELD-VALUE (1:3) TO NUMERIC-VALUE (1:3)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-3-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-3 TO DECIMAL-VALUE
                 END-IF
              WHEN 4
                 MOVE FIELD-VALUE (1:4) TO NUMERIC-VALUE (1:4)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-4-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-4 TO DECIMAL-VALUE
                 END-IF
              WHEN 5
                 MOVE FIELD-VALUE (1:5) TO NUMERIC-VALUE (1:5)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-5-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-5 TO DECIMAL-VALUE
                 END-IF
              WHEN 6
                 MOVE FIELD-VALUE (1:6) TO NUMERIC-VALUE (1:6)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-6-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-6 TO DECIMAL-VALUE
                 END-IF
              WHEN 7
                 MOVE FIELD-VALUE (1:7) TO NUMERIC-VALUE (1:7)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-7-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-7 TO DECIMAL-VALUE
                 END-IF
              WHEN 8
                 MOVE FIELD-VALUE (1:8) TO NUMERIC-VALUE (1:8)
                 IF SIGN-SW > ZERO
                    MOVE PACKED-VALUE-8-S TO DECIMAL-VALUE
                 ELSE
                    MOVE PACKED-VALUE-8 TO DECIMAL-VALUE
                 END-IF
           END-EVALUATE.
           MOVE DECIMAL-VALUE TO NUMERIC-VALUE.
RETURN* Packed Numeric Value
TEST*** DISPLAY 'Packed Field Value=' NUMERIC-VALUE.
TEST*** DISPLAY 'DecimalField Value=' DECIMAL-VALUE.
TEST*** ACCEPT CONTINUE-KEY.
      *****************************************************
       CONVERT-BINARY.
      ***********************
      * convert binary field to display
      ***********************
           MOVE ZEROES TO DECIMAL-VALUE
           IF FIELD-LNG = 2
              MOVE FIELD-VALUE (1:2) TO NUMERIC-VALUE (1:2)
              IF SIGN-SW > ZERO
                 MOVE SHORT-VALUE-S TO DECIMAL-VALUE
              ELSE
                 MOVE SHORT-VALUE   TO DECIMAL-VALUE
              END-IF
           ELSE
              MOVE FIELD-VALUE (1:4) TO NUMERIC-VALUE (1:4)
              IF SIGN-SW > ZERO
                 MOVE LONG-VALUE-S TO DECIMAL-VALUE
              ELSE
                 MOVE LONG-VALUE   TO DECIMAL-VALUE
              END-IF
           END-IF.
           MOVE DECIMAL-VALUE TO NUMERIC-VALUE.
RETURN* Binary Field-Value String
TEST*** DISPLAY 'Binary Field-Value=' NUMERIC-VALUE.
TEST*** DISPLAY 'DecimalField Value=' DECIMAL-VALUE.
      *****************************************************
       ADJUST-NUMERIC-VALUE.
      ***********************
      * extract decimal positions & sign from picture
      ***********************
TEST***    DISPLAY 'Adjust Numeric Value'
TEST***    DISPLAY 'Field Name = ' FIELD-NAME
TEST***    DISPLAY 'Field-Pict = ' FIELD-PICT
           MOVE FIELD-PICT TO PICTURE-VALUE
           MOVE ZERO TO NR-DIGITS
           MOVE ZERO TO PRE-DIGITS
           MOVE ZERO TO POST-DIGITS
           MOVE ZERO TO COMMA-SW
           MOVE ZERO TO SIGN-SW
           MOVE 1 TO J
           PERFORM UNTIL J > PIC-LNG
                      OR PIC-CHAR (J) = SPACE
              IF PIC-CHAR (J) = 'S'
                 MOVE 1 TO SIGN-SW
              END-IF
              IF PIC-CHAR (J) = 'V'
                 MOVE 1 TO COMMA-SW
              END-IF
              IF COMMA-SW = 1
                 IF PIC-CHAR (J) = '9'
                    ADD 1 TO POST-DIGITS
                    ADD 1 TO NR-DIGITS
                 END-IF
                 IF PIC-CHAR (J) = '('
                    ADD 1 TO J
                    IF  PIC-CHAR (J) NUMERIC
                    AND PIC-CHAR (J + 1) NUMERIC
                       MOVE PICTURE-VALUE (J:2) TO PICT-NUMS
                       ADD 2 TO J
                    ELSE
                       MOVE PICTURE-VALUE (J:1) TO PICT-NUMS
                       ADD 1 TO J
                    END-IF
                    MOVE PICT-NUM TO NR-DIGITS
                    MOVE PICT-NUM TO POST-DIGITS
                 END-IF
              ELSE
                 IF PIC-CHAR (J) = '9'
                    ADD 1 TO PRE-DIGITS
                    ADD 1 TO NR-DIGITS
                 END-IF
                 IF PIC-CHAR (J) = '('
                    ADD 1 TO J
                    IF  PIC-CHAR (J) NUMERIC
                    AND PIC-CHAR (J + 1) NUMERIC
                       MOVE PICTURE-VALUE (1:2) TO PICT-NUMS
                       ADD 2 TO J
                    ELSE
                       MOVE PICTURE-VALUE (J:1) TO PICT-NUMS (2:1)
                       MOVE ZERO TO PICT-NUMS (1:1)
                       ADD 1 TO J
                    END-IF
                    MOVE PICT-NUM TO NR-DIGITS
                    MOVE PICT-NUM TO PRE-DIGITS
                 END-IF
              END-IF
              ADD 1 TO J
           END-PERFORM.
TEST***    DISPLAY ' J = ' J
TEST***    DISPLAY 'Nr-Digits=' NR-DIGITS
TEST***    DISPLAY 'Nr-Post-Digits=' POST-DIGITS
TEST***    DISPLAY 'Nr-Pre-Digits=' PRE-DIGITS
TEST***    ACCEPT CONTINUE-KEY.
      ****************************************************************
      *    Adjust the Value according to Picture                     *
      ****************************************************************
           IF FIELD-TYPE = 'D'
              MOVE FIELD-LNG TO SIG-DIGITS
           ELSE
      *    count leading zeroes
              MOVE ZERO TO LEAD-ZEROS
              MOVE 1 TO SIG-DIGITS
              MOVE 1 TO J
              IF NUM-CHAR (J) = '+' OR '-'
                 ADD 1 TO J
              END-IF
              PERFORM UNTIL DEC-CHAR (J) NOT = ZERO
                         OR J > NUM-MAX
                 ADD 1 TO J
              END-PERFORM
TEST***       DISPLAY 'Significant Dec Value:' DECIMAL-STRING (J:5)
              MOVE J TO LEAD-ZEROS
              COMPUTE SIG-DIGITS = NUM-MAX - LEAD-ZEROS + 1
           END-IF.
      *    transfer digits from right to left
           MOVE ZEROS TO DECIMAL-VALUE
           MOVE ZERO TO NUM-LNG
           MOVE NUM-MAX TO K, J
           PERFORM UNTIL J = ZERO
                      OR NUM-LNG > SIG-DIGITS
              IF NUM-CHAR (J) NUMERIC
                 MOVE NUM-CHAR (J) TO DEC-CHAR (K)
                 SUBTRACT 1 FROM K
                 ADD 1 TO NUM-LNG
              END-IF
      *    Insert decimal point
              IF COMMA-SW = 1
                 IF NUM-LNG = POST-DIGITS
                    MOVE '.' TO DEC-CHAR (K)
                    SUBTRACT 1 FROM K
                    ADD 1 TO NUM-LNG
                 END-IF
              END-IF
              SUBTRACT 1 FROM J
           END-PERFORM.
           ADD 1 TO K.
      *    Insert sign
           IF SIGN-SW > ZERO AND K > 1
              SUBTRACT 1 FROM K
              IF SIGNED-VALUE < ZERO
                 MOVE '-' TO DEC-CHAR (K)
              ELSE
                 MOVE '+' TO DEC-CHAR (K)
              END-IF
              ADD 1 TO NUM-LNG
           END-IF.
           MOVE DECIMAL-VALUE (K:NUM-LNG) TO FIELD-VALUE.
RETURN*    Numeric Field Value
TEST***    DISPLAY 'Numeric Field Value=' FIELD-VALUE.
      *****************************************************************
      *****************************************************************
       XML-FILE-HANDLING SECTION.
      *****************************************************************
       OPEN-XML-FILE.
TEST***    DISPLAY 'Open XML File '
           MOVE SPACES TO OUTPUT-FILE
           STRING 'WORKDIR\'     DELIMITED BY SIZE
                  XML-FILNAME    DELIMITED BY SPACE
                  '.xml'         DELIMITED BY SIZE
              INTO OUTPUT-FILE
           END-STRING.
           MOVE ZEROES TO OUTPUT-STAT
           OPEN OUTPUT XML-FILE.
           MOVE OUTPUT-STAT TO XML-RETCODE.
           PERFORM WRITE-DOC-HEADER
           PERFORM WRITE-DOC-FILE.
TEST***    DISPLAY 'Output File = ' OUTPUT-FILE
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************************
       CLOSE-XML-FILE.
TEST***    DISPLAY 'Close XML File '
           PERFORM WRITE-DOC-FILE-END
           CLOSE XML-FILE.
      *****************************************************************
       DOCUMENT-GENERATION SECTION.
      *****************************************************************
       WRITE-DOC-HEADER.
      *  write out the XML document header
           MOVE SPACES TO SOURCE-STMT.
           MOVE ZERO TO NEST-INDEX
           MOVE 1 TO J.
           MOVE XML-PROC-1 TO SOURCE-LINE (J)
           ADD 1 TO J.
           STRING '<!--'              DELIMITED BY SIZE
                  XML-DOCTYPE         DELIMITED BY SIZE
                  ' '                 DELIMITED BY SIZE
                  XML-FILNAME         DELIMITED BY SPACE
                  ' '                 DELIMITED BY SIZE
                  XML-SYSTEM          DELIMITED BY SIZE
                  '"'                 DELIMITED BY SIZE
                  XML-FILNAME         DELIMITED BY SPACE
                  '.xsd'              DELIMITED BY SIZE
                  '"'                 DELIMITED BY SIZE
                  '-->'               DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING
           MOVE J TO SOURCE-INDEX.
           MOVE ZERO TO NEST-PTR.
           PERFORM WRITE-SOURCE-STMT.
      ******************************************************************
       WRITE-DOC-FILE.
      *  write out file tag
           MOVE SPACES TO SOURCE-STMT.
           MOVE ZERO TO NEST-INDEX
           MOVE 1 TO J.
           STRING '<'                 DELIMITED BY SIZE
                  XML-FILNAME         DELIMITED BY SPACE
                  '>'                 DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE J TO SOURCE-INDEX.
           MOVE ZERO TO NEST-PTR.
           PERFORM WRITE-SOURCE-STMT.
      ******************************************************************
       WRITE-DOC-FILE-END.
      *  write out file end tag
           MOVE SPACES TO SOURCE-STMT.
           MOVE ZERO TO NEST-INDEX
           MOVE 1 TO J.
           STRING '</'                DELIMITED BY SIZE
                  XML-FILNAME         DELIMITED BY SPACE
                  '>'                 DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE J TO SOURCE-INDEX.
           MOVE ZERO TO NEST-PTR.
           PERFORM WRITE-SOURCE-STMT.
      ******************************************************************
       WRITE-DOC-RECORD.
      *  write out record tag
           MOVE SPACES TO SOURCE-STMT.
           MOVE 1 TO J.
           STRING '<'                 DELIMITED BY SIZE
                  REC-NAME            DELIMITED BY SPACE
                  '>'                 DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE J TO SOURCE-INDEX.
           MOVE 1 TO NEST-PTR.
           PERFORM WRITE-SOURCE-STMT.
           ADD  1 TO NEST-PTR.
      ******************************************************************
       WRITE-DOC-RECORD-END.
      *  write out record end tag
           MOVE SPACES TO SOURCE-STMT.
           MOVE 1 TO J.
           STRING '</'                DELIMITED BY SIZE
                  REC-NAME            DELIMITED BY SPACE
                  '>'                 DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE 1 TO NEST-PTR.
           MOVE J TO SOURCE-INDEX.
           PERFORM WRITE-SOURCE-STMT.
      ******************************************************************
       WRITE-DOC-GROUP.
      *  write out group tag
           MOVE SPACES TO SOURCE-STMT.
           MOVE 1 TO J.
           STRING '<'                      DELIMITED BY SIZE
                  FIELD-NAME               DELIMITED BY SPACE
                  '>'                      DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE J TO SOURCE-INDEX.
           PERFORM WRITE-SOURCE-STMT.
           ADD  1 TO NEST-PTR.
      ******************************************************************
       WRITE-DOC-GROUP-END.
      *  write out group tag
           MOVE SPACES TO SOURCE-STMT.
           MOVE 1 TO J.
           STRING '</'                     DELIMITED BY SIZE
                  NEST-NAME (NEST-INDEX)   DELIMITED BY SPACE
                  '>'                      DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE J TO SOURCE-INDEX.
           SUBTRACT 1 FROM NEST-PTR.
           PERFORM WRITE-SOURCE-STMT.
      ******************************************************************
       WRITE-DOC-FIELD.
      *  write out field tag
           IF VALUE-CHAR (1) = SPACE
              MOVE 1 TO K
              PERFORM UNTIL K > FIELD-LNG
                         OR VALUE-CHAR (K) NOT = SPACE
                 MOVE '.' TO VALUE-CHAR (K)
                 ADD 1 TO K
              END-PERFORM
           END-IF.
           MOVE SPACES TO SOURCE-STMT.
           MOVE 1 TO J.
           STRING '<'                 DELIMITED BY SIZE
                  FIELD-NAME          DELIMITED BY SPACE
                  '>'                 DELIMITED BY SIZE
                  FIELD-VALUE         DELIMITED BY SPACE
                  '</'                DELIMITED BY SIZE
                  FIELD-NAME          DELIMITED BY SPACE
                  '>'                 DELIMITED BY SIZE
              INTO SOURCE-LINE (J)
           END-STRING.
           MOVE J TO SOURCE-INDEX.
           PERFORM WRITE-SOURCE-STMT.
      ******************************************************************
       WRITE-SOURCE-STMT.
      *  write  out XML Statement from Buffer
           MOVE SPACE TO XML-REC
           COMPUTE K = (NEST-PTR * 2) + 1
           MOVE 1 TO J.
           PERFORM UNTIL J > SOURCE-LINE-MAX
                      OR J > SOURCE-INDEX
              MOVE SOURCE-LINE (J) TO XML-REC (K:)
              WRITE XML-REC
              ADD 1 TO J
           END-PERFORM.
TEST***    DISPLAY 'XML-REC =' XML-REC
TEST***    ACCEPT CONTINUE-KEY.
      ******************************************************************
      *    create the internal table
TEST***    COPY MAKETAB.
      *****************************************************************
      * SECTION TO CREATE INTERNAL DATA TABLE                         *
      *****************************************************************
       CREATE-DATA-TAB SECTION.
      *****************************************************************
       OPEN-XSD-SCHEMA.
TEST***    DISPLAY 'Open Schema File '
           MOVE SPACES TO XSD-FILE
WORK       STRING 'WORKDIR\'     DELIMITED BY SIZE
                  XML-FILNAME    DELIMITED BY SPACE
                  '.xsd'         DELIMITED BY SIZE
              INTO XSD-FILE
           END-STRING.
TEST       DISPLAY 'Schema File = ' XSD-FILE
TEST***    ACCEPT CONTINUE-KEY.
           MOVE ZEROES TO XSD-STAT
           OPEN INPUT SCHEMA-FILE.
           IF XSD-STAT NOT = ZEROES
              DISPLAY 'Schema File not found'
              DISPLAY 'Press Enter to return to Caller'
              ACCEPT  CONTINUE-KEY
              MOVE '09' TO XML-RETCODE
              GOBACK
           END-IF.
      *****************************************************
       PROCESS-XSD-SCHEMA.
TEST***    DISPLAY 'read the schema file'.
           MOVE SPACES TO REC-NAME.
           MOVE ZERO TO FELD-NR.
           MOVE ZEROES TO XSD-STAT
      *  scan thru the Schema File
           PERFORM UNTIL FELD-NR > FELD-MAX
                      OR XSD-EOF
              READ SCHEMA-FILE
              AT END
                 MOVE '09' TO XSD-STAT
              NOT AT END
                 MOVE XSD-REC TO XSD-LINE
TEST***          DISPLAY XSD-LINE
TEST***          ACCEPT CONTINUE-KEY
      *  check line for attributes
                 MOVE 1 TO XSD-PTR
                 PERFORM UNTIL XSD-PTR > LINE-MAX
      *  check File
                    IF XSD-LINE (XSD-PTR:5) = '#file'
                       PERFORM PROCESS-FILE
                    END-IF
      *  check Record
                    IF XSD-LINE (XSD-PTR:7) = '#record'
                    OR XSD-LINE (XSD-PTR:7) = '#params'
                    OR XSD-LINE (XSD-PTR:5) = '#view'
                    OR XSD-LINE (XSD-PTR:4) = '#map'
                       PERFORM PROCESS-RECORD
                    END-IF
      *  check Group
                    IF XSD-LINE (XSD-PTR:6) = '#group'
                       PERFORM PROCESS-GROUP
                    END-IF
      *  check Field
                    IF XSD-LINE (XSD-PTR:5) = '#char'
                    OR XSD-LINE (XSD-PTR:4) = '#dec'
                    OR XSD-LINE (XSD-PTR:7) = '#packed'
                    OR XSD-LINE (XSD-PTR:7) = '#binary'
                    OR XSD-LINE (XSD-PTR:6) = '#float'
                    OR XSD-LINE (XSD-PTR:5) = '#edit'
                    OR XSD-LINE (XSD-PTR:4) = '#bit'
                    OR XSD-LINE (XSD-PTR:4) = '#hex'
                    OR XSD-LINE (XSD-PTR:8) = '#pointer'
                       PERFORM PROCESS-FIELD
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
              END-READ
           END-PERFORM.
TEST***    DISPLAY 'The schema file was read'.
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       CLOSE-XSD-SCHEMA.
      *  close out the Schema File
TEST***    DISPLAY 'close the schema file'.
           IF REC-LNG > REC-MAX
              MOVE REC-MAX TO REC-LNG
           END-IF.
           CLOSE SCHEMA-FILE.
TEST***    DISPLAY 'Record Description read for ' REC-NAME
TEST***    DISPLAY 'Nr. of Fields is : '  FELD-NR
TEST***    DISPLAY 'Record Lng    is : '  REC-LNG
TEST***    DISPLAY 'Press Enter to continue !!'
TEST***    ACCEPT  CONTINUE-KEY.
      *****************************************************************
       CREATE-DATA-TAB-ENTRY SECTION.
      *****************************************************************
       PROCESS-FILE.
      *  convert an element of type file to file name
           MOVE SPACES TO FIL-NAME
           ADD 7 TO XSD-PTR
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO FIL-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
              END-IF
           END-IF.
TEST***    DISPLAY 'File Read = ' FIL-NAME
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       PROCESS-RECORD.
      *  convert an element of type record to a record name
           MOVE SPACES TO REC-NAME
           ADD 7 TO XSD-PTR
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO REC-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
              END-IF
           END-IF.
TEST***    DISPLAY 'Record Read = ' REC-NAME
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       PROCESS-GROUP.
      *  convert an element of type group to a field entry
           ADD 1 TO FELD-NR
           MOVE ZEROES TO FELD-LEV  (FELD-NR)
           MOVE SPACES TO FELD-NAME (FELD-NR)
           MOVE SPACES TO FELD-TYP  (FELD-NR)
           MOVE ZEROES TO FELD-LNG  (FELD-NR)
           MOVE ZEROES TO FELD-POS  (FELD-NR)
           MOVE ZEROES TO FELD-DIM  (FELD-NR)
           MOVE SPACES TO FELD-DEF  (FELD-NR).
           ADD 7 TO XSD-PTR
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 MOVE SPACES TO FIELD-NAME
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 MOVE FIELD-NAME TO FELD-NAME (FELD-NR)
              END-IF
           END-IF.
      *************************************************************
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  check level
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 5 TO XSD-PTR
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 IF XSD-LINE (XSD-PTR:1) = '"'
                    ADD 1 TO XSD-PTR
                    MOVE XSD-LINE (XSD-PTR:2) TO FELD-LEV (FELD-NR)
                 END-IF
              END-IF
           END-READ.
      *************************************************************
      *  read the schema line with occurs
      *  read the schema line with redefinition
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  check level
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                 IF XSD-LINE (XSD-PTR:9) = 'maxOccurs'
                    ADD 9 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-DIM-X
                       MOVE FELD-DIM-N TO FELD-DIM (FELD-NR)
                    END-IF
                 END-IF
                 IF XSD-LINE (XSD-PTR:5) = 'redef'
                    ADD 5 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE ZERO TO K
                       MOVE SPACES TO FIELD-NAME
                       PERFORM UNTIL XSD-PTR > LINE-MAX
                                  OR K > NAME-MAX
                                  OR XSD-CHAR (XSD-PTR) = '"'
                          IF XSD-CHAR (XSD-PTR) NOT = SPACE
                             ADD 1 TO K
                             MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                          END-IF
                          ADD 1 TO XSD-PTR
                       END-PERFORM
                       MOVE FIELD-NAME TO FELD-DEF (FELD-NR)
                    END-IF
                    READ SCHEMA-FILE
                    AT END
                       MOVE '09' TO XSD-STAT
                    NOT AT END
                       MOVE XSD-REC TO XSD-LINE
TEST***                DISPLAY XSD-LINE
TEST***                ACCEPT CONTINUE-KEY
                       MOVE 1 TO XSD-PTR
                    END-READ
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
TEST***    DISPLAY 'Group Read = ' FELD-NAME (FELD-NR)
TEST***    DISPLAY 'Group Level= ' FELD-LEV  (FELD-NR)
TEST***    DISPLAY 'Group Occurs=' FELD-DIM  (FELD-NR)
TEST***    DISPLAY 'Group Redef =' FELD-DEF  (FELD-NR)
TEST***    ACCEPT CONTINUE-KEY.
      *************************************************************
       PROCESS-FIELD.
      *  convert an element of type field to a field entry
      *  initialize table entry
           ADD 1 TO FELD-NR
           MOVE ZEROES TO FELD-LEV  (FELD-NR)
           MOVE SPACES TO FELD-NAME (FELD-NR)
           MOVE SPACES TO FELD-TYP  (FELD-NR)
           MOVE ZEROES TO FELD-LNG  (FELD-NR)
           MOVE ZEROES TO FELD-POS  (FELD-NR)
           MOVE ZEROES TO FELD-DIM  (FELD-NR)
           MOVE SPACES TO FELD-DEF  (FELD-NR).
      *  assign the field type
           IF XSD-LINE (XSD-PTR:5) = '#char'
              MOVE 'X' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:4) = '#dec'
              MOVE 'D' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:7) = '#packed'
              MOVE 'P' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:7) = '#binary'
              MOVE 'B' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:6) = '#float'
              MOVE 'F' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:5) = '#edit'
              MOVE 'X' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:4) = '#bit'
              MOVE 'L' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:4) = '#hex'
              MOVE 'H' TO FELD-TYP  (FELD-NR)
           END-IF
           IF XSD-LINE (XSD-PTR:8) = '#pointer'
              MOVE 'T' TO FELD-TYP  (FELD-NR)
           END-IF.
      *  assign the field name
           PERFORM UNTIL XSD-PTR > LINE-MAX
                      OR XSD-LINE (XSD-PTR:4) = 'name'
              ADD 1 TO XSD-PTR
           END-PERFORM.
           IF XSD-LINE (XSD-PTR:4) = 'name'
              ADD 5 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-CHAR (XSD-PTR) = '"'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:1) = '"'
                 ADD 1 TO XSD-PTR
                 MOVE ZERO TO K
                 MOVE SPACES TO FIELD-NAME
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR K > NAME-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    IF XSD-CHAR (XSD-PTR) NOT = SPACE
                       ADD 1 TO K
                       MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                    END-IF
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 MOVE FIELD-NAME TO FELD-NAME (FELD-NR)
              END-IF
           END-IF.
      *************************************************************
      *  read the schema line with the level
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the field level
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                         OR XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 1 TO XSD-PTR
              END-PERFORM
              IF XSD-LINE (XSD-PTR:5) = 'level'
                 ADD 5 TO XSD-PTR
                 PERFORM UNTIL XSD-PTR > LINE-MAX
                            OR XSD-CHAR (XSD-PTR) = '"'
                    ADD 1 TO XSD-PTR
                 END-PERFORM
                 IF XSD-LINE (XSD-PTR:1) = '"'
                    ADD 1 TO XSD-PTR
                    MOVE XSD-LINE (XSD-PTR:2) TO FELD-LEV (FELD-NR)
                 END-IF
              END-IF
           END-READ.
      *************************************************************
      *  read the schema line with occurance
      *  read the schema line with redefinition
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the field occurs
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
                 IF XSD-LINE (XSD-PTR:9) = 'maxOccurs'
                    ADD 9 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-DIM-X
                       MOVE FELD-DIM-N TO FELD-DIM (FELD-NR)
                    END-IF
                 END-IF
                 IF XSD-LINE (XSD-PTR:5) = 'redef'
                    ADD 5 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE ZERO TO K
                       MOVE SPACES TO FIELD-NAME
                       PERFORM UNTIL XSD-PTR > LINE-MAX
                                  OR K > NAME-MAX
                                  OR XSD-CHAR (XSD-PTR) = '"'
                          IF XSD-CHAR (XSD-PTR) NOT = SPACE
                             ADD 1 TO K
                             MOVE XSD-CHAR (XSD-PTR) TO FIELD-NAME (K:1)
                          END-IF
                          ADD 1 TO XSD-PTR
                       END-PERFORM
                       MOVE FIELD-NAME TO FELD-DEF (FELD-NR)
                    END-IF
                    READ SCHEMA-FILE
                    AT END
                       MOVE '09' TO XSD-STAT
                    NOT AT END
                       MOVE XSD-REC TO XSD-LINE
TEST***                DISPLAY XSD-LINE
TEST***                ACCEPT CONTINUE-KEY
                       MOVE 1 TO XSD-PTR
                    END-READ
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
      *************************************************************
      *  read the schema line with position and length
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the field redefinition if there is one
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
      *  assign the field position
                 IF XSD-LINE (XSD-PTR:3) = 'pos'
                    ADD 3 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-POS-X
                       MOVE FELD-POS-N TO FELD-POS (FELD-NR)
                    END-IF
                 END-IF
      *  assign the field position
                 IF XSD-LINE (XSD-PTR:3) = 'lng'
                    ADD 3 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE XSD-LINE (XSD-PTR:4) TO FELD-LNG-X
                       MOVE FELD-LNG-N TO FELD-LNG (FELD-NR)
                    END-IF
      *************************************************************
      * change type binary to integer if length = 2
                    IF FELD-TYP (FELD-NR) = 'B'
                       IF FELD-LNG (FELD-NR) = 2
                          MOVE 'I' TO FELD-TYP (FELD-NR)
                       END-IF
                    END-IF
      *************************************************************
      * calculate record length after each field     **************
                    COMPUTE REC-LNG =  FELD-POS-N
                                    + (FELD-LNG-N * FELD-DIM-N)
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
      *************************************************************
      *  read the schema line with picture and usage
           READ SCHEMA-FILE
           AT END
              MOVE '09' TO XSD-STAT
           NOT AT END
              MOVE XSD-REC TO XSD-LINE
TEST***       DISPLAY XSD-LINE
TEST***       ACCEPT CONTINUE-KEY
      *  assign the picture
              MOVE 1 TO XSD-PTR
              PERFORM UNTIL XSD-PTR > LINE-MAX
      *  assign the field position
                 IF XSD-LINE (XSD-PTR:3) = 'pic'
                    ADD 3 TO XSD-PTR
                    PERFORM UNTIL XSD-PTR > LINE-MAX
                               OR XSD-CHAR (XSD-PTR) = '"'
                       ADD 1 TO XSD-PTR
                    END-PERFORM
                    IF XSD-LINE (XSD-PTR:1) = '"'
                       ADD 1 TO XSD-PTR
                       MOVE ZERO TO K
                       MOVE SPACES TO FIELD-PICT
                       PERFORM UNTIL XSD-PTR > LINE-MAX
                                  OR K > NAME-MAX
                                  OR XSD-CHAR (XSD-PTR) = '"'
                          IF XSD-CHAR (XSD-PTR) NOT = SPACE
                             ADD 1 TO K
                             MOVE XSD-CHAR (XSD-PTR) TO FIELD-PICT (K:1)
                          END-IF
                          ADD 1 TO XSD-PTR
                       END-PERFORM
                       MOVE FIELD-PICT TO FELD-PIC (FELD-NR)
                    END-IF
                 END-IF
                 ADD 1 TO XSD-PTR
              END-PERFORM
           END-READ.
TEST***    DISPLAY 'Field Name = ' FELD-NAME (FELD-NR)
TEST***    DISPLAY 'Field Level= ' FELD-LEV  (FELD-NR)
TEST***    DISPLAY 'Field Occur =' FELD-DIM  (FELD-NR)
TEST***    DISPLAY 'Field Redef =' FELD-DEF  (FELD-NR)
TEST***    DISPLAY 'Field Type  =' FELD-TYP  (FELD-NR)
TEST***    DISPLAY 'Field Pos   =' FELD-POS  (FELD-NR)
TEST***    DISPLAY 'Field Lng   =' FELD-LNG  (FELD-NR)
TEST***    DISPLAY 'Field Pic   =' FELD-PIC  (FELD-NR)
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      ***************************************************************
      *
      ********************** end of source *****************************
