       IDENTIFICATION DIVISION.
       PROGRAM-ID. XPLIMESS.
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      ***************************************************************
      *
      *     Indexed File with one record per module tested
      *
           SELECT COBMESS-FILE ASSIGN TO 'XCOBMESS.dat'
                  ORGANIZATION IS INDEXED
                  ACCESS IS RANDOM
                  RECORD KEY IS COBMESS-KEY
                  FILE STATUS IS COBMESS-STAT.
      *
      *     Output File for Coverage Report
      *
           SELECT COBMESS-REPO ASSIGN TO DYNAMIC TEST-REPORT-FILE
                  ACCESS IS SEQUENTIAL
                  ORGANIZATION IS LINE SEQUENTIAL
                  FILE STATUS IS REPORT-STAT.
      *
           SELECT EXPORT-FILE
           ASSIGN TO DYNAMIC TEST-EXPORT-FILE
           ORGANIZATION IS LINE SEQUENTIAL
           ACCESS IS SEQUENTIAL.
      *
      ******************************************************************
      *
       DATA DIVISION.
      *
       FILE SECTION.
      *
      ******************************************************************
      *
       FD  COBMESS-FILE
           RECORD CONTAINS 4012 CHARACTERS
           DATA RECORD IS COBMESS-REC.
      *
       01  COBMESS-REC.
           05  COBMESS-KEY              PIC X(8).
           05  COBMESS-NODE-NR          PIC 9999.
           05  COBMESS-NODES            PIC X(4000).
           05  COBMESS-NODE-TAB         REDEFINES COBMESS-NODES.
               10  COBMESS-NODE         PICTURE 9(8) BINARY
                                        OCCURS 1000 TIMES.
      *
       FD  COBMESS-REPO
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS REPORT-REC.
      *
       01  REPORT-REC                   PIC X(72).
      *
       FD  EXPORT-FILE
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS EXPORT-REC.
       01  EXPORT-REC                        PIC X(72).
      *
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *
       77  CONTINUE-KEY                 PIC X.
       77  TEST-EXPORT-FILE             PIC X(50).
       77  TEST-REPORT-FILE             PIC X(50).
       01  FILE-STATUS-VARS.
           05 COBMESS-STAT              PIC XX.
              88 COBMESS-OK             VALUE '00'.
           05 REPORT-STAT               PIC XX.
              88 REPORT-OK              VALUE '00'.
      *
KATI   77  UPDATE-FLAG                  PIC 9.
           88 INSERTION                 VALUE 1.
           88 MUTATION                  VALUE 0.
      *
       01  CONSTANTS.
           05 LINE-MAX                  PIC 9999 VALUE 72.
           05 NODE-MAX                  PIC 9999 VALUE 1000.
      *
       01  VARIABLES.
           05 LINE-NR                   PIC 9999.
           05 NODE-NR                   PIC 9999.
           05 NR-HITS                   PIC 9999.
           05 NR-MISSES                 PIC 9999.
           05 HIT-RATIO                 PIC 99V99.
           05 NODE-SUM                  PIC 99999 BINARY.
           05 OPEN-SW                   PIC 9    VALUE ZERO.
           05 RET-CODE                  PIC 99   VALUE ZERO.
           05  TODAYS-DATE                   PIC S9(6).
           05  CURRENT-DATUM                 REDEFINES TODAYS-DATE.
               10  CURRENT-YEAR              PIC 99.
               10  CURRENT-MONTH             PIC 99.
               10  CURRENT-DAY               PIC 99.
      *
       01  INDEXES.
           05 I                              PIC 9(4) BINARY.
           05 J                              PIC 9(4) BINARY.
           05 K                              PIC 9(4) BINARY.
      *
      ****************************************************************
       01  COVERAGE-PRINT-LINES.
           05  HYPHEN-LINE.
               10 FILLER                     PIC X     VALUE '+'.
               10 FILLER                     PIC X(55) VALUE ALL '-'.
               10 FILLER                     PIC X     VALUE '+'.
           05  BLANK-LINE.
               10 FILLER                     PIC X     VALUE '!'.
               10 FILLER                     PIC X(55) VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  TEXT-LINE.
               10 FILLER                     PIC X     VALUE '!'.
               10 FILLER                     PIC X(5)  VALUE SPACES.
               10 FILLER                     PIC X(45) VALUE
                  '  T E S T   C O V E R A G E  R E P O R T    '.
               10 FILLER                     PIC X(5)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  DATE-LINE.
               10 FILLER                     PIC X     VALUE '!'.
               10 FILLER                     PIC X(9)  VALUE
                  ' DATE: '.
               10 DATUM.
                  20  TAG                    PIC 99.
                  20  FILLER                 PIC X  VALUE '.'.
                  20  MONAT                  PIC 99.
                  20  FILLER                 PIC X  VALUE '.'.
                  20  JAHR                   PIC 99.
               10 FILLER                     PIC X(25) VALUE SPACES.
               10 FILLER                     PIC X(6)  VALUE 'PAGE: '.
               10 SEITE                      PIC 99    VALUE ZERO.
               10 FILLER                     PIC X(5)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  OBJECT-LINE.
               10 FILLER                     PIC X     VALUE '!'.
               10 FILLER                     PIC X(9)  VALUE
                  ' MODULE: '.
               10 MODULE-NAME                PIC X(12).
               10 FILLER                     PIC X(34) VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  TITLE-LINE.
               10 FILLER      PIC X(14) VALUE '!     Node-Nr.'.
               10 FILLER      PIC X(14) VALUE '     Past Test'.
               10 FILLER      PIC X(14) VALUE '     This Test'.
               10 FILLER      PIC X(14) VALUE '      Missed  '.
               10 FILLER      PIC X     VALUE '!'.
           05  DATA-LINE.
               10 FILLER                     PIC X(5)  VALUE '!    '.
               10 NODE-OUT                   PIC ZZZZ9 VALUE ZEROS.
               10 FILLER                     PIC X(10) VALUE SPACES.
               10 PAST-OUT                   PIC ZZZZ9 VALUE ZEROS.
               10 FILLER                     PIC X(10) VALUE SPACES.
               10 THIS-OUT                   PIC ZZZZ9 VALUE ZEROS.
               10 FILLER                     PIC X(10) VALUE SPACES.
               10 MISSED-OUT                 PIC X     VALUE '*'.
               10 FILLER                     PIC X(5)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  END-LINE-1.
               10 FILLER                     PIC X(5)  VALUE '!    '.
               10 FILLER                     PIC X(40) VALUE
                 'Total Number of Test Probes inserted =  '.
               10 TOTAL-PROBES               PIC ZZZZ9 VALUE ZERO.
               10 FILLER                     PIC X(6)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  END-LINE-2.
               10 FILLER                     PIC X(5)  VALUE '!    '.
               10 FILLER                     PIC X(40) VALUE
                 'Total Number of Test Probes executed =  '.
               10 TOTAL-HITS                 PIC ZZZZ9 VALUE ZERO.
               10 FILLER                     PIC X(6)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  END-LINE-3.
               10 FILLER                     PIC X(5)  VALUE '!    '.
               10 FILLER                     PIC X(40) VALUE
                 'Total Number of Test Probes missed   =  '.
               10 TOTAL-MISSES               PIC ZZZZ9 VALUE ZERO.
               10 FILLER                     PIC X(6)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
           05  END-LINE-4.
               10 FILLER                     PIC X(5)  VALUE '!    '.
               10 FILLER                     PIC X(40) VALUE
                 'Test Coverage Ratio for this module  =  '.
               10 TEST-RATIO                 PIC Z9.99 VALUE ZERO.
               10 FILLER                     PIC X(6)  VALUE SPACES.
               10 FILLER                     PIC X     VALUE '!'.
      *
      ****************************************************************
      *
       01  METRIK-AUSGABE.
           05  ZEILE-1.
               07  TEXT-1           PIC X(12) VALUE "/FILENAME = ".
               07  ZUWEISUNG-1      PIC X(50).
           05  ZEILE-2.
               07  TEXT-2           PIC X(12) VALUE "/USERID   = ".
               07  ZUWEISUNG-2      PIC X(50).
           05  ZEILE-3.
               07  TEXT-3           PIC X(12) VALUE "/DATE     = ".
               07  ZUWEISUNG-3      PIC X(50).
           05  ZEILE-4.
               07  TEXT-4           PIC X(12) VALUE "/FROM     = ".
               07  ZUWEISUNG-4      PIC X(50) VALUE "COBTEST".
           05  ZEILE-5.
               07  TEXT-5           PIC X(12) VALUE "/TO       = ".
               07  ZUWEISUNG-5      PIC X(50) VALUE "PC-MESS".
           05  ZEILE-6.
               07  TEXT-6           PIC X(12) VALUE "//ENTITY = ".
               07  ZUWEISUNG-6      PIC X(50) VALUE "TEST; ".
           05  ZEILE-7.
               07  TEXT-7           PIC X(12) VALUE "  ENAME  = ".
               07  ZUWEISUNG-7.
                   10 ZUW-71        PIC X(12).
           05  ZEILE-8.
               07  TEXT-8           PIC X(12) VALUE "  EDATE  = ".
               07  ZUWEISUNG-8.
                   10 ZUW-81        PIC X(8).
                   10 ZUW-82        PIC X VALUE ";".
           05  ZEILE-9.
               07  TEXT-9           PIC X(12) VALUE "  ETYPE  = ".
               07  ZUWEISUNG-9.
                   10 ZUW-91        PIC X(8).
           05  ZEILE-11.
               07  TEXT-11          PIC X(11) VALUE "  ATTR = ( ".
               07  ZUWEISUNG-11.
                   15  Z-SPALTE1-1   PIC X(8) VALUE "NODES   ".
                   15  Z-SPALTE1-2   PIC X VALUE "=".
                   15  Z-SPALTE1-3   PIC ZZZZZ999.
                   15  Z-SPALTE1-4   PIC X(3) VALUE ",  ".
                   15  Z-SPALTE1-5   PIC X(8) VALUE "HITS    ".
                   15  Z-SPALTE1-6   PIC X VALUE "=".
                   15  Z-SPALTE1-7   PIC ZZZZZ999.
                   15  Z-SPALTE1-8   PIC X(3) VALUE ",  ".
                   15  Z-SPALTE1-9   PIC X(8) VALUE "RATIO   ".
                   15  Z-SPALTE1-10  PIC X VALUE "=".
                   15  Z-SPALTE1-11  PIC ZZ9.99.
                   15  Z-SPALTE4-11  PIC X(3) VALUE " );".
      ******************************************************************
      *
       LINKAGE SECTION.
      *
      ******************************************************************
      *
           COPY XPLIDATA.
      *
      ******************************************************************
      *
       PROCEDURE DIVISION USING XCOBINST-DATA.
      *
      ******************************************************************
      *
       COBMESS-MAIN SECTION.
       COBMESS-CONTROL.
           PERFORM COBMESS-OPEN.
           IF XMESS-READ
              PERFORM COBMESS-READ
           END-IF.
           IF XMESS-WRITE
              PERFORM COBMESS-WRITE
              PERFORM COBMESS-EXPORT
           END-IF.
           CLOSE COBMESS-FILE.
           EXIT PROGRAM.
      *
      ******************************************************************
       COBMESS-SUB SECTION.
      ******************************************************************
      *
       COBMESS-OPEN.
           MOVE ZEROES TO COBMESS-STAT
           OPEN I-O COBMESS-FILE
           IF NOT COBMESS-OK
ERROR         DISPLAY 'XCOBMESS.dat File could not be found!'
ERROR         DISPLAY 'Test is terminated !!!! '
ERROR         STOP RUN
           END-IF.
      *
      ************************************************************
      *
       COBMESS-READ.
           MOVE XTEST-OBJECT TO COBMESS-KEY
KATI       MOVE ZERO TO UPDATE-FLAG
           READ COBMESS-FILE
              INVALID KEY
                 MOVE LOW-VALUE TO COBMESS-NODES
                 MOVE ZERO TO COBMESS-NODE-NR
KATI             MOVE 1 TO UPDATE-FLAG
KATI             DISPLAY 'Coverage Record missing for ' XTEST-OBJECT
KATI             DISPLAY 'It will be created! '
           END-READ.
           MOVE XNODE-NR TO COBMESS-NODE-NR
KATI       IF INSERTION
HARRY        MOVE LOW-VALUE TO COBMESS-NODES
KATI         WRITE COBMESS-REC
KATI           INVALID KEY
KATI             DISPLAY 'Key Error in inserting ' XTEST-OBJECT
KATI         END-WRITE
KATI       ELSE
KATI         REWRITE COBMESS-REC
KATI           INVALID KEY
KATI             DISPLAY 'Key Error in rewriting ' XTEST-OBJECT
KATI         END-REWRITE
KATI       END-IF.
      *
      ************************************************************
      *
       COBMESS-WRITE.
           MOVE XTEST-OBJECT TO COBMESS-KEY
KATI       MOVE ZERO TO UPDATE-FLAG
           READ COBMESS-FILE
              INVALID KEY
                 MOVE LOW-VALUE TO COBMESS-NODES
                 MOVE ZERO TO COBMESS-NODE-NR
KATI             MOVE 1 TO UPDATE-FLAG
KATI             DISPLAY 'Coverage Record missing for ' XTEST-OBJECT
KATI             DISPLAY 'It will be created! '
           END-READ.
      *
      *    Write Current Coverage Report
      *
           PERFORM COBMESS-REPORT.
      *
      *    Store Current Coverage Values
      *
           MOVE 1 TO I.
           PERFORM UNTIL I > XNODE-NR OR I > NODE-MAX
              MOVE XNODE (I) TO COBMESS-NODE (I)
              ADD 1 TO I
           END-PERFORM.
           MOVE XNODE-NR     TO COBMESS-NODE-NR.
      *
KATI       IF INSERTION
KATI         WRITE COBMESS-REC
KATI           INVALID KEY
KATI             DISPLAY 'Key Error in inserting ' XTEST-OBJECT
KATI         END-WRITE
KATI       ELSE
KATI         REWRITE COBMESS-REC
KATI           INVALID KEY
KATI             DISPLAY 'Key Error in rewriting ' XTEST-OBJECT
KATI         END-REWRITE
KATI       END-IF.
      *
      ************************************************************
      *
       COBMESS-REPORT.
      *
           MOVE SPACES TO TEST-REPORT-FILE
           STRING XTEST-OBJECT  DELIMITED BY SPACE
                 '.out'         DELIMITED BY SIZE
             INTO TEST-REPORT-FILE
           END-STRING.
           OPEN OUTPUT COBMESS-REPO.
      *
           ACCEPT TODAYS-DATE FROM DATE.
           MOVE CURRENT-YEAR TO JAHR.
           MOVE CURRENT-MONTH TO MONAT.
           MOVE CURRENT-DAY  TO TAG.
           MOVE XTEST-OBJECT TO MODULE-NAME.
           PERFORM WRITE-PAGE-HEADER.
      *
           MOVE ZERO TO NR-HITS.
           MOVE ZERO TO NR-MISSES.
           MOVE 1 TO I.
           PERFORM UNTIL I > XNODE-NR OR I > NODE-MAX
              MOVE I TO NODE-OUT
              MOVE COBMESS-NODE (I) TO PAST-OUT
              MOVE XNODE (I) TO THIS-OUT
              COMPUTE NODE-SUM = COBMESS-NODE (I) +  XNODE (I)
              IF NODE-SUM = ZERO
                 MOVE '*' TO MISSED-OUT
                 ADD 1 TO NR-MISSES
              ELSE
                 MOVE ' ' TO MISSED-OUT
                 ADD 1 TO NR-HITS
              END-IF
              MOVE DATA-LINE TO REPORT-REC
              WRITE REPORT-REC
              ADD 1 TO LINE-NR
              IF LINE-NR > LINE-MAX
                 PERFORM WRITE-PAGE-HEADER
              END-IF
              ADD 1 TO I
           END-PERFORM.
           MOVE BLANK-LINE TO REPORT-REC
           WRITE REPORT-REC
           MOVE HYPHEN-LINE TO REPORT-REC
           WRITE REPORT-REC
      *
      *    Compute Test Coverage Measures
      *
           MOVE XNODE-NR TO TOTAL-PROBES
           MOVE NR-HITS  TO TOTAL-HITS
           MOVE NR-MISSES TO TOTAL-MISSES
           COMPUTE HIT-RATIO = NR-HITS /  XNODE-NR
           MOVE HIT-RATIO TO TEST-RATIO.
      *
      *    Print out Test Coverage Measures
      *
           ADD 5 TO LINE-NR
           IF LINE-NR > LINE-MAX
              PERFORM WRITE-PAGE-HEADER
           END-IF.
           MOVE END-LINE-1 TO REPORT-REC
           WRITE REPORT-REC
           MOVE END-LINE-2 TO REPORT-REC
           WRITE REPORT-REC
           MOVE END-LINE-3 TO REPORT-REC
           WRITE REPORT-REC
           MOVE END-LINE-4 TO REPORT-REC
           WRITE REPORT-REC
           MOVE HYPHEN-LINE TO REPORT-REC
           WRITE REPORT-REC.
      *
           CLOSE COBMESS-REPO.
      *
      ************************************************************
      *
       WRITE-PAGE-HEADER.
           ADD 1 TO SEITE.
           IF SEITE > 1
              MOVE HYPHEN-LINE TO REPORT-REC
              WRITE REPORT-REC
           END-IF
           MOVE '/' TO REPORT-REC.
           WRITE REPORT-REC
           MOVE HYPHEN-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE BLANK-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE TEXT-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE BLANK-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE DATE-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE OBJECT-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE BLANK-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE TITLE-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE HYPHEN-LINE TO REPORT-REC.
           WRITE REPORT-REC.
           MOVE 10 TO LINE-NR.
      *
      **************************************************************
       COBMESS-EXPORT SECTION.
      ******************************************************************
      *      IO Routines for Export File        ************************
      ******************************************************************
      *
       OPEN-METRIC-FILE.
           MOVE SPACES TO TEST-EXPORT-FILE.
           STRING XTEST-OBJECT  DELIMITED BY SPACE
                 '.pcm'         DELIMITED BY SIZE
              INTO TEST-EXPORT-FILE
           END-STRING.
           OPEN OUTPUT EXPORT-FILE.
      *
       WRITE-METRIC-FILE-HEADER.
           MOVE SPACES TO ZUWEISUNG-1
           STRING 'TEST\'      DELIMITED BY SIZE
                  XTEST-OBJECT DELIMITED BY SPACE
                  '.pcm'       DELIMITED BY SIZE
              INTO ZUWEISUNG-1
           END-STRING.
           MOVE ZEILE-1 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE 'COBINST'  TO ZUWEISUNG-2
           MOVE ZEILE-2 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE DATUM       TO ZUWEISUNG-3
           MOVE ZEILE-3 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE ZEILE-4 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE ZEILE-5 TO EXPORT-REC
           WRITE EXPORT-REC.
       WRITE-METRIC-FILE-ENTRY.
           MOVE SPACE TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE ZEILE-6 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE SPACES TO ZUW-71
           STRING XTEST-OBJECT DELIMITED BY SPACE
                 ';'           DELIMITED BY SIZE
             INTO ZUW-71.
           MOVE ZEILE-7 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE DATUM TO ZUW-81
           MOVE ZEILE-8 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE 'COBOL;' TO ZUW-91
           MOVE ZEILE-9 TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE SPACE TO EXPORT-REC
           WRITE EXPORT-REC.
           MOVE XNODE-NR TO Z-SPALTE1-3.
           MOVE NR-HITS  TO Z-SPALTE1-7.
           MOVE HIT-RATIO TO Z-SPALTE1-11.
           MOVE ZEILE-11 TO EXPORT-REC
           WRITE EXPORT-REC.
      *
       CLOSE-METRIC-FILE.
           CLOSE EXPORT-FILE.
      ******************************************************************
      *               End  of   Source
      ******************************************************************
